module average_sparse_mod

  implicit none

  interface average_sparse_1_op
    module procedure average_sparse_1_op_1d_real
    module procedure average_sparse_1_op_2d_real
    module procedure average_sparse_1_op_3d_real
  end interface

  interface average_sparse_2_op
    module procedure average_sparse_2_op_2d_real
    module procedure average_sparse_2_op_3d_real
  end interface

  interface average_sparse_3_op
    module procedure average_sparse_3_op_3d_real
  end interface

  interface average_sparse_12_op
    module procedure average_sparse_12_op_2d_real
    module procedure average_sparse_12_op_3d_real
  end interface

  interface average_sparse_13_op
    module procedure average_sparse_13_op_3d_real
  end interface

  interface average_sparse_23_op
    module procedure average_sparse_23_op_3d_real
  end interface
  
  
  contains

  subroutine average_sparse_init()
    call average_sparse_close()
  end subroutine

  subroutine average_sparse_close()
  end subroutine

  subroutine average_sparse_1_op_1d_real(data_in   ,data_out)
    real,intent(in)       :: data_in(:)
    real,intent(out)      :: data_out
    integer               :: n1,fold,ii
    real(8)               :: dummy

    fold = 0
    n1 = size(data_in,1)
    do ii=1,n1
      if (data_in(ii).ne.0.) then
      dummy  = dummy + dble(data_in(ii))
      fold   = fold  + 1
      end if
    end do
    if (fold.ne.0.) then
      data_out = real( dummy/dble(fold) )
    end if

  end subroutine

  subroutine average_sparse_1_op_2d_real(data_in     ,data_out)
    real,intent(in)       :: data_in(:,:)
    real,intent(out)      :: data_out(:)
    integer               :: n2,jj

    n2 = size(data_in,2)
    do jj=1,n2
      call average_sparse_1_op_1d_real( data_in(:,jj) , data_out(jj) )
    end do

  end subroutine

  subroutine average_sparse_1_op_3d_real(data_in       ,data_out)
    real,intent(in)       :: data_in(:,:,:)
    real,intent(out)      :: data_out(:,:)
    integer               :: n2,jj,n3,kk
    
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    do kk=1,n3
    do jj=1,n2
      call average_sparse_1_op_1d_real( data_in(:,jj,kk) , data_out(jj,kk) )
    end do
    end do
    
  end subroutine










  subroutine average_sparse_2_op_2d_real(data_in     ,data_out)
    real,intent(in)       :: data_in(:,:)
    real,intent(out)      :: data_out(:)
    integer               :: n1,n2,jj

    n1 = size(data_in,1)
    do jj=1,n1
      call average_sparse_1_op_1d_real( data_in(jj,:) , data_out(jj) )
    end do

  end subroutine

  subroutine average_sparse_2_op_3d_real(data_in     ,data_out)
    real,intent(in)       :: data_in(:,:,:)
    real,intent(out)      :: data_out(:,:)
    integer               :: n1,ii,n2,jj,n3,kk

    n1 = size(data_in,1)
    n3 = size(data_in,3)
    do jj=1,n1
    do kk=1,n3
      call average_sparse_1_op_1d_real( data_in(jj,:,kk) , data_out(jj,kk) )
    end do
    end do

  end subroutine



  subroutine average_sparse_3_op_3d_real(data_in       ,data_out)
    real,intent(in)       :: data_in(:,:,:)
    real,intent(out)      :: data_out(:,:)
    integer               :: n1,ii,n2,jj,n3,kk
    
    n1 = size(data_in,1)
    n2 = size(data_in,2)
    do jj=1,n2
    do ii=1,n1
      call average_sparse_1_op_1d_real( data_in(ii,jj,:) , data_out(ii,jj) )
    end do
    end do
    
  end subroutine


  subroutine average_sparse_12_op_2d_real(data_in     ,data_out)
    real,intent(in)       :: data_in(:,:)
    real,intent(out)      :: data_out
    real(8)               :: dummy
    integer               :: n1,ii,n2,jj,fold

    n1 = size(data_in,1)
    n2 = size(data_in,2)

    fold  = 0
    dummy = 0.
    do ii=1,n1
    do jj=1,n2
      if (data_in(ii,jj).ne.0.) then
        dummy = dummy + data_in(ii,jj)
        fold  = fold  + 1
      end if
    end do
    end do
    if (fold.ne.0.) then
      data_out = real( dummy/dble(fold) )
    end if
    
  end subroutine

  subroutine average_sparse_12_op_3d_real(data_in     ,data_out)
    real,intent(in)       :: data_in(:,:,:)
    real,intent(out)      :: data_out(:)
    real(8),allocatable   :: dummy(:)
    integer               :: n1,ii,n2,jj,n3,kk,fold
    
    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    allocate(dummy(n3))

    fold  = 0
    dummy = 0.
    do ii=1,n1
    do jj=1,n2
      if (sum(abs(data_in(ii,jj,:))).ne.0.) then
        dummy = dummy + data_in(ii,jj,:)
        fold  = fold  + 1
      end if
    end do
    end do
    if (fold.ne.0.) then
      data_out = real( dummy/dble(fold) )
    end if
    deallocate(dummy)

  end subroutine


  subroutine average_sparse_13_op_3d_real(data_in     ,data_out)
    real,intent(in)       :: data_in(:,:,:)
    real,intent(out)      :: data_out(:)
    real(8),allocatable   :: dummy(:)
    integer               :: n1,ii,n2,jj,n3,kk,fold
    
    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    allocate(dummy(n2))

    fold  = 0
    dummy = 0.
    do ii=1,n1
    do kk=1,n3
      if (sum(abs(data_in(ii,:,kk))).ne.0.) then
        dummy = dummy + data_in(ii,:,kk)
        fold  = fold  + 1
      end if
    end do
    end do
    if (fold.ne.0.) then
      data_out = real( dummy/dble(fold) )
    end if
    deallocate(dummy)

  end subroutine
  
  
  
  subroutine average_sparse_23_op_3d_real(data_in     ,data_out)
    real,intent(in)       :: data_in(:,:,:)
    real,intent(out)      :: data_out(:)
    real(8),allocatable   :: dummy(:)
    integer               :: n1,ii,n2,jj,n3,kk,fold
    
    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    allocate(dummy(n1))

    fold  = 0
    dummy = 0.
    do ii=1,n1
    do kk=1,n3
      if (sum(abs(data_in(:,jj,kk))).ne.0.) then
        dummy = dummy + data_in(:,jj,kk)
        fold  = fold  + 1
      end if
    end do
    end do
    if (fold.ne.0.) then
      data_out = real( dummy/dble(fold) )
    end if
    deallocate(dummy)

  end subroutine

end module
