module compression_rwr_mod
 
 !use sep
 use bitcompression_mod
 !use bitprinting_mod
 
 implicit none

 integer, private              :: n1, nc, pc, bs, ii
 real   , private, allocatable :: data_scl(:)
 integer, private, allocatable :: data_com(:), data_int(:)
 
 interface compression_rite
   module procedure compression_rite1
   module procedure compression_rite2
   module procedure compression_rite3
 end interface
 
 interface compression_reed
   module procedure compression_reed1
   module procedure compression_reed2
   module procedure compression_reed3
 end interface
 
 contains
  
  subroutine compression_rwr_init(n1_in, pc_in, bs_in )
    integer, optional          ::        pc_in, bs_in
    integer                    :: n1_in
    
    n1 = n1_in
    if (present(pc_in)) then; pc = pc_in; else; pc = 32          ; end if;
    if (present(bs_in)) then; bs = bs_in; else; bs = bit_size(pc); end if;
    nc = n1/bs*pc
    
    call compression_init(pc, bs)
    
    allocate( data_int(n1) , data_scl(n1) , data_com(nc) )
    
  end subroutine
  
  
  subroutine compression_rwr_close( )
    
    call compression_close( )
    deallocate( data_int , data_com , data_scl )
    
  end subroutine
  
  
  subroutine compression_rite1(data)
    real,intent(inout)  ::     data(:)
    real                ::     scale
    
    ! Initialize scaling function
    call compression_scale_init(scale,data)

    ! Scale and Compress
    call compression_scale_fop(.false.,.false.,data    ,data_scl)
    call compression_r2i      (.false.,.false.,data_scl,data_int)
    call compression          (.false.,.false.,data_int,data_com)

    ! Write to Disk
    call srite( "out", scale   , 4*1  )
    call srite( "out", data_com, 4*nc )
    
  end subroutine
  
  
  subroutine compression_rite2(data)
    real,intent(inout)  ::     data(:,:)
    integer             ::     jj
    
    do jj=1,size(data,2)
      call compression_rite1(data(:,jj))
    end do
    
  end subroutine


  subroutine compression_rite3(data)
    real,intent(inout)  ::     data(:,:,:)
    integer             ::     jj
    integer             ::     kk
    
    do kk=1,size(data,3)
      do jj=1,size(data,2)
        call compression_rite1(data(:,jj,kk))
      end do
    end do
    
  end subroutine


  subroutine compression_reed1(data)
    real,intent(inout)  ::     data(:)
    real                ::     scale

    ! read from Disk
    call sreed( "in", scale   , 4*1  )
    call sreed( "in", data_com, 4*nc )

    ! Initialize scaling function
    call compression_scale_init(scale)

    ! Expand and Unscale
    call compression          (.true. ,.false.,data_int,data_com)
    call compression_r2i      (.true. ,.false.,data_scl,data_int)
    call compression_scale_fop(.true. ,.false.,data    ,data_scl)

  end subroutine


  subroutine compression_reed2(data)
    real,intent(inout)  ::     data(:,:)
    integer             ::     jj
    
    do jj=1,size(data,2)
      call compression_reed1(data(:,jj))
    end do
     
  end subroutine
  
  
  subroutine compression_reed3(data)
    real,intent(inout)  ::     data(:,:,:)
    integer             ::     jj
    integer             ::     kk
    
    do kk=1,size(data,3)
      do jj=1,size(data,2)
        call compression_reed1(data(:,jj,kk))
      end do
    end do

  end subroutine


end module

