module demean_mod
  implicit none

  integer,private                  :: n_start,n_end,n_totat

  integer         , private        :: n1,n2,n3
  double precision, private        :: mean
  integer         , private        :: ii,jj,kk

  interface demean1d_op
    module procedure demean1d_op1d
    module procedure demean1d_op2d
    module procedure demean1d_op3d
  end interface

  interface demean2d_op
    module procedure demean2d_op2d
    module procedure demean2d_op3d
  end interface

  interface demean3d_op
    module procedure demean3d_op3d
  end interface

  contains

  subroutine demean1d_op1d(data)
    real,intent(inout)   :: data(:)

    n1   = size(      data  )
    mean = sum ( dble(data) )/dble(n1)
    data = real( dble(data) ) - mean

  end subroutine


  subroutine demean1d_op2d(data)
    real,intent(inout)   :: data(:,:)

    n1 = size(data,1)
    n2 = size(data,2)
    do jj=1,n2
      mean       = sum ( dble(data(:,jj)) )/dble(n1)
      data(:,jj) = real( dble(data(:,jj)) ) - mean
    end do

  end subroutine


  subroutine demean1d_op3d(data)
    real,intent(inout)   :: data(:,:,:)
    
    n1 = size(data,1)
    n2 = size(data,2)
    n3 = size(data,3)
    do kk=1,n3
      do jj=1,n2
        mean          = sum ( dble(data(:,jj,kk)) )/dble(n1)
        data(:,jj,kk) = real( dble(data(:,jj,kk)) ) - mean
      end do
    end do
  
  end subroutine


  subroutine demean2d_op2d(data)
    real,intent(inout)   :: data(:,:)

    n1 = size(data,1)
    n2 = size(data,2)
    mean = sum ( dble(data) )/dble(n1*n2)
    data = real( dble(data) ) - mean

  end subroutine


  subroutine demean2d_op3d(data)
    real,intent(inout)   :: data(:,:,:)
    
    n1 = size(data,1)
    n2 = size(data,2)
    n3 = size(data,3)
    do kk=1,n3
      mean         = sum ( dble(data(:,:,kk)) )/dble(n1*n2)
      data(:,:,kk) = real( dble(data(:,:,kk)) ) - mean
    end do
  
  end subroutine


  subroutine demean3d_op3d(data)
    real,intent(inout)   :: data(:,:,:)
    
    n1 = size(data,1)
    n2 = size(data,2)
    n3 = size(data,3)
    mean = sum ( dble(data) )/dble(n1*n2*n3)
    data = real( dble(data) ) - mean

  end subroutine


end module
