module fftshift_io_mod_scratch
  
  use sep
  
  interface fftshiftsreed
    module procedure fftshift1dsreed_1d
    module procedure fftshift2dsreed_2d
    module procedure fftshift3dsreed_3d
  end interface
  
  interface fftshiftsrite
    module procedure fftshift1dsrite_1d
    module procedure fftshift2dsrite_2d
    module procedure fftshift3dsrite_3d
  end interface
  
  interface fftshift1dsreed
    module procedure fftshift1dsreed_1d
    module procedure fftshift1dsreed_2d
    module procedure fftshift1dsreed_3d
  end interface
  
  interface fftshift1dsrite
    module procedure fftshift1dsrite_1d
    module procedure fftshift1dsrite_2d
    module procedure fftshift1dsrite_3d
  end interface
  
  interface fftshift2dsreed
    module procedure fftshift2dsreed_2d
    module procedure fftshift2dsreed_3d
  end interface
  
  interface fftshift2dsrite
    module procedure fftshift2dsrite_2d
    module procedure fftshift2dsrite_3d
  end interface
  
  interface fftshift3dsreed
    module procedure fftshift3dsreed_3d
  end interface
  
  interface fftshift3dsrite
    module procedure fftshift3dsrite_3d
  end interface
  
  contains
  
  subroutine fftshift1dsreed_1d(tag,array,n1)
    integer             ::                n1
    real                ::        array(:)
    character*50        ::    tag
    integer             ::                nt
    
    nt = size(array,1)
    call sreed( tag, array(nt-n1/2+1:nt     ), n1/2 );
    call sreed( tag, array(        1:nt-n1/2), n1/2 );
    
  end subroutine
  
  subroutine fftshift1dsreed_2d(tag,array,n1,nx)
    integer             ::              size,nt,jj,nx
    real                ::        array(:,:)
    character*50        ::    tag
    
    nt = size(array,1)
    do jj=1,nx
      call sreed( tag, array(nt-n1/2+1:nt     ,jj), size/2/nx );
      call sreed( tag, array(        1:nt-n1/2,jj), size/2/nx );
    end do
    
  end subroutine

  subroutine fftshift1dsreed_3d(tag,array,size)
    integer             ::              size,nt,jj,nx,kk,ny
    real                ::        array(:,:,:)
    character*50        ::    tag
    
    nt = size(array,1)
    nx = size(array,2)
    ny = size(array,3)
    do kk=1,ny
    do jj=1,nx
      call sreed( tag, array(nt/2+1:nt  ,jj,kk), size/2/nx/ny );
      call sreed( tag, array(     1:nt/2,jj,kk), size/2/nx/ny );
    end do
    end do
    
  end subroutine
  



  subroutine fftshift1dsrite_1d(tag,array,size)
    integer             ::              size,nt
    real                ::        array(:)
    character*50        ::    tag
    
    nt = size(array,1)
    call srite( tag, array(nt/2+1:nt  ), size/2 );
    call srite( tag, array(     1:nt/2), size/2 );
    
  end subroutine

  subroutine fftshift1dsrite_2d(tag,array,size)
    integer             ::              size,nt,nx,jj
    real                ::        array(:,:)
    character*50        ::    tag
    
    nt = size(array,1)
    nx = size(array,2)
    do jj=1,ny
      call srite( tag, array(nt/2+1:nt  ,jj), size/2/nx );
      call srite( tag, array(     1:nt/2,jj), size/2/nx );
    end do
    
  end subroutine

  subroutine fftshift1dsrite_3d(tag,array,size)
    integer             ::              size,nt,nx,jj,ny,kk
    real                ::        array(:,:,:)
    character*50        ::    tag
    
    nt = size(array,1)
    nx = size(array,2)
    ny = size(array,3)
    do kk=1,ny
    do jj=1,nx
      call srite( tag, array(nt/2+1:nt  ,jj,kk), size/2/nx/ny );
      call srite( tag, array(     1:nt/2,jj,kk), size/2/nx/ny );
    end do
    end do
    
  end subroutine







  subroutine fftshift2dsreed_2d(tag,array,size)
    integer             ::              size,nt,jj,nx
    real                ::        array(:,:)
    character*50        ::    tag
    
    nt = size(array,1)
    nx = size(array,2)

    do jj=1,nx/2
      call sreed( tag, array(nt/2+1:nt  ,nx/2+jj), size/2/nx );
      call sreed( tag, array(     1:nt/2,nx/2+jj), size/2/nx );
    end do
    do jj=1,nx/2
      call sreed( tag, array(nt/2+1:nt  ,     jj), size/2/nx );
      call sreed( tag, array(     1:nt/2,     jj), size/2/nx );
    end do

  end subroutine

  subroutine fftshift2dsreed_3d(tag,array,size)
    integer             ::              size,nt,jj,nx,kk,ny
    real                ::        array(:,:,:)
    character*50        ::    tag
    
    nt = size(array,1)
    nx = size(array,2)
    ny = size(array,3)

    do kk=1,ny
    do jj=1,nx/2
      call sreed( tag, array(nt/2+1:nt  ,nx/2+jj,kk), size/2/nx/ny );
      call sreed( tag, array(     1:nt/2,nx/2+jj,kk), size/2/nx/ny );
    end do
    do jj=1,nx/2
      call sreed( tag, array(nt/2+1:nt  ,     jj,kk), size/2/nx/ny );
      call sreed( tag, array(     1:nt/2,     jj,kk), size/2/nx/ny );
    end do
    end do

  end subroutine

  
  




  subroutine fftshift2dsrite_2d(tag,array,size)
    integer             ::              size,nt,jj,nx
    real                ::        array(:,:)
    character*50        ::    tag
    
    nt = size(array,1)
    nx = size(array,2)
    
    do jj=1,nx/2
      call srite( tag, array(nt/2+1:nt  ,nx/2+jj), size/2/nx );
      call srite( tag, array(     1:nt/2,nx/2+jj), size/2/nx );
    end do
    do jj=1,nx/2
      call srite( tag, array(nt/2+1:nt  ,     jj), size/2/nx );
      call srite( tag, array(     1:nt/2,     jj), size/2/nx );
    end do
    
  end subroutine
  
  subroutine fftshift2dsrite_3d(tag,array,size)
    integer             ::              size,nt,jj,nx,kk,ny
    real                ::        array(:,:,:)
    character*50        ::    tag
    
    nt = size(array,1)
    nx = size(array,2)
    ny = size(array,3)
    
    do kk=1,ny
    do jj=1,nx/2
      call srite( tag, array(nt/2+1:nt  ,nx/2+jj,kk), size/2/nx/ny );
      call srite( tag, array(     1:nt/2,nx/2+jj,kk), size/2/nx/ny );
    end do
    do jj=1,nx/2
      call srite( tag, array(nt/2+1:nt  ,     jj,kk), size/2/nx/ny );
      call srite( tag, array(     1:nt/2,     jj,kk), size/2/nx/ny );
    end do
    end do
    
  end subroutine








  subroutine fftshift3dsreed_3d(tag,array,size)
    integer             ::              size,nt,jj,nx,kk,ny
    real                ::        array(:)
    character*50        ::    tag
    
    nt = size(array,1)
    nx = size(array,2)
    ny = size(array,3)
    
    do kk=1,ny/2
    do jj=1,nx/2
      call sreed( tag, array(nt/2+1:nt  ,nx/2+jj,ny/2+kk), size/2/nx/ny );
      call sreed( tag, array(     1:nt/2,nx/2+jj,ny/2+kk), size/2/nx/ny );
    end do
    do jj=1,nx/2
      call sreed( tag, array(nt/2+1:nt  ,     jj,ny/2+kk), size/2/nx/ny );
      call sreed( tag, array(     1:nt/2,     jj,ny/2+kk), size/2/nx/ny );
    end do
    end do
    do kk=1,ny/2
    do jj=1,nx/2
      call sreed( tag, array(nt/2+1:nt  ,nx/2+jj,     kk), size/2/nx/ny );
      call sreed( tag, array(     1:nt/2,nx/2+jj,     kk), size/2/nx/ny );
    end do
    do jj=1,nx/2
      call sreed( tag, array(nt/2+1:nt  ,     jj,     kk), size/2/nx/ny );
      call sreed( tag, array(     1:nt/2,     jj,     kk), size/2/nx/ny );
    end do
    end do

  end subroutine
  
  
  subroutine fftshift3dsrite_3d(tag,array,size)
    integer             ::              size,nt,jj,nx,kk,ny
    real                ::        array(:)
    character*50        ::    tag
    
    nt = size(array,1)
    nx = size(array,2)
    ny = size(array,3)
    
    do kk=1,ny/2
    do jj=1,nx/2
      call srite( tag, array(nt/2+1:nt  ,nx/2+jj,ny/2+kk), size/2/nx/ny );
      call srite( tag, array(     1:nt/2,nx/2+jj,ny/2+kk), size/2/nx/ny );
    end do
    do jj=1,nx/2
      call srite( tag, array(nt/2+1:nt  ,     jj,ny/2+kk), size/2/nx/ny );
      call srite( tag, array(     1:nt/2,     jj,ny/2+kk), size/2/nx/ny );
    end do
    end do
    do kk=1,ny/2
    do jj=1,nx/2
      call srite( tag, array(nt/2+1:nt  ,nx/2+jj,     kk), size/2/nx/ny );
      call srite( tag, array(     1:nt/2,nx/2+jj,     kk), size/2/nx/ny );
    end do
    do jj=1,nx/2
      call srite( tag, array(nt/2+1:nt  ,     jj,     kk), size/2/nx/ny );
      call srite( tag, array(     1:nt/2,     jj,     kk), size/2/nx/ny );
    end do
    end do
    
  end subroutine




end module
