  subroutine fftw1d_1_r2c_lop_init(nt_in)
    integer,intent(in),optional::nt_in
    integer                    ::nt,nf

    call fftw1d_1_r2c_lop_close()

    if ( present( nt_in) ) then; nt = nt_in; end if;
    nf = nt/2+1

    ! Prepare IFFTW
    allocate( in_1d_1(nt), out_1d_1(nf) )
    call sfftw_plan_dft_r2c_1d(planf_1d_1, nt,  in_1d_1, out_1d_1, FFTW_ESTIMATE);
    call sfftw_plan_dft_c2r_1d(plani_1d_1, nt, out_1d_1,  in_1d_1, FFTW_ESTIMATE);
    
  end subroutine


  subroutine fftw1d_1_r2c_lop_close()
    ! TO BE DONE: DESTROY PLANS
    if (allocated( in_1d_1)) then; deallocate( in_1d_1); end if
    if (allocated(out_1d_1)) then; deallocate(out_1d_1); end if
  end subroutine


  subroutine fftw1d_1_r2c_lop_1d(adj,add,m,d)
    logical, intent(in)      :: adj, add
    real                     ::           m(:)
    complex                  ::              d(:)
    integer                  :: n1
    ! model is temporal-domain; data is fourier-domain.

    call adjnull( adj, add, m, d )

    n1 = size(m,1)
    if ( adj ) then
      out_1d_1 = d
      call sfftw_execute( plani_1d_1 )
      m  = m + in_1d_1 / sqrt( real(n1) )
    else
      in_1d_1 = m
      call sfftw_execute( planf_1d_1 )
      d  = d + out_1d_1/ sqrt( real(n1) )
    end if

  end subroutine



  subroutine fftw1d_1_r2c_lop_2d(adj,add,m,d)
    logical, intent(in)      :: adj, add
    real                     ::           m(:,:)
    complex                  ::              d(:,:)
    integer                  :: n1,jj,n2
    ! model is temporal-domain; data is fourier-domain.

    call adjnull( adj, add, m, d )
    
    n1 = size(m,1)
    n2 = size(m,2)
    if ( adj ) then
      do jj=1,n2
        out_1d_1     = d(:,jj)
        call sfftw_execute( plani_1d_1 )
        m(:,jj)  = m(:,jj) +  in_1d_1 / sqrt( real(n1) )
      end do
    else
      do jj=1,n2
        in_1d_1 = m(:,jj)
        call sfftw_execute( planf_1d_1 )
        d(:,jj)  = d(:,jj) + out_1d_1 / sqrt( real(n1) )
      end do
    end if
    
  end subroutine



  subroutine fftw1d_1_r2c_lop_3d(adj,add,m,d)
    logical, intent(in)      :: adj, add
    real                     ::           m(:,:,:)
    complex                  ::              d(:,:,:)
    integer                  :: n1,jj,kk,n2,n3
    ! model is temporal-domain; data is fourier-domain.

    call adjnull( adj, add, m, d )
    
    n1 = size(m,1)
    n2 = size(m,2)
    n3 = size(m,3)
    if ( adj ) then
      do kk=1,n3
      do jj=1,n2
        out_1d_1        = d(:,jj,kk)
        call sfftw_execute( plani_1d_1 )
        m(:,jj,kk)  = m(:,jj,kk) +  in_1d_1 / sqrt( real(n1) )
      end do
      end do
    else
      do kk=1,n3
      do jj=1,n2
        in_1d_1         = m(:,jj,kk)
        call sfftw_execute( planf_1d_1 )
        d(:,jj,kk)  = d(:,jj,kk) + out_1d_1 / sqrt( real(n1) )
      end do
      end do
    end if
    
  end subroutine





  subroutine fftw1d_2_r2c_lop_init(nt_in)
    integer,intent(in),optional::nt_in
    integer                    ::nt,nf

    call fftw1d_2_r2c_lop_close()

    if ( present( nt_in) ) then; nt = nt_in; end if;
    nf = nt/2+1

    ! Prepare IFFTW
    allocate( in_1d_2(nt), out_1d_2(nf) )
    call sfftw_plan_dft_r2c_1d(planf_1d_2, nt,  in_1d_2, out_1d_2, FFTW_ESTIMATE);
    call sfftw_plan_dft_c2r_1d(plani_1d_2, nt, out_1d_2,  in_1d_2, FFTW_ESTIMATE);
    
  end subroutine


  subroutine fftw1d_2_r2c_lop_close()
    ! TO BE DONE: DESTROY PLANS
    if (allocated( in_1d_2)) then; deallocate( in_1d_2); end if
    if (allocated(out_1d_2)) then; deallocate(out_1d_2); end if
  end subroutine


  subroutine fftw1d_2_r2c_lop_2d(adj,add,m,d)
    logical, intent(in)      :: adj, add
    real                     ::           m(:,:)
    complex                  ::              d(:,:)
    integer                  :: ii,n1,n2
    ! model is temporal-domain; data is fourier-domain.

    call adjnull( adj, add, m, d )
    
    n1 = size(m,1)
    n2 = size(m,2)
    if ( adj ) then
      do ii=1,n1
        out_1d_2      = d(ii,:)
        call sfftw_execute( plani_1d_2 )
        m(ii,:)  = m(ii,:) +  in_1d_2 / sqrt( real(n2) )
      end do
    else
      do ii=1,n1
        in_1d_2 = m(ii,:)
        call sfftw_execute( planf_1d_2 )
        d(ii,:)  = d(ii,:) + out_1d_2 / sqrt( real(n2) )
      end do
    end if
    
  end subroutine


  subroutine fftw1d_2_r2c_lop_3d(adj,add,m,d)
    logical, intent(in)      :: adj, add
    real                     ::           m(:,:,:)
    complex                  ::              d(:,:,:)
    integer                  :: ii,kk,n1,n2,n3
    ! model is temporal-domain; data is fourier-domain.

    call adjnull( adj, add, m, d )
    
    n1 = size(m,1)
    n2 = size(m,2)
    n3 = size(m,3)
    if ( adj ) then
      do kk=1,n3
      do ii=1,n1
        out_1d_2    = d(ii,:,kk)
        call sfftw_execute( plani_1d_2 )
        m(ii,:,kk)  = m(ii,:,kk) +  in_1d_2 / sqrt( real(n2) )
      end do
      end do
    else
      do kk=1,n3
      do ii=1,n1
        in_1d_2     = m(ii,:,kk)
        call sfftw_execute( planf_1d_2 )
        d(ii,:,kk)  = d(ii,:,kk) + out_1d_2 / sqrt( real(n2) )
      end do
      end do
    end if
    
  end subroutine





  subroutine fftw1d_3_r2c_lop_init(nt_in)
    integer,intent(in),optional::nt_in
    integer                    ::nt,nf

    call fftw1d_3_r2c_lop_close()

    if ( present( nt_in) ) then; nt = nt_in; end if;
    nf = nt/2+1

    ! Prepare IFFTW
    allocate( in_1d_3(nt), out_1d_3(nf) )
    call sfftw_plan_dft_r2c_1d(planf_1d_3, nt,  in_1d_3, out_1d_3, FFTW_ESTIMATE);
    call sfftw_plan_dft_c2r_1d(plani_1d_3, nt, out_1d_3,  in_1d_3, FFTW_ESTIMATE);
    
  end subroutine


  subroutine fftw1d_3_r2c_lop_close()
    ! TO BE DONE: DESTROY PLANS
    if (allocated( in_1d_3)) then; deallocate( in_1d_3); end if
    if (allocated(out_1d_3)) then; deallocate(out_1d_3); end if
  end subroutine



  subroutine fftw1d_3_r2c_lop_3d(adj,add,m,d)
    logical, intent(in)      :: adj, add
    real                     ::           m(:,:,:)
    complex                  ::              d(:,:,:)
    integer                  :: ii,jj,n1,n2,n3
    ! model is temporal-domain; data is fourier-domain.
    
    call adjnull( adj, add, m, d )
    
    n1 = size(m,1)
    n2 = size(m,2)
    n3 = size(m,3)
    if ( adj ) then
      do jj=1,n2
      do ii=1,n1
        out_1d_3    = d(ii,jj,:)
        call sfftw_execute( plani_1d_3 )
        m(ii,jj,:)  = m(ii,jj,:) +  in_1d_3 / sqrt( real(n3) )
      end do
      end do
    else
      do jj=1,n2
      do ii=1,n1
        in_1d_3     = m(ii,jj,:)
        call sfftw_execute( planf_1d_3 )
        d(ii,jj,:)  = d(ii,jj,:) + out_1d_3 / sqrt( real(n3) )
      end do
      end do
    end if
    
  end subroutine



