  subroutine fftw3d_r2c_lop_init(n1_in,n2_in,n3_in)
    integer,intent(in),optional::n1_in,n2_in,n3_in
    integer                    ::n1,n2,n3,nk1,nk2,nk3

    call fftw3d_r2c_lop_close()

    if ( present( n1_in) ) then; n1 = n1_in; end if;
    if ( present( n2_in) ) then; n2 = n2_in; else; n2=1; end if;
    if ( present( n3_in) ) then; n3 = n3_in; else; n3=1; end if;
    nk1 = n1/2+1
    nk2 = n2
    nk3 = n3

    ! Prepare IFFTW
    if (n1.ne.1 .and. n2.eq.1 .and. n3.eq.1) then
      allocate( in_3d_1(n1), out_3d_1(nk1) )
      call sfftw_plan_dft_r2c_1d(planf_3d_1, n1, in_3d_1, out_3d_1, FFTW_ESTIMATE);
      call sfftw_plan_dft_c2r_1d(plani_3d_1, n1, out_3d_1, in_3d_1, FFTW_ESTIMATE);
    end if
    if (n1.ne.1 .and. n2.ne.1 .and. n3.eq.1) then
      allocate( in_3d_2(n1,n2), out_3d_2(nk1,nk2) )
      call sfftw_plan_dft_r2c_2d(planf_3d_2, n1, n2, in_3d_2, out_3d_2, FFTW_ESTIMATE);
      call sfftw_plan_dft_c2r_2d(plani_3d_2, n1, n2, out_3d_2, in_3d_2, FFTW_ESTIMATE);
    end if
    if (n1.ne.1 .and. n2.ne.1 .and. n3.ne.1) then
      allocate( in_3d_3(n1,n2,n3), out_3d_3(nk1,nk2,nk3) )
      call sfftw_plan_dft_r2c_3d(planf_3d_3, n1, n2, n3,  in_3d_3, out_3d_3, FFTW_ESTIMATE);
      call sfftw_plan_dft_c2r_3d(plani_3d_3, n1, n2, n3, out_3d_3,  in_3d_3, FFTW_ESTIMATE);
    end if

  end subroutine

             
  subroutine fftw3d_r2c_lop_close()
    ! TO BE DONE: DESTROY PLANS
    if (allocated( in_3d_1)) then; deallocate( in_3d_1); end if
    if (allocated(out_3d_1)) then; deallocate(out_3d_1); end if

    if (allocated( in_3d_2)) then; deallocate( in_3d_2); end if
    if (allocated(out_3d_2)) then; deallocate(out_3d_2); end if

    if (allocated( in_3d_3)) then; deallocate( in_3d_3); end if
    if (allocated(out_3d_3)) then; deallocate(out_3d_3); end if
  end subroutine
  
  
  
  subroutine fftw3d_r2c_lop_3d(adj,add,m,d)
    logical, intent(in)      :: adj, add
    real                     ::           m(:,:,:)
    complex                  ::              d(:,:,:)
    integer                  :: n1,n2,n3
    
    call adjnull( adj, add, m, d )
    
    n1 = size(m,1)
    n2 = size(m,2)
    n3 = size(m,3)
    if ( adj ) then
      out_3d_3 = d
      call sfftw_execute( plani_3d_3 )
      m  = m +  in_3d_3/ sqrt( real(n1*n2*n3) )
    else
      in_3d_3  = m
      call sfftw_execute( planf_3d_3 )
      d  = d + out_3d_3/ sqrt( real(n1*n2*n3) )
    end if
    
  end subroutine



  subroutine fftw3d_r2c_lop_2d(adj,add,m,d)
    logical, intent(in)      :: adj, add
    real                     ::           m(:,:)
    complex                  ::              d(:,:)
    integer                  :: n1,n2
    
    call adjnull( adj, add, m, d )
    
    n1 = size(m,1)
    n2 = size(m,2)
    if ( adj ) then
      out_3d_2 = d
      call sfftw_execute( plani_3d_2 )
      m  = m +  in_3d_2/ sqrt( real(n1*n2) )
    else
      in_3d_2  = m
      call sfftw_execute( planf_3d_2 )
      d  = d + out_3d_2/ sqrt( real(n1*n2) )
    end if
    
  end subroutine



  subroutine fftw3d_r2c_lop_1d(adj,add,m,d)
    logical, intent(in)      :: adj, add
    real                     ::           m(:)
    complex                  ::              d(:)
    integer                  :: n1
    
    call adjnull( adj, add, m, d )
    
    n1 = size(m,1)
    if ( adj ) then
      out_3d_1 = d
      call sfftw_execute( plani_3d_1 )
      m  = m +  in_3d_1/ sqrt( real(n1) )
    else
      in_3d_1  = m
      call sfftw_execute( planf_3d_1 )
      d  = d + out_3d_1/ sqrt( real(n1) )
    end if
    
  end subroutine
