module mtspectra_mod_omp
  ! Multitaper Correlation, Coherence and Deconvolution
  
  implicit none

  real,private :: wl

  interface mtspectra_ampl
    module procedure mtspectra_ampl_1_1d
    module procedure mtspectra_ampl_1_2d
    module procedure mtspectra_ampl_1_3d
  end interface

  interface mtspectra_1_ampl
    module procedure mtspectra_ampl_1_1d
    module procedure mtspectra_ampl_1_2d
    module procedure mtspectra_ampl_1_3d
  end interface

  interface mtspectra_2_ampl
    module procedure mtspectra_ampl_2_2d
    module procedure mtspectra_ampl_2_3d
  end interface

  interface mtspectra_3_ampl
    module procedure mtspectra_ampl_3_3d
  end interface

  interface mtspectra_powr
    module procedure mtspectra_powr_1_1d
    module procedure mtspectra_powr_1_2d
    module procedure mtspectra_powr_1_3d
  end interface

  interface mtspectra_1_powr
    module procedure mtspectra_powr_1_1d
    module procedure mtspectra_powr_1_2d
    module procedure mtspectra_powr_1_3d
  end interface

  interface mtspectra_2_powr
    module procedure mtspectra_powr_2_2d
    module procedure mtspectra_powr_2_3d
  end interface

  interface mtspectra_3_powr
    module procedure mtspectra_powr_3_3d
  end interface

  interface mtspectra_crsp
    module procedure mtspectra_crsp_1d_1d
    module procedure mtspectra_crsp_1d_2d
    module procedure mtspectra_crsp_1d_3d
    module procedure mtspectra_crsp_2d_1d
    module procedure mtspectra_crsp_3d_1d
    module procedure mtspectra_crsp_2d_2d
    module procedure mtspectra_crsp_3d_3d
  end interface


  interface mtspectra_cohr
    module procedure mtspectra_cohr_1d_1d
    module procedure mtspectra_cohr_1d_2d
    module procedure mtspectra_cohr_1d_3d
    module procedure mtspectra_cohr_2d_1d
    module procedure mtspectra_cohr_3d_1d
    module procedure mtspectra_cohr_2d_2d
    module procedure mtspectra_cohr_3d_3d
  end interface


  interface mtspectra_decv
    module procedure mtspectra_decv_1d_1d
    module procedure mtspectra_decv_1d_2d
    module procedure mtspectra_decv_1d_3d
    module procedure mtspectra_decv_2d_1d
    module procedure mtspectra_decv_3d_1d
    module procedure mtspectra_decv_2d_2d
    module procedure mtspectra_decv_3d_3d
  end interface

  include "openmp.inc"

  contains

  subroutine mtspectra_init(wl_in)
    real, intent(in),optional    ::  wl_in
    call mtspectra_close()
    if (present(wl_in)) then;  wl = wl_in;  else;  wl = 0.; end if
    include "openmp_init.inc"
  end subroutine

  subroutine mtspectra_close()
    wl = 0.
  end subroutine


  subroutine mtspectra_ampl_1_1d(a,c)
    complex            :: a(:,:)
    real               :: c(:)
    integer            :: nk
    nk = size(a,2)
    c = sum(abs(a),2)/nk
  end subroutine


  subroutine mtspectra_ampl_1_2d(a,c)
    complex            :: a(:,:,:)
    real               :: c(:,:)
    integer            :: nk
    nk = size(a,2)
    c = sum(abs(a),2)/nk
  end subroutine

  subroutine mtspectra_ampl_1_3d(a,c)
    complex            :: a(:,:,:,:)
    real               :: c(:,:,:)
    integer            :: nk
    nk = size(a,2)
    c = sum(abs(a),2)/nk
  end subroutine

  subroutine mtspectra_ampl_2_2d(a,c)
    complex            :: a(:,:,:)
    real               :: c(:,:)
    integer            :: nk
    nk = size(a,3)
    c = sum(abs(a),3)/nk
  end subroutine

  subroutine mtspectra_ampl_2_3d(a,c)
    complex            :: a(:,:,:,:)
    real               :: c(:,:,:)
    integer            :: nk
    nk = size(a,3)
    c = sum(abs(a),3)/nk
  end subroutine

  subroutine mtspectra_ampl_3_3d(a,c)
    complex            :: a(:,:,:,:)
    real               :: c(:,:,:)
    integer            :: nk
    nk = size(a,4)
    c = sum(abs(a),4)/nk
  end subroutine






  subroutine mtspectra_powr_1_1d(a,c)
    complex            :: a(:,:)
    real               :: c(:)
    integer            :: nk
    nk = size(a,2)
    c = sum(a*conjg(a),2)/nk
  end subroutine


  subroutine mtspectra_powr_1_2d(a,c)
    complex            :: a(:,:,:)
    real               :: c(:,:)
    integer            :: nk
    nk = size(a,2)
    c = sum(a*conjg(a),2)/nk
  end subroutine

  subroutine mtspectra_powr_1_3d(a,c)
    complex            :: a(:,:,:,:)
    real               :: c(:,:,:)
    integer            :: nk
    nk = size(a,2)
    c = sum(a*conjg(a),2)/nk
  end subroutine

  subroutine mtspectra_powr_2_2d(a,c)
    complex            :: a(:,:,:)
    real               :: c(:,:)
    integer            :: nk
    nk = size(a,3)
    c = sum(a*conjg(a),3)/nk
  end subroutine

  subroutine mtspectra_powr_2_3d(a,c)
    complex            :: a(:,:,:,:)
    real               :: c(:,:,:)
    integer            :: nk
    nk = size(a,3)
    c = sum(a*conjg(a),3)/nk
  end subroutine

  subroutine mtspectra_powr_3_3d(a,c)
    complex            :: a(:,:,:,:)
    real               :: c(:,:,:)
    integer            :: nk
    nk = size(a,4)
    c = sum(a*conjg(a),4)/nk
  end subroutine















  subroutine mtspectra_crsp_1d_1d(a,b,c)
    complex            :: a(:,:),b(:,:),c(:)
    integer            :: n1,ii
    
    n1 = size(c,1)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1)
    do ii=1,n1
      c(ii      ) = sum( a(ii,:      )*conjg(b(ii,:   )) )
    end do
    !$OMP END PARALLEL DO

  end subroutine

  subroutine mtspectra_crsp_2d_1d(a,b,c)
    complex            :: a(:,:,:),b(:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = sum( a(ii,:,jj   )*conjg(b(ii,:   )) )
    end do;         end do
    !$OMP END PARALLEL DO

  end subroutine

  subroutine mtspectra_crsp_3d_1d(a,b,c)
    complex            :: a(:,:,:,:),b(:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,n3)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = sum( a(ii,:,jj,kk)*conjg(b(ii,:   )) )
    end do;         end do;         end do
    !$OMP END PARALLEL DO

  end subroutine


  subroutine mtspectra_crsp_1d_2d(a,b,c)
    complex            :: a(:,:),b(:,:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = sum( a(ii,:      )*conjg(b(ii,:,jj   )) )
    end do;         end do
    !$OMP END PARALLEL DO

  end subroutine


  subroutine mtspectra_crsp_1d_3d(a,b,c)
    complex            :: a(:,:),b(:,:,:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,n3)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = sum( a(ii,:      )*conjg(b(ii,:,jj,kk)) )
    end do;       end do;           end do
    !$OMP END PARALLEL DO

  end subroutine


  subroutine mtspectra_crsp_2d_2d(a,b,c)
    complex            :: a(:,:,:),b(:,:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = sum( a(ii,:,jj   )*conjg(b(ii,:,jj   )) )
    end do;         end do
    !$OMP END PARALLEL DO

  end subroutine


  subroutine mtspectra_crsp_3d_3d(a,b,c)
    complex            :: a(:,:,:,:),b(:,:,:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,n3)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = sum( a(ii,:,jj,kk)*conjg(b(ii,:,jj,kk)) )
    end do;         end do;         end do
    !$OMP END PARALLEL DO

  end subroutine



















  subroutine mtspectra_cohr_1d_1d(a,b,c)
    complex            :: a(:,:),b(:,:),c(:)
    integer            :: n1,ii
    
    n1 = size(c,1)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,wl)
    do ii=1,n1
      c(ii      ) = sum( a(ii,:      )*conjg(b(ii,:   )) ) / ( sqrt( sum(a(ii,:)**2.) )*sqrt( sum(b(ii,:)**2.) ) + wl )
    end do
    !$OMP END PARALLEL DO

  end subroutine

  subroutine mtspectra_cohr_2d_1d(a,b,c)
    complex            :: a(:,:,:),b(:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,wl)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = sum( a(ii,:,jj   )*conjg(b(ii,:   )) ) / ( sqrt( sum(a(ii,:,jj)**2.) )*sqrt( sum(b(ii,:)**2.) ) + wl )
    end do;         end do
    !$OMP END PARALLEL DO

  end subroutine

  subroutine mtspectra_cohr_3d_1d(a,b,c)
    complex            :: a(:,:,:,:),b(:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,n3,wl)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = sum( a(ii,:,jj,kk)*conjg(b(ii,:   )) ) / ( sqrt( sum(a(ii,:,jj,kk)**2.) )*sqrt( sum(b(ii,:)**2.) ) + wl )
    end do;         end do;         end do
    !$OMP END PARALLEL DO

  end subroutine


  subroutine mtspectra_cohr_1d_2d(a,b,c)
    complex            :: a(:,:),b(:,:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,wl)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = sum( a(ii,:      )*conjg(b(ii,:,jj   )) ) / ( sqrt( sum(a(ii,:)**2.) )*sqrt( sum(b(ii,:,jj)**2.) ) + wl )
    end do;         end do
    !$OMP END PARALLEL DO

  end subroutine


  subroutine mtspectra_cohr_1d_3d(a,b,c)
    complex            :: a(:,:),b(:,:,:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,n3,wl)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = sum( a(ii,:      )*conjg(b(ii,:,jj,kk)) ) / ( sqrt( sum(a(ii,:)**2.) )*sqrt( sum(b(ii,:,jj,kk)**2.) ) + wl )
    end do;       end do;           end do
    !$OMP END PARALLEL DO

  end subroutine


  subroutine mtspectra_cohr_2d_2d(a,b,c)
    complex            :: a(:,:,:),b(:,:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,wl)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = sum( a(ii,:,jj   )*conjg(b(ii,:,jj   )) ) / ( sqrt( sum(a(ii,:,jj)**2.) )*sqrt( sum(b(ii,:,jj)**2.) ) + wl )
    end do;         end do
    !$OMP END PARALLEL DO

  end subroutine


  subroutine mtspectra_cohr_3d_3d(a,b,c)
    complex            :: a(:,:,:,:),b(:,:,:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,n3,wl)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = sum( a(ii,:,jj,kk)*conjg(b(ii,:,jj,kk)) ) / ( sqrt( sum(a(ii,:,jj,kk)**2.) )*sqrt( sum(b(ii,:,jj,kk)**2.) ) + wl )
    end do;         end do;         end do
    !$OMP END PARALLEL DO

  end subroutine


















  subroutine mtspectra_decv_1d_1d(a,b,c)
    complex            :: a(:,:),b(:,:),c(:)
    integer            :: n1,ii
    
    n1 = size(c,1)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,wl)
    do ii=1,n1
      c(ii      ) = sum( a(ii,:      )*conjg(b(ii,:   )) ) / ( sum(b(ii,:)**2.) + wl )
    end do
    !$OMP END PARALLEL DO

  end subroutine

  subroutine mtspectra_decv_2d_1d(a,b,c)
    complex            :: a(:,:,:),b(:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,wl)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = sum( a(ii,:,jj   )*conjg(b(ii,:   )) ) / ( sum(b(ii,:)**2.) + wl )
    end do;         end do
    !$OMP END PARALLEL DO

  end subroutine

  subroutine mtspectra_decv_3d_1d(a,b,c)
    complex            :: a(:,:,:,:),b(:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,wl)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = sum( a(ii,:,jj,kk)*conjg(b(ii,:   )) ) / ( sum(b(ii,:)**2.) + wl )
    end do;         end do;         end do
    !$OMP END PARALLEL DO

  end subroutine


  subroutine mtspectra_decv_1d_2d(a,b,c)
    complex            :: a(:,:),b(:,:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,wl)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = sum( a(ii,:      )*conjg(b(ii,:,jj   )) ) / ( sum(b(ii,:,jj)**2.) + wl )
    end do;         end do
    !$OMP END PARALLEL DO

  end subroutine


  subroutine mtspectra_decv_1d_3d(a,b,c)
    complex            :: a(:,:),b(:,:,:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,n3,wl)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = sum( a(ii,:      )*conjg(b(ii,:,jj,kk)) ) / ( sum(b(ii,:,jj,kk)**2.) + wl )
    end do;       end do;           end do
    !$OMP END PARALLEL DO

  end subroutine


  subroutine mtspectra_decv_2d_2d(a,b,c)
    complex            :: a(:,:,:),b(:,:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,wl)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = sum( a(ii,:,jj   )*conjg(b(ii,:,jj   )) ) / ( sum(b(ii,:,jj)**2.) + wl )
    end do;         end do
    !$OMP END PARALLEL DO

  end subroutine


  subroutine mtspectra_decv_3d_3d(a,b,c)
    complex            :: a(:,:,:,:),b(:,:,:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(a,b,c,n1,n2,n3,wl)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = sum( a(ii,:,jj,kk)*conjg(b(ii,:,jj,kk)) ) / ( sum(b(ii,:,jj,kk)**2.) + wl )
    end do;         end do;         end do
    !$OMP END PARALLEL DO

  end subroutine





end module
