program Scale_Correction
  ! Amplitude comes out as (m^2/s^2)
  
  ! Taper Specifics (like in srTaper1d)
  ! opt: (taper types 
  ! 0)  'zero'            --  needed: 'nothing' 
  ! 1)  'rect' (default)  --  needed: f1,f2
  ! 2)  'linear'          --  needed: f1,f2
  ! 3)  'pyramid'         --  needed: f1,f2
  ! 4)  'pyramid-band'    --  needed: f1,f2,f3,f4
  ! 5)  'hann'            --  needed: f1,f2
  ! 6)  'hann_half'       --  needed: f1,f2
  ! 7)  'hann-band'       --  needed: f1,f2,f3,f4
  ! 8)  'gaussian'        --  needed: f1,f2,sigma
  ! 9)  'gaussian_half'   --  needed: f1,f2,sigma
  ! 10) 'gaussian_block'  --  needed: f1,f2,f3,f4,sigma


  use sep
  use fftw_mod
  use fftshift_io_mod
  implicit none

  integer             :: n1,n2,n3,n4
  integer             :: ii,jj,kk,ll
  
  real                :: dt,df,of
  integer             :: nt,nf
  integer             :: ti,fi
  
  real                :: d1,d2,d3,d4
  real                :: o1,o2,o3,o4
  
  real                :: factor
  real                :: spec_ave, sens_ave, ave
  
  real                :: f,w

  real,allocatable    :: datat(:,:,:)
  complex,allocatable :: dataf(:,:,:)
  real,allocatable    :: bandp(:)
  real,allocatable    :: wfunc(:)
  complex,allocatable :: sensy(:)
  complex,allocatable :: tracf(:)

  real                :: la,f0,sg,w0
  
  real                :: eps

  ! For Tapers
  integer                         :: f1_ind,f2_ind,f3_ind,f4_ind,hind
  real                            :: f1,    f2,    f3,    f4
  real                            :: scaler
  real                            :: sigma  ! For Gaussian taper
  integer                         :: opt
  logical                         :: inv
  integer                         :: funit    ! 1: Hz      2: Radians

  real, parameter                 :: pi=3.1415926535
  logical                         :: verbose
  
  call sep_init()
  call from_param('verbose',verbose,.false.)
  call from_param('verb'   ,verbose,verbose)
  if (verbose) write(0,*) '-------------------------------------------'
  if (verbose) write(0,*) '            BANDPASS A TRACE               '
  if (verbose) write(0,*) '-------------------------------------------'
  if (verbose) write(0,*) '        i/o looping over traces            '

  
  call from_history('n1',n1,1 )
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  call from_history('n4',n4,1 )

  call from_history('d1',d1,1.)
  call from_history('d2',d2,1.)
  call from_history('d3',d3,1.)
  call from_history('d4',d4,1.)

  call from_history('o1',o1,1.)
  call from_history('o2',o2,1.)
  call from_history('o3',o3,1.)
  call from_history('o4',o4,1.)
  
  call from_param('eps',eps,0.0000001)

 
  nt = n1
  dt = d1
  nf = nt/2+1
  df = 1./(nt*dt)
  of = 0. 

  allocate(datat(nt,n2,n3)); datat=0.
!  allocate(dataf(nf,n2,n3)); dataf=0.
  allocate(tracf(nf)); tracf=0.
  allocate(wfunc(nf)); wfunc=0.
  allocate(sensy(nf)); sensy=0.
  allocate(bandp(nf)); bandp=0.
  
  ! Read input parameters for taper, there is lots of options :-)
  call from_param ("opt"  ,  opt,   7    )
  if (opt.lt.0..or.opt.gt.10) opt =  1
  call from_param ("inv"  , inv, .false. )
  call from_param ("sigma",sigma,   1.   )
  call from_param ("scale",scaler,  1.   )
  
  ! Read Tie points from param
  call from_param ("f1"    ,f1    , .15          )
  call from_param ("f1_ind",f1_ind, nint((f1-of)/df)+1)
  
  if (opt.eq.4 .or. opt.eq.7 .or. opt.eq.10) then
  call from_param ("f2"    ,f2    , .18)
  call from_param ("f2_ind",f2_ind, nint((f2-of)/df)+1)
  else
  call from_param ("f2"    ,f2    , 1.75)
  call from_param ("f2_ind",f2_ind, nint((f2-of)/df)+1)
  end if
  
  call from_param ("f3"    ,f3    , 1.5)
  call from_param ("f3_ind",f3_ind, nint((f3-of)/df)+1)
  
  call from_param ("f4"    ,f4    , 1.75)
  call from_param ("f4_ind",f4_ind, nint((f4-of)/df)+1)

  ! Make sure they fall within range
  if (f1_ind.lt.1.) f1_ind=1
  if (f2_ind.lt.1.) f2_ind=1
  if (f3_ind.lt.1.) f3_ind=1
  if (f4_ind.lt.1.) f4_ind=1
  if (f1_ind.gt.n1) f1_ind=n1
  if (f2_ind.gt.n1) f2_ind=n1
  if (f3_ind.gt.n1) f3_ind=n1
  if (f4_ind.gt.n1) f4_ind=n1

  if (verbose) then
    if (opt.eq.4 .or. opt.eq.7 .or. opt.eq.10) then
      if (verbose) write(0,"(A,F,X,F,X,F,X,F)") ' taper tie values : ',f1    ,f2    ,f3    ,f4
      if (verbose) write(0,"(A,I3.3,X,I3.3,X,I3.3,X,I3.3)") ' taper tie indices: ',f1_ind,f2_ind,f3_ind,f4_ind
    else
      if (verbose) write(0,"(A,F,X,F)") ' taper tie values : ',f1    ,f2
      if (verbose) write(0,"(A,I,X,I)") ' taper tie indices: ',f1_ind,f2_ind
    end if
  end if
  
  
  !!! TAPER CREATION
  select case(opt)

    case(0)
      if (verbose) write(0,*) 'taper: 0.'

    case(1)
      if (verbose) write(0,*) 'taper: rectangle'
      bandp(f1_ind:f2_ind)=1.

    case(2)
      if (verbose) write(0,*) 'taper: linear'
      bandp(1:f1_ind)=1.
      do ii=f1_ind,f2_ind
        bandp(ii)=1.-real(ii-f1_ind)/real(f2_ind-f1_ind)
      end do
      
    case(3)
      if (verbose) write(0,*) 'taper: pyramid'
      hind = nint(real(f2_ind-f1_ind)/2.)
      do ii=f1_ind,f1_ind+hind
        bandp(ii)=   real(ii-f1_ind     )/real(f1_ind+hind-f1_ind)
      end do
      do ii=f1_ind+hind,f2_ind
        bandp(ii)=1.-real(ii-f1_ind-hind)/real(f2_ind-f1_ind-hind)
      end do
      
    case(4)
      if (verbose) write(0,*) 'taper: pyramid-band'
      hind = nint(real(f2_ind-f1_ind)/2.)
      do ii=f1_ind,f2_ind
        bandp(ii)=   real(ii-f1_ind     )/real(f2_ind-f1_ind)
      end do
      bandp(f2_ind:f3_ind) = 1.
      do ii=f3_ind,f4_ind
        bandp(ii)=1.-real(ii-f3_ind     )/real(f4_ind-f3_ind)
      end do
      
    case(5)
      if (verbose) write(0,*) 'taper: hann'
      do ii=f1_ind,f2_ind
        bandp(ii)= .5*(1.-cos(2.*pi*(ii-f1_ind)/real(f2_ind-f1_ind)))
      end do
      
    case(6)
      if (verbose) write(0,*) 'taper: hann-half'
      bandp(1:f1_ind) = 1.
      do ii=f1_ind,f2_ind
        bandp(ii)= .5*(1.+cos(   pi*(ii-f1_ind)/real(f2_ind-f1_ind)))
      end do
      
    case(7)
      if (verbose) write(0,*) 'taper: hann-band'
      do ii=f1_ind,f2_ind
        bandp(ii)= .5*(1.-cos(pi*(ii-f1_ind)/real(f2_ind-f1_ind)))
      end do
      bandp(f2_ind:f3_ind) = 1.
      do ii=f3_ind,f4_ind
        bandp(ii)= .5*(1.+cos(pi*(ii-f3_ind)/real(f4_ind-f3_ind)))
      end do

    case(8)
      if (verbose) write(0,*) 'taper: gaussian'
      hind = nint(real(f2_ind-f1_ind))
      do ii=f1_ind,f2_ind
        bandp(ii)= exp(-(ii-hind)**2./(2*d1*sigma**2.) )
      end do
      
    case(9)
      if (verbose) write(0,*) 'taper: gaussian-half'
      bandp(1:f1_ind)=1.
      do ii=f1_ind,f2_ind
        bandp(ii)= exp(-((ii-f1_ind)**2.)/(2*d1*sigma**2.) )
      end do
      
    case(10)
      if (verbose) write(0,*) 'taper: gaussian-band'
      do ii=f1_ind,f2_ind
        bandp(ii)= exp(-((ii-f2_ind)**2.)/(2*d1*sigma**2.) )
      end do
      bandp(f2_ind:f3_ind) = 1.
      do ii=f3_ind,f4_ind
        bandp(ii)= exp(-((ii-f3_ind)**2.)/(2*d1*sigma**2.) )
      end do
  
  end select
  
  ! Invert taper to lowpass or notch
  if (inv) bandp = 1.-bandp
  
  ! Taper as output
  if (exist_file("taper")) then
    if (verbose) write(0,*) 'write taper to file'
    call to_history("n1", nf ,"taper")
    call to_history("d1", df ,"taper")
    call to_history("o1", of ,"taper")
    call srite("taper",bandp,4*nf)
  end if

  call sep_close()
  
  ! Combination of mV->V's and lacking dt's and factor of 2 in correlations:
  factor = 2./1000./1000./sqrt(18000.) 
  

  ! Prepare expected amplitude versus frequency behavior of earth's impulse response function
  do ii=1,nf
    wfunc(ii) = (ii-1)*df*sqrt((ii-1)*df)
  end do

  
  ! Prepare geophone sensitivity
  la = 68
  sg = 100*.220
  f0 = 15
  w0 = 2*pi*f0
  do ii=1,nf
    f  = (ii-1)*df
    w  = 2*pi*f
    sensy(ii) = sg*(w**2./w0**2.)/((-w**2./w0**2.)+cmplx(1,2*la/100.*w/w0))
  end do
  
  do ii=1,nf
    sensy(ii) = bandp(ii)/cabs(sensy(ii))**2.
  end do
  sensy(1) = 0.
  sens_ave = sum(sensy) / sum(bandp)
  write(0,*) 'sens_ave',sens_ave

!  ! Prepare Fourier transformation
!  call fftw1d_1_r2c_lop_init(nt_in=nt)
  
  ! Read  the data
  do kk=1,n3
    call sreed("in" ,datat(:,:,kk),4*n1*n2)
  end do

  ! Apply scalar factor
  datat = factor*datat*sens_ave
  
!  ! Forward Fourier Transformation
!  call fftw1d_1_r2c_lop(.false.,.false.,datat,dataf);
!  dataf = dt*dataf*sqrt(real(nt))

!  do kk=1,n3; do jj=1,n2
!    
!    tracf = 0.
!    tracf = dataf(:,jj,kk)
!    
!    ! Get rid of sensitivity
!    if ( sum(abs(tracf)).ne.0.) then
!      ave = sum(cabs(tracf)*sensy)/sum(abs(tracf))
!    else
!      ave = 0.
!    end if
!    write(0,*) 'sens_ave'
!    write(0,*)  sens_ave
!    write(0,*) 'sum(abs(tracf))'
!    write(0,*)  sum(abs(tracf))
!   
!    ! Determine spectral average
!    dataf(:,jj,kk) = sens_ave*dataf(:,jj,kk)
!    
!  end do;     end do
  
!  ! Inverse Fourier Transformation
!  call fftw1d_1_r2c_lop(.true. ,.false.,datat,dataf);
!  datat = datat/dt/sqrt(real(nt));
  
  ! Write the data
  do kk=1,n3
    call srite("out" ,datat(:,:,kk),4*n1*n2)
  end do

  deallocate(datat)
!  deallocate(dataf)
  deallocate(tracf)
  deallocate(wfunc)
  deallocate(sensy)
  deallocate(bandp)
  call exit()

end program
