module reshapes_mod
 
  implicit none

  private
  public :: conv1dto2d,conv2dto1d,conv1dto3d,conv3dto1d
 

contains
 
  ! convert 1D array to 2D array
  subroutine conv1dto2d(data1,data2,n1,n2)
    real, dimension(:)  ,intent(in)  :: data1
    real, dimension(:,:),intent(out) :: data2
    integer              :: ii,jj,ij,n1,n2
    do jj=1,n2
      do ii=1,n1
        ij= ii+(jj-1)*n1
        data2(ii,jj)=data2(ii,jj)+data1(ij)
      enddo
    enddo
  end subroutine
  
  ! convert 2D array to 1D array
  subroutine conv2dto1d(data2,data1,n1,n2)
    real, dimension(:,:),intent(in)  :: data2
    real, dimension(:)  ,intent(out) :: data1
    integer              :: ii,jj,ij,n1,n2
    do jj=1,n2
      do ii=1,n1
        ij= ii+(jj-1)*n1
        data1(ij)=data1(ij)+data2(ii,jj)
      enddo
    enddo
  end subroutine
  
  ! convert 1D array to 3D array
  subroutine conv1dto3d(data1,data3,n1,n2,n3)
    real, dimension(:)  ,intent(in)  :: data1
    real, dimension(:,:,:),intent(out) :: data3
    integer              :: ii,jj,kk,ijk,n1,n2,n3
    do kk=1,n3
     do jj=1,n2
      do ii=1,n1
        ijk=ii+(jj-1)*n1+(kk-1)*n1*n2
        data3(ii,jj,kk)=data3(ii,jj,kk)+data1(ijk)
      enddo
     enddo
    enddo
  end subroutine

  ! convert 3D array to 1D array
  subroutine conv3dto1d(data3,data1,n1,n2,n3)
    real, dimension(:,:,:),intent(in)  :: data3
    real, dimension(:)  ,intent(out) :: data1
    integer              :: ii,jj,kk,ijk,n1,n2,n3
    do kk=1,n3
     do jj=1,n2
      do ii=1,n1
        ijk=ii+(jj-1)*n1+(kk-1)*n1*n2
        data1(ijk)=data1(ijk)+data3(ii,jj,kk)
      enddo
     enddo
    enddo
  end subroutine

end module
