program sdrCylinTrans
  ! Program to transform three componxyt data into cylindrical coordinates,
  ! With origin at source location

  ! Parameter: regular (logical) determines if the grid is regular, and if
  ! the 4th and 5 dimentions are taken as the source locations, else, nyyd
  ! header file: head with as many traces as n2 (for x,y and z). and
  ! paremeters rx_key,ry_key,sx_key and sy_key
  
  use sep
  implicit none
  
  integer            :: n1,  nyy1,nyx1,nyz1,  nxy1,nxx1,nxz1,  nzy1,nzx1,nzz1
  integer            :: n2,  nyy2,nyx2,nyz2,  nxy2,nxx2,nxz2,  nzy2,nzx2,nzz2
  integer            :: n3,  nyy3,nyx3,nyz3,  nxy3,nxx3,nxz3,  nzy3,nzx3,nzz3
  integer            :: n4,  nyy4,nyx4,nyz4,  nxy4,nxx4,nxz4,  nzy4,nzx4,nzz4
  integer            :: n5,  nyy5,nyx5,nyz5,  nxy5,nxx5,nxz5,  nzy5,nzx5,nzz5
  
  real               :: o1,  oyy1,oyx1,oyz1,  oxy1,oxx1,oxz1,  ozy1,ozx1,ozz1
  real               :: o2,  oyy2,oyx2,oyz2,  oxy2,oxx2,oxz2,  ozy2,ozx2,ozz2
  real               :: o3,  oyy3,oyx3,oyz3,  oxy3,oxx3,oxz3,  ozy3,ozx3,ozz3
  real               :: o4,  oyy4,oyx4,oyz4,  oxy4,oxx4,oxz4,  ozy4,ozx4,ozz4
  real               :: o5,  oyy5,oyx5,oyz5,  oxy5,oxx5,oxz5,  ozy5,ozx5,ozz5
  
  real               :: d1,  dyy1,dyx1,dyz1,  dxy1,dxx1,dxz1,  dzy1,dzx1,dzz1
  real               :: d2,  dyy2,dyx2,dyz2,  dxy2,dxx2,dxz2,  dzy2,dzx2,dzz2
  real               :: d3,  dyy3,dyx3,dyz3,  dxy3,dxx3,dxz3,  dzy3,dzx3,dzz3
  real               :: d4,  dyy4,dyx4,dyz4,  dxy4,dxx4,dxz4,  dzy4,dzx4,dzz4
  real               :: d5,  dyy5,dyx5,dyz5,  dxy5,dxx5,dxz5,  dzy5,dzx5,dzz5
  
  character*50       :: label1,label2,label3,label4,label5
  
  logical            :: regular
  integer            :: n1hdr,n2hdr,n3hdr,n4hdr,n5hdr
  
  integer            :: rx,ry,sx,sy,ind   ! Counters
  
  real               :: angle,delx,dely
  integer            :: rx_key,ry_key,sx_key,sy_key
  
  real, allocatable  :: datayy(:),datayx(:),datayz(:)
  real, allocatable  :: dataxy(:),dataxx(:),dataxz(:)
  real, allocatable  :: datazy(:),datazx(:),datazz(:)

  real, allocatable  :: datayr(:),datayt(:),datayv(:)
  real, allocatable  :: dataxr(:),dataxt(:),dataxv(:)
  real, allocatable  :: datazr(:),datazt(:),datazv(:)

  real, allocatable  :: datarr(:),datart(:),datarv(:)
  real, allocatable  :: datatr(:),datatt(:),datatv(:)
  real, allocatable  :: datavr(:),datavt(:),datavv(:)

  real, allocatable  :: head(:,:)
  real               :: M(3,3)

  logical            :: kill

  real,parameter     :: pi=3.14159265359

  call sep_init()

  ! *********************************************************************
  !  Read Grid
  ! *********************************************************************
  write(0,*) '------------------------'
  write(0,*) 'Retrieve geometry'

  ! Geometry of the experiments
  if (exist_file("yy")) then; 
   call from_aux("yy","n1",nyy1,1 )
   call from_aux("yy","n2",nyy2,1 )
   call from_aux("yy","n3",nyy3,1 )
   call from_aux("yy","n4",nyy4,1 )
   call from_aux("yy","n5",nyy5,1 )
   call from_aux("yy","d1",dyy1,1.)
   call from_aux("yy","d2",dyy2,1.)
   call from_aux("yy","d3",dyy3,1.)
   call from_aux("yy","d4",dyy4,1.)
   call from_aux("yy","d5",dyy5,1.)
   call from_aux("yy","o1",oyy1,0.)
   call from_aux("yy","o2",oyy2,0.)
   call from_aux("yy","o3",oyy3,0.)
   call from_aux("yy","o4",oyy4,0.)
   call from_aux("yy","o5",oyy5,0.)
   call from_aux("yy","label1",label1," ")
   call from_aux("yy","label2",label2," ")
   call from_aux("yy","label3",label3," ")
   call from_aux("yy","label4",label4," ")
   call from_aux("yy","label5",label5," ")
  end if
  
  if (exist_file("yx")) then; 
   call from_aux("yx","n1",nyx1,1 )
   call from_aux("yx","n2",nyx2,1 )
   call from_aux("yx","n3",nyx3,1 )
   call from_aux("yx","n4",nyx4,1 )
   call from_aux("yx","n5",nyx5,1 )
   call from_aux("yx","d1",dyx1,1.)
   call from_aux("yx","d2",dyx2,1.)
   call from_aux("yx","d3",dyx3,1.)
   call from_aux("yx","d4",dyx4,1.)
   call from_aux("yx","d5",dyx5,1.)
   call from_aux("yx","o1",oyx1,0.)
   call from_aux("yx","o2",oyx2,0.)
   call from_aux("yx","o3",oyx3,0.)
   call from_aux("yx","o4",oyx4,0.)
   call from_aux("yx","o5",oyx5,0.)
   call from_aux("yx","label1",label1,label1)
   call from_aux("yx","label2",label2,label2)
   call from_aux("yx","label3",label3,label3)
   call from_aux("yx","label4",label4,label4)
   call from_aux("yx","label5",label5,label5)
  else;
   nyx1 = nyy1; dyx1 = dyy1; oyx1 = oyy1; ! nyyded for error check only
   nyx2 = nyy1; dyx2 = dyy2; oyx2 = oyy2; ! nyyded for error check only
   nyx3 = nyy1; dyx3 = dyy3; oyx3 = oyy3; ! nyyded for error check only
   nyx4 = nyy1; dyx4 = dyy4; oyx4 = oyy4; ! nyyded for error check only
   nyx5 = nyy1; dyx5 = dyy5; oyx5 = oyy5; ! nyyded for error check only
  end if



  if (exist_file("yz")) then;
   call from_aux("yz","n1",nyz1,1 )
   call from_aux("yz","n2",nyz2,1 )
   call from_aux("yz","n3",nyz3,1 )
   call from_aux("yz","n4",nyz4,1 )
   call from_aux("yz","n5",nyz5,1 )
   call from_aux("yz","d1",dyz1,1.)
   call from_aux("yz","d2",dyz2,1.)
   call from_aux("yz","d3",dyz3,1.)
   call from_aux("yz","d4",dyz4,1.)
   call from_aux("yz","d5",dyz5,1.)
   call from_aux("yz","o1",oyz1,0.)
   call from_aux("yz","o2",oyz2,0.)
   call from_aux("yz","o3",oyz3,0.)
   call from_aux("yz","o4",oyz4,0.)
   call from_aux("yz","o5",oyz5,0.)
   call from_aux("yz","label1",label1,label1)
   call from_aux("yz","label2",label2,label2)
   call from_aux("yz","label3",label3,label3)
   call from_aux("yz","label4",label4,label4)
   call from_aux("yz","label5",label5,label5)
  else;
   nyz1 = nyy1; dyz1 = dyy1; oyz1 = oyy1; ! nyyded for error check only
   nyz2 = nyy1; dyz2 = dyy2; oyz2 = oyy2; ! nyyded for error check only
   nyz3 = nyy1; dyz3 = dyy3; oyz3 = oyy3; ! nyyded for error check only
   nyz4 = nyy1; dyz4 = dyy4; oyz4 = oyy4; ! nyyded for error check only
   nyz5 = nyy1; dyz5 = dyy5; oyz5 = oyy5; ! nyyded for error check only
  end if
  
  
  
  
  if (exist_file("xy")) then; 
   call from_aux("xy","n1",nxy1,1 )
   call from_aux("xy","n2",nxy2,1 )
   call from_aux("xy","n3",nxy3,1 )
   call from_aux("xy","n4",nxy4,1 )
   call from_aux("xy","n5",nxy5,1 )
   call from_aux("xy","d1",dxy1,1.)
   call from_aux("xy","d2",dxy2,1.)
   call from_aux("xy","d3",dxy3,1.)
   call from_aux("xy","d4",dxy4,1.)
   call from_aux("xy","d5",dxy5,1.)
   call from_aux("xy","o1",oxy1,0.)
   call from_aux("xy","o2",oxy2,0.)
   call from_aux("xy","o3",oxy3,0.)
   call from_aux("xy","o4",oxy4,0.)
   call from_aux("xy","o5",oxy5,0.)
   call from_aux("xy","label1",label1,label1)
   call from_aux("xy","label2",label2,label2)
   call from_aux("xy","label3",label3,label3)
   call from_aux("xy","label4",label4,label4)
   call from_aux("xy","label5",label5,label5)
  else;
   nxy1 = nyy1; dxy1 = dyy1; oyx1 = oyy1; ! nyyded for error check only
   nxy2 = nyy1; dxy2 = dyy2; oyx2 = oyy2; ! nyyded for error check only
   nxy3 = nyy1; dxy3 = dyy3; oyx3 = oyy3; ! nyyded for error check only
   nxy4 = nyy1; dxy4 = dyy4; oyx4 = oyy4; ! nyyded for error check only
   nxy5 = nyy1; dxy5 = dyy5; oyx5 = oyy5; ! nyyded for error check only
  end if





  if (exist_file("xx")) then; 
   call from_aux("xx","n1",nxx1,1 )
   call from_aux("xx","n2",nxx2,1 )
   call from_aux("xx","n3",nxx3,1 )
   call from_aux("xx","n4",nxx4,1 )
   call from_aux("xx","n5",nxx5,1 )
   call from_aux("xx","d1",dxx1,1.)
   call from_aux("xx","d2",dxx2,1.)
   call from_aux("xx","d3",dxx3,1.)
   call from_aux("xx","d4",dxx4,1.)
   call from_aux("xx","d5",dxx5,1.)
   call from_aux("xx","o1",oxx1,0.)
   call from_aux("xx","o2",oxx2,0.)
   call from_aux("xx","o3",oxx3,0.)
   call from_aux("xx","o4",oxx4,0.)
   call from_aux("xx","o5",oxx5,0.)
   call from_aux("xx","label1",label1,label1)
   call from_aux("xx","label2",label2,label2)
   call from_aux("xx","label3",label3,label3)
   call from_aux("xx","label4",label4,label4)
   call from_aux("xx","label5",label5,label5)
  else;
   nyy1 = nyy1; dxx1 = dyy1; oxx1 = oyy1; ! nyyded for error check only
   nyy2 = nyy1; dxx2 = dyy2; oxx2 = oyy2; ! nyyded for error check only
   nyy3 = nyy1; dxx3 = dyy3; oxx3 = oyy3; ! nyyded for error check only
   nyy4 = nyy1; dxx4 = dyy4; oxx4 = oyy4; ! nyyded for error check only
   nyy5 = nyy1; dxx5 = dyy5; oxx5 = oyy5; ! nyyded for error check only
  end if




  if (exist_file("xz")) then
   call from_aux("xz","n1",nxz1,1 )
   call from_aux("xz","n2",nxz2,1 )
   call from_aux("xz","n3",nxz3,1 )
   call from_aux("xz","n4",nxz4,1 )
   call from_aux("xz","n5",nxz5,1 )
   call from_aux("xz","d1",dxz1,1.)
   call from_aux("xz","d2",dxz2,1.)
   call from_aux("xz","d3",dxz3,1.)
   call from_aux("xz","d4",dxz4,1.)
   call from_aux("xz","d5",dxz5,1.)
   call from_aux("xz","o1",oxz1,0.)
   call from_aux("xz","o2",oxz2,0.)
   call from_aux("xz","o3",oxz3,0.)
   call from_aux("xz","o4",oxz4,0.)
   call from_aux("xz","o5",oxz5,0.)
   call from_aux("xz","label1",label1,label1)
   call from_aux("xz","label2",label2,label2)
   call from_aux("xz","label3",label3,label3)
   call from_aux("xz","label4",label4,label4)
   call from_aux("xz","label5",label5,label5)
  else
   nxz1 = nyy1; dxz1 = dyy1; oxz1 = oyy1; ! nyyded for error check only
   nxz2 = nyy1; dxz2 = dyy2; oxz2 = oyy2; ! nyyded for error check only
   nxz3 = nyy1; dxz3 = dyy3; oxz3 = oyy3; ! nyyded for error check only
   nxz4 = nyy1; dxz4 = dyy4; oxz4 = oyy4; ! nyyded for error check only
   nxz5 = nyy1; dxz5 = dyy5; oxz5 = oyy5; ! nyyded for error check only
  end if







  if (exist_file("zy")) then   
   call from_aux("zy","n1",nzy1,1 )
   call from_aux("zy","n2",nzy2,1 )
   call from_aux("zy","n3",nzy3,1 )
   call from_aux("zy","n4",nzy4,1 )
   call from_aux("zy","n5",nzy5,1 )
   call from_aux("zy","d1",dzy1,1.)
   call from_aux("zy","d2",dzy2,1.)
   call from_aux("zy","d3",dzy3,1.)
   call from_aux("zy","d4",dzy4,1.)
   call from_aux("zy","d5",dzy5,1.)
   call from_aux("zy","o1",ozy1,0.)
   call from_aux("zy","o2",ozy2,0.)
   call from_aux("zy","o3",ozy3,0.)
   call from_aux("zy","o4",ozy4,0.)
   call from_aux("zy","o5",ozy5,0.)
   call from_aux("zy","label1",label1,label1)
   call from_aux("zy","label2",label2,label2)
   call from_aux("zy","label3",label3,label3)
   call from_aux("zy","label4",label4,label4)
   call from_aux("zy","label5",label5,label5)
  else
   nzy1 = nyy1; dzy1 = dyy1; ozy1 = oyy1; ! nyyded for error check only
   nzy2 = nyy2; dzy2 = dyy2; ozy2 = oyy2; ! nyyded for error check only
   nzy3 = nyy3; dzy3 = dyy3; ozy3 = oyy3; ! nyyded for error check only
   nzy4 = nyy4; dzy4 = dyy4; ozy4 = oyy4; ! nyyded for error check only
   nzy5 = nyy5; dzy5 = dyy5; ozy5 = oyy5; ! nyyded for error check only
  end if

  if (exist_file("zx")) then   
   call from_aux("zx","n1",nzx1,1 )
   call from_aux("zx","n2",nzx2,1 )
   call from_aux("zx","n3",nzx3,1 )
   call from_aux("zx","n4",nzx4,1 )
   call from_aux("zx","n5",nzx5,1 )
   call from_aux("zx","d1",dzx1,1.)
   call from_aux("zx","d2",dzx2,1.)
   call from_aux("zx","d3",dzx3,1.)
   call from_aux("zx","d4",dzx4,1.)
   call from_aux("zx","d5",dzx5,1.)
   call from_aux("zx","o1",ozx1,0.)
   call from_aux("zx","o2",ozx2,0.)
   call from_aux("zx","o3",ozx3,0.)
   call from_aux("zx","o4",ozx4,0.)
   call from_aux("zx","o5",ozx5,0.)
   call from_aux("zx","label1",label1,label1)
   call from_aux("zx","label2",label2,label2)
   call from_aux("zx","label3",label3,label3)
   call from_aux("zx","label4",label4,label4)
   call from_aux("zx","label5",label5,label5)
  else
   nzx1 = nyy1; dzx1 = dyy1; ozx1 = oyy1; ! nyyded for error check only
   nzx2 = nyy2; dzx2 = dyy2; ozx2 = oyy2; ! nyyded for error check only
   nzx3 = nyy3; dzx3 = dyy3; ozx3 = oyy3; ! nyyded for error check only
   nzx4 = nyy4; dzx4 = dyy4; ozx4 = oyy4; ! nyyded for error check only
   nzx5 = nyy5; dzx5 = dyy5; ozx5 = oyy5; ! nyyded for error check only
  end if

  if (exist_file("zz")) then   
   call from_aux("zz","n1",nzz1,1 )
   call from_aux("zz","n2",nzz2,1 )
   call from_aux("zz","n3",nzz3,1 )
   call from_aux("zz","n4",nzz4,1 )
   call from_aux("zz","n5",nzz5,1 )
   call from_aux("zz","d1",dzz1,1.)
   call from_aux("zz","d2",dzz2,1.)
   call from_aux("zz","d3",dzz3,1.)
   call from_aux("zz","d4",dzz4,1.)
   call from_aux("zz","d5",dzz5,1.)
   call from_aux("zz","o1",ozz1,0.)
   call from_aux("zz","o2",ozz2,0.)
   call from_aux("zz","o3",ozz3,0.)
   call from_aux("zz","o4",ozz4,0.)
   call from_aux("zz","o5",ozz5,0.)
   call from_aux("zz","label1",label1,label1)
   call from_aux("zz","label2",label2,label2)
   call from_aux("zz","label3",label3,label3)
   call from_aux("zz","label4",label4,label4)
   call from_aux("zz","label5",label5,label5)
  else
   nzz1 = nyy1; dzz1 = dyy1; ozz1 = oyy1; ! nyyded for error check only
   nzz2 = nyy2; dzz2 = dyy2; ozz2 = oyy2; ! nyyded for error check only
   nzz3 = nyy3; dzz3 = dyy3; ozz3 = oyy3; ! nyyded for error check only
   nzz4 = nyy4; dzz4 = dyy4; ozz4 = oyy4; ! nyyded for error check only
   nzz5 = nyy5; dzz5 = dyy5; ozz5 = oyy5; ! nyyded for error check only
  end if

  
  
  ! Check if receiver grid of these receivers is complete and consistent
  if (.not. exist_file("yy")) then; write(0,*) 'no nn-componxyt file'; end if
  if (.not. exist_file("yx")) then; write(0,*) 'no ne-componxyt file'; end if
  if (.not. exist_file("yz")) then; write(0,*) 'no nz-componxyt file'; end if
  
  if (.not. exist_file("xy")) then; write(0,*) 'no en-componxyt file'; end if
  if (.not. exist_file("xx")) then; write(0,*) 'no ee-componxyt file'; end if
  if (.not. exist_file("xz")) then; write(0,*) 'no ez-componxyt file'; end if
   
  if (.not. exist_file("zy")) then; write(0,*) 'no zn-componxyt file'; end if
  if (.not. exist_file("zx")) then; write(0,*) 'no ze-componxyt file'; end if
  if (.not. exist_file("zz")) then; write(0,*) 'no zz-componxyt file'; end if

  n1=nyy1; n2=nyy2;  n3=nyy3;  n4=nyy4;  n5=nyy5;
  d1=dyy1; d2=dyy2;  d3=dyy3;  d4=dyy4;  d5=dyy5;
  o1=oyy1; o2=oyy2;  o3=oyy3;  o4=oyy4;  o5=oyy5;

  if (n1.ne.nyy1 .or. n1.ne.nyx1 .or. n1.ne.nyz1 .or. n1.ne.nxy1.or. n1.ne.nxx1.or. n1.ne.nxz1.or. n1.ne.nzy1.or. n1.ne.nzx1.or. n1.ne.nzz1) then
    write(0,*) 'n1 not consistent';  call exit(0);   end if
  if (n2.ne.nyy2 .or. n2.ne.nyx2 .or. n2.ne.nyz2 .or. n2.ne.nxy2.or. n2.ne.nxx2.or. n2.ne.nxz2.or. n2.ne.nzy2.or. n2.ne.nzx2.or. n2.ne.nzz2) then
    write(0,*) 'n1 not consistent';  call exit(0);   end if
  if (n3.ne.nyy3 .or. n3.ne.nyx3 .or. n3.ne.nyz3 .or. n3.ne.nxy3.or. n3.ne.nxx3.or. n3.ne.nxz3.or. n3.ne.nzy3.or. n3.ne.nzx3.or. n3.ne.nzz3) then
    write(0,*) 'n1 not consistent';  call exit(0);   end if
  if (n4.ne.nyy4 .or. n4.ne.nyx4 .or. n4.ne.nyz4 .or. n4.ne.nxy4.or. n4.ne.nxx4.or. n4.ne.nxz4.or. n4.ne.nzy4.or. n4.ne.nzx4.or. n4.ne.nzz4) then
    write(0,*) 'n1 not consistent';  call exit(0);   end if
  if (n5.ne.nyy5 .or. n5.ne.nyx5 .or. n5.ne.nyz5 .or. n5.ne.nxy5.or. n5.ne.nxx5.or. n5.ne.nxz5.or. n5.ne.nzy5.or. n5.ne.nzx5.or. n5.ne.nzz5) then
    write(0,*) 'n1 not consistent';  call exit(0);   end if
  
  regular=.false.  
  call from_param('regular',regular,regular)
  
  rx_key = 1;   ry_key = 2
  sx_key = 3;   sy_key = 4
  if (.not.regular) then
    ! Read header of irregular grid
    call from_history("n1",n1hdr,1)
    call from_history("n2",n2hdr,1)
    call from_history("n3",n3hdr,1)
    call from_history("n4",n4hdr,1)
    call from_history("n5",n5hdr,1)
!    write(0,*) n2,n3,n4,n5
!    write(0,*) n2hdr,n3hdr,n4hdr,n5hdr
    if (n2hdr*n3hdr*n4hdr*n5hdr.ne.n2*n3*n4*n5) then; call erexit("n2hdr not equal to nr of traces"); end if
    allocate(head(n1hdr,n2hdr*n3hdr*n4hdr*n5hdr)); head=0.
    call sreed("in",head,4*size(head))
    call from_param("rx_key",rx_key,rx_key)
    call from_param("ry_key",ry_key,ry_key)
    call from_param("sx_key",sx_key,sx_key)
    call from_param("sy_key",sy_key,sy_key)
  else
    ! Create header for regular grid
    n2hdr = n2*n3*n4*n5
    allocate(head(4,n2hdr))
    do sy=1,n5
    do sx=1,n4
    do ry=1,n3
    do rx=1,n2
      ind = (sy-1)*n4*n3*n2+(sx-1)*n3*n2+(ry-1)*n2+rx
      head(1,ind) = o2+(rx-1)*d2
      head(2,ind) = o3+(ry-1)*d3
      head(3,ind) = o4+(sx-1)*d4
      head(4,ind) = o5+(sy-1)*d5
    end do
    end do
    end do
    end do
  end if
  
  ! Create output headers
  if (exist_file("rr")) then;
    call to_history("n1",n1,"rr")
    call to_history("n2",n2,"rr")
    call to_history("n3",n3,"rr")
    call to_history("n4",n4,"rr")
    call to_history("n5",n5,"rr")

    call to_history("d1",d1,"rr")
    call to_history("d2",d2,"rr")
    call to_history("d3",d3,"rr")
    call to_history("d4",d4,"rr")
    call to_history("d5",d5,"rr")

    call to_history("o1",o1,"rr")
    call to_history("o2",o2,"rr")
    call to_history("o3",o3,"rr")
    call to_history("o4",o4,"rr")
    call to_history("o5",o5,"rr")

    call to_history("label1",label1,"rr")
    call to_history("label2",label2,"rr")
    call to_history("label3",label3,"rr")
    call to_history("label4",label4,"rr")
    call to_history("label5",label5,"rr")
  end if
  if (exist_file("rt")) then;
    call to_history("n1",n1,"rt")
    call to_history("n2",n2,"rt")
    call to_history("n3",n3,"rt")
    call to_history("n4",n4,"rt")
    call to_history("n5",n5,"rt")

    call to_history("d1",d1,"rt")
    call to_history("d2",d2,"rt")
    call to_history("d3",d3,"rt")
    call to_history("d4",d4,"rt")
    call to_history("d5",d5,"rt")

    call to_history("o1",o1,"rt")
    call to_history("o2",o2,"rt")
    call to_history("o3",o3,"rt")
    call to_history("o4",o4,"rt")
    call to_history("o5",o5,"rt")

    call to_history("label1",label1,"rt")
    call to_history("label2",label2,"rt")
    call to_history("label3",label3,"rt")
    call to_history("label4",label4,"rt")
    call to_history("label5",label5,"rt")
  end if
  if (exist_file("rv")) then;
    call to_history("n1",n1,"rv")
    call to_history("n2",n2,"rv")
    call to_history("n3",n3,"rv")
    call to_history("n4",n4,"rv")
    call to_history("n5",n5,"rv")

    call to_history("d1",d1,"rv")
    call to_history("d2",d2,"rv")
    call to_history("d3",d3,"rv")
    call to_history("d4",d4,"rv")
    call to_history("d5",d5,"rv")

    call to_history("o1",o1,"rv")
    call to_history("o2",o2,"rv")
    call to_history("o3",o3,"rv")
    call to_history("o4",o4,"rv")
    call to_history("o5",o5,"rv")

    call to_history("label1",label1,"rv")
    call to_history("label2",label2,"rv")
    call to_history("label3",label3,"rv")
    call to_history("label4",label4,"rv")
    call to_history("label5",label5,"rv")
  end if
  
  
  
  
  
  
  if (exist_file("tr")) then;
    call to_history("n1",n1,"tr")
    call to_history("n2",n2,"tr")
    call to_history("n3",n3,"tr")
    call to_history("n4",n4,"tr")
    call to_history("n5",n5,"tr")

    call to_history("d1",d1,"tr")
    call to_history("d2",d2,"tr")
    call to_history("d3",d3,"tr")
    call to_history("d4",d4,"tr")
    call to_history("d5",d5,"tr")

    call to_history("o1",o1,"tr")
    call to_history("o2",o2,"tr")
    call to_history("o3",o3,"tr")
    call to_history("o4",o4,"tr")
    call to_history("o5",o5,"tr")

    call to_history("label1",label1,"tr")
    call to_history("label2",label2,"tr")
    call to_history("label3",label3,"tr")
    call to_history("label4",label4,"tr")
    call to_history("label5",label5,"tr")
  end if
  if (exist_file("tt")) then;
    call to_history("n1",n1,"tt")
    call to_history("n2",n2,"tt")
    call to_history("n3",n3,"tt")
    call to_history("n4",n4,"tt")
    call to_history("n5",n5,"tt")

    call to_history("d1",d1,"tt")
    call to_history("d2",d2,"tt")
    call to_history("d3",d3,"tt")
    call to_history("d4",d4,"tt")
    call to_history("d5",d5,"tt")

    call to_history("o1",o1,"tt")
    call to_history("o2",o2,"tt")
    call to_history("o3",o3,"tt")
    call to_history("o4",o4,"tt")
    call to_history("o5",o5,"tt")

    call to_history("label1",label1,"tt")
    call to_history("label2",label2,"tt")
    call to_history("label3",label3,"tt")
    call to_history("label4",label4,"tt")
    call to_history("label5",label5,"tt")
  end if

  if (exist_file("tv")) then;
    call to_history("n1",n1,"tv")
    call to_history("n2",n2,"tv")
    call to_history("n3",n3,"tv")
    call to_history("n4",n4,"tv")
    call to_history("n5",n5,"tv")

    call to_history("d1",d1,"tv")
    call to_history("d2",d2,"tv")
    call to_history("d3",d3,"tv")
    call to_history("d4",d4,"tv")
    call to_history("d5",d5,"tv")

    call to_history("o1",o1,"tv")
    call to_history("o2",o2,"tv")
    call to_history("o3",o3,"tv")
    call to_history("o4",o4,"tv")
    call to_history("o5",o5,"tv")

    call to_history("label1",label1,"tv")
    call to_history("label2",label2,"tv")
    call to_history("label3",label3,"tv")
    call to_history("label4",label4,"tv")
    call to_history("label5",label5,"tv")
  end if


  if (exist_file("vr")) then;
    call to_history("n1",n1,"vr")
    call to_history("n2",n2,"vr")
    call to_history("n3",n3,"vr")
    call to_history("n4",n4,"vr")
    call to_history("n5",n5,"vr")

    call to_history("d1",d1,"vr")
    call to_history("d2",d2,"vr")
    call to_history("d3",d3,"vr")
    call to_history("d4",d4,"vr")
    call to_history("d5",d5,"vr")

    call to_history("o1",o1,"vr")
    call to_history("o2",o2,"vr")
    call to_history("o3",o3,"vr")
    call to_history("o4",o4,"vr")
    call to_history("o5",o5,"vr")

    call to_history("label1",label1,"vr")
    call to_history("label2",label2,"vr")
    call to_history("label3",label3,"vr")
    call to_history("label4",label4,"vr")
    call to_history("label5",label5,"vr")
  end if
  if (exist_file("vt")) then;
    call to_history("n1",n1,"vt")
    call to_history("n2",n2,"vt")
    call to_history("n3",n3,"vt")
    call to_history("n4",n4,"vt")
    call to_history("n5",n5,"vt")

    call to_history("d1",d1,"vt")
    call to_history("d2",d2,"vt")
    call to_history("d3",d3,"vt")
    call to_history("d4",d4,"vt")
    call to_history("d5",d5,"vt")

    call to_history("o1",o1,"vt")
    call to_history("o2",o2,"vt")
    call to_history("o3",o3,"vt")
    call to_history("o4",o4,"vt")
    call to_history("o5",o5,"vt")

    call to_history("label1",label1,"vt")
    call to_history("label2",label2,"vt")
    call to_history("label3",label3,"vt")
    call to_history("label4",label4,"vt")
    call to_history("label5",label5,"vt")
  end if
  if (exist_file("vv")) then;
    call to_history("n1",n1,"vv")
    call to_history("n2",n2,"vv")
    call to_history("n3",n3,"vv")
    call to_history("n4",n4,"vv")
    call to_history("n5",n5,"vv")

    call to_history("d1",d1,"vv")
    call to_history("d2",d2,"vv")
    call to_history("d3",d3,"vv")
    call to_history("d4",d4,"vv")
    call to_history("d5",d5,"vv")

    call to_history("o1",o1,"vv")
    call to_history("o2",o2,"vv")
    call to_history("o3",o3,"vv")
    call to_history("o4",o4,"vv")
    call to_history("o5",o5,"vv")

    call to_history("label1",label1,"vv")
    call to_history("label2",label2,"vv")
    call to_history("label3",label3,"vv")
    call to_history("label4",label4,"vv")
    call to_history("label5",label5,"vv")
  end if



  call sep_close()
  
  allocate( datayy(n1) , datayx(n1) , datayz(n1) )
  allocate( dataxy(n1) , dataxx(n1) , dataxz(n1) )
  allocate( datazy(n1) , datazx(n1) , datazz(n1) )

  allocate( datayr(n1) , datayt(n1) , datayv(n1) )
  allocate( dataxr(n1) , dataxt(n1) , dataxv(n1) )
  allocate( datazr(n1) , datazt(n1) , datazv(n1) )

  allocate( datarr(n1) , datart(n1) , datarv(n1) )
  allocate( datatr(n1) , datatt(n1) , datatv(n1) )
  allocate( datavr(n1) , datavt(n1) , datavv(n1) )
  
  
  ! *********************************************************************
  !  Rotations
  ! *********************************************************************
  write(0,*) '------------------------'
  write(0,*) 'Actual rotations'

  do sy=1,n5
  do sx=1,n4

  do ry=1,n3
  do rx=1,n2
  ind  = (sy-1)*n4*n3*n2+(sx-1)*n3*n2+(ry-1)*n2+rx
  kill = .false.
  
  ! Zero out traces
  datayy=0.; datayx=0.; datayz=0.
  dataxy=0.; dataxx=0.; dataxz=0.
  datazy=0.; datazx=0.; datazz=0.

  datayr=0.; datayt=0.; datayv=0.
  dataxr=0.; dataxt=0.; dataxv=0.
  datazr=0.; datazt=0.; datavv=0.

  datarr=0.; datart=0.; datarv=0.
  datatr=0.; datatt=0.; datatv=0.
  datavr=0.; datavt=0.; datavv=0.
  
  ! Read data traces
  call sreed("yy",datayy,4*n1)
  call sreed("yx",datayx,4*n1)
  call sreed("yz",datayz,4*n1)
  
  call sreed("xy",dataxy,4*n1)
  call sreed("xx",dataxx,4*n1)
  call sreed("xz",dataxz,4*n1)
  
  call sreed("zy",datazy,4*n1)
  call sreed("zx",datazx,4*n1)
  call sreed("zz",datazz,4*n1)
  
  if ( sum(abs(datayy)).eq.0 ) kill=.true.
  if ( sum(abs(datayx)).eq.0 ) kill=.true.
  if ( sum(abs(datayz)).eq.0 ) kill=.true.
  
  if ( sum(abs(dataxy)).eq.0 ) kill=.true.
  if ( sum(abs(dataxx)).eq.0 ) kill=.true.
  if ( sum(abs(dataxz)).eq.0 ) kill=.true.
  
  if ( sum(abs(datazy)).eq.0 ) kill=.true.
  if ( sum(abs(datazx)).eq.0 ) kill=.true.
  if ( sum(abs(datazz)).eq.0 ) kill=.true.

!  datayz = -datayz
!  dataxz = -dataxz
!  datazy = -datazy
!  datazx = -datazx
  
  ! Rotation Angle (0-2pi)
  delx  = head(rx_key,ind) - head(sx_key,ind)
  dely  = head(ry_key,ind) - head(sy_key,ind)
  angle = atan2(delx,dely)  ! x is north, y is east (and z is positive down), angle is positive clockwise (seen from above)
  if (angle.lt.0) angle = angle + 2*pi
!  angle = atan(delx/dely)  ! x is north, y is east (and z is positive down), angle is positive clockwise (seen from above)

!  if (ind.eq.1) write(0,*) head(rx_key,ind),head(ry_key,ind),head(sx_key,ind),head(sy_key,ind)
!  if (ind.eq.2) write(0,*) head(rx_key,ind),head(ry_key,ind),head(sx_key,ind),head(sy_key,ind)

  M=0.
  M(1,1)= cos(angle)
  M(2,1)=-sin(angle)
  M(1,2)= sin(angle)
  M(2,2)= cos(angle)
  M(3,3)= 1.
  
  ! Rotation formula
  dataxr = dataxx*M(1,1) + dataxy*M(1,2) + dataxz*M(1,3)
  datayr = datayx*M(1,1) + datayy*M(1,2) + datayz*M(1,3)
  datazr = datazx*M(1,1) + datazy*M(1,2) + datazz*M(1,3)

  dataxt = dataxx*M(2,1) + dataxy*M(2,2) + dataxz*M(2,3)
  datayt = datayx*M(2,1) + datayy*M(2,2) + datayz*M(2,3)
  datazt = datazx*M(2,1) + datazy*M(2,2) + datazz*M(2,3)

  dataxv = dataxx*M(3,1) + dataxy*M(3,2) + dataxz*M(3,3)
  datayv = datayx*M(3,1) + datayy*M(3,2) + datayz*M(3,3)
  datazv = datazx*M(3,1) + datazy*M(3,2) + datazz*M(3,3)
  
  datarr = M(1,1)*dataxr + M(1,2)*datayr + M(1,3)*datazr
  datatr = M(2,1)*dataxr + M(2,2)*datayr + M(2,3)*datazr
  datavr = M(3,1)*dataxr + M(3,2)*datayr + M(3,3)*datazr

  datart = M(1,1)*dataxt + M(1,2)*datayt + M(1,3)*datazt
  datatt = M(2,1)*dataxt + M(2,2)*datayt + M(2,3)*datazt
  datavt = M(3,1)*dataxt + M(3,2)*datayt + M(3,3)*datazt

  datarv = M(1,1)*dataxv + M(1,2)*datayv + M(1,3)*datazv
  datatv = M(2,1)*dataxv + M(2,2)*datayv + M(2,3)*datazv
  datavv = M(3,1)*dataxv + M(3,2)*datayv + M(3,3)*datazv
!  datavv = angle
  
  if (kill) datarr=0.
  if (kill) datart=0.
  if (kill) datarv=0.
  
  if (kill) datatr=0.
  if (kill) datatt=0.
  if (kill) datatv=0.
  
  if (kill) datavr=0.
  if (kill) datavt=0.
  if (kill) datavv=0.

  ! Write out traces
  if (exist_file("rr" )) then;  call srite("rr" ,datarr,4*n1);  end if
  if (exist_file("rt" )) then;  call srite("rt" ,datart,4*n1);  end if
  if (exist_file("rv" )) then;  call srite("rv" ,datarv,4*n1);  end if
  
  if (exist_file("tr" )) then;  call srite("tr" ,datatr,4*n1);  end if
  if (exist_file("tt" )) then;  call srite("tt" ,datatt,4*n1);  end if
  if (exist_file("tv" )) then;  call srite("tv" ,datatv,4*n1);  end if
  
  if (exist_file("vr" )) then;  call srite("vr" ,datavr,4*n1);  end if
  if (exist_file("vt" )) then;  call srite("vt" ,datavt,4*n1);  end if
  if (exist_file("vv" )) then;  call srite("vv" ,datavv,4*n1);  end if
  
  
  end do
  end do
  
  end do
  end do
  
  
  
  ! *********************************************************************
  !  Clean up and exit
  ! *********************************************************************
  write(0,*) '------------------------'
  deallocate( datayy , datayx , datayz )
  deallocate( dataxy , dataxx , dataxz )
  deallocate( datazy , datazx , datazz )

  deallocate( datayr , datayt , datayv )
  deallocate( dataxr , dataxt , dataxv )
  deallocate( datazr , datazt , datazv )

  deallocate( datarr , datart , datarv )
  deallocate( datatr , datatt , datatv )
  deallocate( datavr , datavt , datavv )
  deallocate( head )
  call exit(0)


end program
