program sdrSI2d_1d_t_omp
  ! computes AB*
  
  use sep                      ! Main SEP module
  use fftw_mod
  use spectra_mod_omp
  use fftshift_io_mod
  use sep_timers_mod
  
  implicit none
  
  ! Memory variables
  real,allocatable                :: dataA_io(:,:), dataB_io(:,:)
  complex,allocatable             :: dataA_ft(:,:), dataB_ft(:,:),data_si(:,:)
  
  ! Axis units
  integer                         :: nt,nw,nfft
  
  integer                         :: n(2),n3,n4,n5
  real                            :: d(2),o(2),d3,d4,d5,o3,o4,o5
  
  integer                         :: na(2),na3,na4,na5
  real                            :: da(2),oa(2),da3,da4,da5,oa3,oa4,oa5
  
  integer                         :: nb(2),nb3,nb4,nb5
  real                            :: db(2),ob(2),db3,db4,db5,ob3,ob4,ob5
  
  integer                         :: ns,js,fs,ind,ntau
  real                            :: eps
  
  ! Type
  integer                         :: type                    ! 1: cross-spectrum, 2: coherence, 3: deconvolution (A/B), 4: deconvolution (A\B)
  
  ! Misc
  integer                         :: ii,jj,kk,ll,mm
  real, parameter                 :: pi=3.14159265
  logical                         :: verb,padd,sym_t,true_t  ! true_t, take all t as positive values, sym_t, write positive and negative lags
  
  ! For timers
  logical           :: logic
  integer           :: tm

  logic = init_sep_timers()
  logic = setup_next_timer("Times",tm)
  call start_timer_num(tm)

  ! Open I/O headerfiles
  call sep_init()              ! Call to SEP module 
  
  if (exist_file("A")) then;
    call from_aux("A","n1", na(1) ,1 )
    call from_aux("A","n2", na(2) ,1 )
    call from_aux("A","n3", na3   ,1 )
    call from_aux("A","n4", na4   ,1 )
    call from_aux("A","n5", na5   ,1 )
    call from_aux("A","d1", da(1) ,1.)
    call from_aux("A","d2", da(2) ,1.)
    call from_aux("A","o1", oa(1) ,0.)
    call from_aux("A","o2", oa(2) ,0.)    
  end if

  if (exist_file("B")) then;
    call from_aux("B","n1", nb(1) ,1 )
    call from_aux("B","n2", nb(2) ,1 )
    call from_aux("B","n3", nb3   ,1 )
    call from_aux("B","n4", nb4   ,1 )
    call from_aux("B","n5", nb5   ,1 )
    call from_aux("B","d1", db(1) ,1.)
    call from_aux("B","d2", db(2) ,1.)
    call from_aux("B","o1", ob(1) ,0.)
    call from_aux("B","o2", ob(2) ,0.)    
  end if

  write(0,*) "na, nb"
  write(0,*) na, nb

  ! Error checks, check if files have equal sizes
  if (na(1).eq.nb(1)) then; n(1)=na(1); else; write(0,*) "na(1).ne.nb(1)"; call exit(0); end if
  if (na(2).eq.nb(2)) then; n(2)=na(2); else; write(0,*) "na(2).ne.nb(2)"; call exit(0); end if
  if (na3  .eq.nb3  ) then; n3  =na3  ; else; write(0,*) "na3  .eq.nb3  "; call exit(0); end if
  if (na4  .eq.nb4  ) then; n4  =na4  ; else; write(0,*) "na4  .eq.nb4  "; call exit(0); end if
  if (na5  .eq.nb5  ) then; n5  =na5  ; else; write(0,*) "na5  .eq.nb5  "; call exit(0); end if
  if (da(1).eq.db(1)) then; d(1)=da(1); else; write(0,*) "da(1).eq.db(1)"; call exit(0); end if
  if (da(2).eq.db(2)) then; d(1)=da(1); else; write(0,*) "da(2).eq.db(2)"; call exit(0); end if
  if (oa(1).eq.ob(1)) then; o(1)=oa(1); else; write(0,*) "oa(1).eq.ob(1)"; call exit(0); end if
  if (oa(2).eq.ob(2)) then; o(2)=oa(2); else; write(0,*) "oa(2).eq.ob(2)"; call exit(0); end if

  ! Get correlation geometries  
  call from_param  ("ns",ns,n(2))
  call from_param  ("js",js,  1 )
  call from_param  ("fs",fs,  0 )
  
  ! FFTW dettails
  call from_param("nfft"   , nfft  ,  n(1)  )     ! Determine over what length to calculate the fft (can best be powers of two), does symmetric zero padding
  call from_param("true_t" , true_t, .true. )     ! Puts data at true indices in data-array, otherwise, it will assume all data has irrelevant absolute t,x,y coordinates
  call from_param("sym_t"  , sym_t , .true. )     ! To be implemented (to get t-symmetric output [-nt:nt])
  
  ! Misc
  call from_param("verbose",verb ,.false.)
  call from_param("ntau"   ,ntau , n(1)  )            ! Amount of time lags to print out
  call from_param("type"   ,type ,   1   )
  call from_param("eps"  , eps , 0. )
  
  ! internal or external circular correlation (sym_t)
  if (true_t        ) then; nt=2*n(1)  ; else; nt=n(1)     ; end if
  nfft = max(nt,nfft)   ! make sure nfft is large enough

  ! Update History File
  if (sym_t) then;  call to_history  ("n1", 2*ntau    );
  else;             call to_history  ("n1",   ntau    );  end if
  call to_history  ("d1",       d(1) )
  if (sym_t) then;  call to_history  ("o1", -ntau*d(1));
  else;             call to_history  ("o1",        0. );  end if
  call to_history  ("n2",      n(2)    )
  call to_history  ("n3",      ns      )
  call to_history  ("d3",      js*d(2) )
  call to_history  ("o3", o(2)+fs*d(2) )
  call to_history  ("n4", n3 )
  call to_history  ("n5", n4 )
  call to_history  ("n6", n5 )
  call to_history  ("d4", d3 )
  call to_history  ("d5", d4 )
  call to_history  ("d6", d5 )
  call to_history  ("o4", o3 )
  call to_history  ("o5", o4 )
  call to_history  ("o6", o5 )
  
  ! Close properly
  call sep_close ()            ! Close all I/O to headerfiles
  
  ! Initialize FFTW routine
  call fftw1d_1_r2c_lop_init(nt_in=nfft); nw = nfft/2+1
  call spectra_init(eps)

  write(0,*) 'nfft,nt,nw'
  write(0,*)  nfft,nt,nw

  ! Allocate the arrays datain and dataout
  allocate( dataA_io(nt, n(2)), dataA_ft(nw,n(2)) , data_si(nw,n(2)) )
  allocate( dataB_io(nt, n(2)), dataB_ft(nw,n(2)) )
  
  ! Loup over 3rd, 4th and 5th axis
  do mm=1,n5; do ll=1,n4; do kk=1,n3;
    
    ! Verbose which Cube
    write(0,*) "Plane",(mm-1)*n5*n4+(ll-1)*n4+kk,"of",n5*n4*n3
    
    
    ! Read Data
    dataA_io = 0.
    dataB_io = 0.
        
    if (true_t) then;  call sreed("A",dataA_io(1:n(1),:),n(1)*n(2)*4);
    else;   call fftshift1d_sreed("A",dataA_io          ,n(1),n(2),4); end if
    if (true_t) then;  call sreed("B",dataB_io(1:n(1),:),n(1)*n(2)*4);
    else;   call fftshift1d_sreed("B",dataB_io          ,n(1),n(2),4); end if

    ! FFTW
    call fftw1d_1_r2c_lop(.false.,.false.,dataA_io,dataA_ft);
    call fftw1d_1_r2c_lop(.false.,.false.,dataB_io,dataB_ft);
    
    do jj=1,ns
      ! Calculate index
      ind = 1+fs+(jj-1)*js
      write(0,*) "Interfering",ind,'/',ns,"to",n(2)
      
      ! Interferometry
      select case(type)
        case(1);  call spectra_crsp(dataA_ft,dataB_ft(:, ind),data_si);
        case(2);  call spectra_cohr(dataA_ft,dataB_ft(:, ind),data_si);
        case(3);  call spectra_decv(dataA_ft,dataB_ft(:, ind),data_si);
        case(4);  call spectra_decv(dataA_ft(:, ind),dataB_ft,data_si);
      end select
      
      ! FFTW      
      call fftw1d_1_r2c_lop(.true.,.false.,dataA_io,data_si);
      
      ! Write Data
      if (sym_t) then;  call fftshift1d_srite("out" ,dataA_io          ,2*ntau,n(2),4);
      else;             call            srite("out", dataA_io(1:ntau,:),  ntau*n(2)*4);  end if

    end do
    
    
  end do;     end do;     end do;
  
  
  ! Deallocate and close stuff
  deallocate( dataA_io , dataA_ft , data_si )
  deallocate( dataB_io , dataB_ft )
  call fftw1d_1_r2c_lop_close()
  call spectra_close()
  call stop_timer_num (tm)
  call print_timers()

  ! Force clean exit
  call exit (0)
  
end program




