program sdrSI2d_self1d_t

  use sep                      ! Main SEP module
  use fftw_mod
  use spectra_mod_omp
  use fftshift_io_mod
  use sep_timers_mod
  
  implicit none
  
  ! Memory variables
  real,allocatable                :: data_io(:,:)
  complex,allocatable             :: data_ft(:,:),data_si(:,:)
  
  ! Axis units
  integer                         :: n(2),nt,nw,nfft,n3,n4,n5
  real                            :: d(2),o(2),d3,d4,d5,o3,o4,o5
  integer                         :: ns,js,fs,ind,ntau
  
  ! Type
  integer                         :: type                    ! 1: cross-spectrum, 2: coherence, 3: deconvolution (A/B), 4: deconvolution (A\B)

  ! Misc
  integer                         :: ii,jj,kk,ll,mm
  real, parameter                 :: pi=3.14159265
  logical                         :: verb,padd,sym_t,true_t  ! true_t, take all t as positive values, sym_t, write positive and negative lags
  real                            :: eps

  ! For timers
  logical           :: logic
  integer           :: tm

  logic = init_sep_timers()
  logic = setup_next_timer("Times",tm)
  call start_timer_num(tm)

  ! Open I/O headerfiles
  call sep_init()              ! Call to SEP module 
  
  ! Read acis Details
  call from_history("n1", n(1) ,1 )
  call from_history("n2", n(2) ,1 )
  call from_history("n3", n3,   1 )
  call from_history("n4", n4,   1 )
  call from_history("n5", n5,   1 )
  call from_history("d1", d(1) ,1.)
  call from_history("d2", d(2) ,1.)
  call from_history("d3", d3,   1.)
  call from_history("d4", d4,   1.)
  call from_history("d5", d5,   1.)
  call from_history("o1", o(1) ,0.)
  call from_history("o2", o(2) ,0.)
  call from_history("o3", o3,   0.)
  call from_history("o4", o4,   0.)
  call from_history("o5", o5,   0.)
  
  call from_param  ("ns",ns,n(2))
  call from_param  ("js",js,  1 )
  call from_param  ("fs",fs,  0 )
  
  ! FFTW dettails
  call from_param("nfft"   , nfft  ,  n(1)  )     ! Determine over what length to calculate the fft (can best be powers of two), does symmetric zero padding
  call from_param("true_t" , true_t, .true. )     ! Puts data at true indices in data-array, otherwise, it will assume all data has irrelevant absolute t,x,y coordinates
  call from_param("sym_t"  , sym_t , .true. )     ! To be implemented (to get t-symmetric output [-nt:nt])
  
  ! Misc
  call from_param("verbose",verb ,.false.)
  call from_param("ntau"   ,ntau , n(1)  )            ! Amount of time lags to print out
  call from_param("type"   ,type ,   1   )
  call from_param("eps"  , eps , 0. )
  
  ! internal or external circular correlation (sym_t)
  if (true_t        ) then; nt=2*n(1)  ; else; nt=n(1)     ; end if
  nfft = max(nt,nfft)   ! make sure nfft is large enough
  
  ! Update History File
  if (sym_t) then;  call to_history  ("n1", 2*ntau    );
  else;             call to_history  ("n1",   ntau    );  end if
  call to_history  ("d1",       d(1) )
  if (sym_t) then;  call to_history  ("o1", -ntau*d(1));
  else;             call to_history  ("o1",        0. );  end if
  call to_history  ("n3",      ns      )
  call to_history  ("d3",      js*d(2) )
  call to_history  ("o3", o(2)+fs*d(2) )
  call to_history  ("n4", n3 )
  call to_history  ("n5", n4 )
  call to_history  ("n6", n5 )
  call to_history  ("d4", d3 )
  call to_history  ("d5", d4 )
  call to_history  ("d6", d5 )
  call to_history  ("o4", o3 )
  call to_history  ("o5", o4 )
  call to_history  ("o6", o5 )
  
  ! Close properly
  call sep_close ()            ! Close all I/O to headerfiles
  
  ! Initialize FFTW routine
  call fftw1d_1_r2c_lop_init(nt_in=nfft); nw = nfft/2+1
  call spectra_init(eps)

  write(0,*) 'nfft,nt,nw'
  write(0,*)  nfft,nt,nw

  ! Allocate the arrays datain and dataout
  allocate( data_io(nt, n(2)), data_ft(nw,n(2)) , data_si(nw,n(2)) )
  
  ! Loup over 3rd, 4th and 5th axis
  do mm=1,n5; do ll=1,n4; do kk=1,n3;
    
    ! Verbose which Cube
    write(0,*) "Plane",(mm-1)*n5*n4+(ll-1)*n4+kk,"of",n5*n4*n3
    

    ! Read Data
    data_io = 0.
    
    if (true_t) then;
      do jj=1,n(2)
        call sreed("in",data_io(1,jj),n(1)*4);
      end do
    else;   call fftshift1d_sreed("in",data_io          ,n(1),n(2),4); end if

    ! FFTW
    call fftw1d_1_r2c_lop(.false.,.false.,data_io,data_ft);
    
    do jj=1,ns
      ! Calculate index
      ind = 1+fs+(jj-1)*js
      write(0,*) "Interfering",ind,'/',ns,"to",n(2)
      
      ! Interferometry
      select case(type)
        case(1);  call spectra_crsp(data_ft,data_ft(:, ind),data_si);
        case(2);  call spectra_cohr(data_ft,data_ft(:, ind),data_si);
        case(3);  call spectra_decv(data_ft,data_ft(:, ind),data_si);
        case(4);  call spectra_decv(data_ft(:, ind),data_ft,data_si);
      end select
      
      ! Compute Inverse (Padd) (Shift) and FFT
      call fftw1d_1_r2c_lop(.true.,.false.,data_io,data_si);

      ! Write Data
      if (sym_t) then;  call fftshift1d_srite("out" ,data_io          ,2*ntau,n(2),4);
      else;             call            srite("out", data_io(1:ntau,:),  ntau*n(2)*4);  end if

    end do
    
    
  end do;     end do;     end do;
  
  
  ! Deallocate and close stuff
  deallocate( data_io , data_ft , data_si )
  call fftw1d_1_r2c_lop_close();
  call stop_timer_num (tm)
  call print_timers()

  ! Force clean exit
  call exit (0)
  
end program




