program SIexpand_header
 ! program that treats the header values as a datafile and expands it according to
 ! creating seismic sources as in seismic interferometry
 
 ! Needs rx,ry,sx,sy (integers) that are the keynumbers of the receiver and source
 ! x and y coordinates in the header files.
 
 use sep
 implicit none

 integer                :: nhdrs,ntraces
 integer                :: rx,ry,sx,sy
 
 integer                :: ns,fs,js,ss    ! Seismic interferometry parameters
 integer                :: ii
 
 real,allocatable       :: headers(:,:)
 
 call sep_init()
 
 call from_history('n1',  nhdrs,0)
 call from_history('n2',ntraces,0)
 
 call from_param('rx',rx,1)
 call from_param('ry',ry,2)
 call from_param('sx',sx,4)
 call from_param('sy',sy,5)
 call from_param( "sx_key" ,sx,sx)
 call from_param( "sy_key" ,sy,sy)
 call from_param( "rx_key" ,rx,rx)
 call from_param( "ry_key" ,ry,ry)

 call from_param('fs',fs,0)
 call from_param('js',js,1)
 call from_param('ns',ns,1)

 call to_history('n1',  nhdrs  )
 call to_history('n2',ntraces*ns)
 
 call sep_close()
 
 allocate(headers(nhdrs,ntraces))
 
 ! read all headers
 call sreed("in",headers,4*nhdrs*ntraces)
 
 write(0,*) 'rx,ry,sx,sy'
 write(0,*)  rx,ry,sx,sy
 
 ! Write headers
 do ii=1,ns
   ! Source at trace index
   ss = fs+(ii-1)*js+1

   ! Set key header values properly
   headers(sx,:) = headers(rx,ss)
   headers(sy,:) = headers(ry,ss)

   ! Write out headers
   call srite("out",headers,4*nhdrs*ntraces)
   
 end do
 
 
 ! deallocate
 deallocate(headers)
 
end program
