program sdrTPtrans2d
  
  use sep
  use tp_trans_mod
  use sep_timers_mod

  implicit none

  integer           :: nt,nx,n3,n4,n5
  real              :: dt,dx,ot,ox
  
  integer           :: ntau,np
  real              :: dtau,otau,op,dp,pmin,pmax
  
  real,allocatable  :: datain (:,:)
  real,allocatable  :: dataout(:,:)
  
  integer           :: ii,jj,kk,ll,mm
  
  logical           :: inv,true_x

  ! For timers
  logical           :: logic
  integer           :: tm
  logic = init_sep_timers()
  logic = setup_next_timer("Times",tm)
  call start_timer_num(tm)

  call sep_init()

  call from_param("inv"   ,inv   ,.false.)
  call from_param("true_x",true_x,.false.)

  call from_history("n3",n3,1)
  call from_history("n4",n4,1)
  call from_history("n5",n5,1)

  if (inv) then

    ! Read (tau,p) axis from History
    call from_history("n1",ntau, 1 )
    call from_history("d1",dtau, 1.)
    call from_history("o1",otau, 0.)

    call from_history("n2",np  , 1 )
    call from_history("d2",dp  , 1.)
    call from_history("o2",op  , 0.)
    
    ! Write (t,x) axis to Parameter
    call from_param  ("nt",nt  ,-1      )
    call from_param  ("dt",dt  ,-1.     )
    call from_param  ("ot",ot  ,-11.1111)

    call from_param  ("nx",nx  ,-1      )
    call from_param  ("dx",dx  ,-1.     )
    call from_param  ("ox",ox  ,-11.1111)
    
    ! Backup (t,x) from History
    if (nt.eq.-1      ) then; call from_history("nt",nt,ntau); end if
    if (dt.eq.-1.     ) then; call from_history("dt",dt,dtau); end if
    if (ot.eq.-11.1111) then; call from_history("ot",ot,otau); end if
    
    if (nx.eq.-1      ) then; call from_history("nx",nx,  np); end if
    if (nx.eq.-1      ) then; call from_history("dx",dx,  1.); end if
    if (ox.eq.-11.1111) then; call from_history("ox",ox,  0.); end if
    
    ! Write (t,x) axis to History
    call to_history  ("n1",nt)
    call to_history  ("d1",dt)
    call to_history  ("o1",ot)
    
    call to_history  ("n2",nx)
    call to_history  ("d2",dx)
    call to_history  ("o2",ox)  

    call to_history("label1",'t')
    call to_history("label2",'x')

  else

    ! Write (t,x) axis to History
    call from_history("n1",nt,1 )
    call from_history("d1",dt,1.)
    call from_history("o1",ot,0.)
    
    call from_history("n2",nx,1 )
    call from_history("d2",dx,1.)
    call from_history("o2",ox,0.)
    if (.not.true_x) then; ox = -nx/2*dx; end if
    
    ! Read (tau,p) axis from Parameter
    call from_param  ("ntau",ntau, nt)
    call from_param  ("dtau",dtau, dt)
    call from_param  ("otau",otau, ot)
    
    call from_param  ("pmin",pmin, -1./500.)
    call from_param  ("pmax",pmax,  1./500.)
    call from_param  (  "np",np  , nx  )
    call from_param  (  "dp",dp  , (pmax-pmin)/(np-1) )
    call from_param  (  "op",op  , pmin)
    
    ! Write (tau,p) axis to History    
    call to_history  ("n1",ntau)
    call to_history  ("d1",dtau)
    call to_history  ("o1",otau)
    
    call to_history  ("n2",  np)
    call to_history  ("d2",  dp)
    call to_history  ("o2",  op)

    call to_history("label1",'\F10 t')   ! '\F10 t\F-1  [s]'
    call to_history("label2",'p'     )

  end if
  
  call sep_close()
  
  
  ! Allocations and Initializations
  allocate( datain(nt,nx) , dataout(ntau,np) )
  call tp_trans_2d_init(  nt_in=nt  ,  dt_in=dt  ,  ot_in=ot  ,nx_in=nx,dx_in=dx,ox_in=ox, &
                        ntau_in=ntau,dtau_in=dtau,otau_in=otau,np_in=np,dp_in=dp,op_in=op)
  
  
  ! Kernel
  do mm=1,n5;  do ll=1,n4;  do kk=1,n3
    write(0,*) "Plane",(mm-1)*n5*n4+(ll-1)*n4+kk,"of",n4*n5*n3
    
    if (inv) then
      call sreed("in" , dataout, 4*size(dataout))
    else
      call sreed("in" , datain , 4*size(datain ))
    end if
    
    call tp_trans_2d_fop(inv,.false.,datain,dataout)
    
    if (inv) then
      call srite("out", datain , 4*size(datain ))
    else
      call srite("out", dataout, 4*size(dataout))
    end if
    
  end do;      end do;      end do
  
  
  ! Deallocations and closures  
  deallocate(datain,dataout)
  call tp_trans_2d_close()
  call stop_timer_num (tm)
  call print_timers()

  call exit(0)

end program
