program sdrTPtrans3d_irr
  
  use sep
  use tp_trans_irr_mod
  use sep_timers_mod
  use solver_smp_mod
  use cgstep_mod
  
  implicit none
  
  integer           :: nt,n2,n3,n4,n5,n2xy
  real              :: dt,ot
  
  integer           :: ntau,npx,npy
  real              :: dtau,otau,opx,opy,dpx,dpy,pxmin,pxmax,pymin,pymax
  
  real,allocatable  :: datain (:),dataxy(:,:)
  real,allocatable  :: dataout(:)
  
  integer           :: ii,jj,kk,ll,mm
  
  logical           :: center_xy
  
  ! For timers
  logical           :: logic
  integer           :: tm


  logic = init_sep_timers()
  logic = setup_next_timer("Times",tm)
  call start_timer_num(tm)

  call sep_init()


  ! Write (t,x) axis to History
  call from_history("n1",nt,1 )
  call from_history("d1",dt,1.)
  call from_history("o1",ot,0.)
    
  call from_history("n2",n2,1 )
  call from_history("n3",n3,1 )
  call from_history("n4",n4,1 )
  call from_history("n5",n5,1 )

!  if (.not.true_x) then; ox = -nx/2*dx; end if
!  if (.not.true_y) then; oy = -ny/2*dy; end if
    
  ! Read (tau,p) axis from Parameter
  call from_param  ("ntau",ntau, nt)
  call from_param  ("dtau",dtau, dt)
  call from_param  ("otau",otau, ot)
  
  call from_param  ("pxmin",pxmin,-1./500.)
  call from_param  ("pxmax",pxmax, 1./500.)
  call from_param  ("pymin",pymin,-1./500.)
  call from_param  ("pymax",pymax, 1./500.)
  
  call from_param  ( "npx",npx , int(sqrt(real(n2)))  )
  call from_param  ( "dpx",dpx , (pxmax-pxmin)/(npx-1) )
  call from_param  ( "opx",opx , pxmin)

  call from_param  ( "npy",npy , int(sqrt(real(n2)))  )
  call from_param  ( "dpy",dpy , (pymax-pymin)/(npy-1) )
  call from_param  ( "opy",opy , pymin)
    
  ! Write (tau,p) axis to History    
  call to_history  ("n1",ntau)
  call to_history  ("d1",dtau)
  call to_history  ("o1",otau)
    
  call to_history  ("n2", npx)
  call to_history  ("d2", dpx)
  call to_history  ("o2", opx)

  call to_history  ("n3", npy)
  call to_history  ("d3", dpy)
  call to_history  ("o3", opy)

  call to_history  ("n4", n3*n4*n5 )
  call to_history  ("d4", 1. )
  call to_history  ("o4", 0. )

  call to_history("label1",'\F10 t')   ! '\F10 t\F-1  [s]'
  call to_history("label2",'px'    )
  call to_history("label3",'py'    )
  call to_history("label4",' '    )
  
 ! call sep_close()
  
  ! Get xy coordinates of receivers
  call from_aux('xy_in','n2',n2xy,1)
  if (n2xy.ne.n2) call erexit("n2xy.ne.n2")
  allocate(dataxy(2,n2xy)); dataxy=0.
  call sreed("xy_in",dataxy,4*size(dataxy))
  call from_param('center_xy',center_xy,.true.)
 ! write(0,*) 'X',dataxy(1,:)
 ! write(0,*) 'Y',dataxy(2,:)
  if (center_xy) then
    dataxy(1,:) = dataxy(1,:)-real(sum(dble(dataxy(1,:)))/dble(n2xy))
    dataxy(2,:) = dataxy(2,:)-real(sum(dble(dataxy(2,:)))/dble(n2xy))
  end if
  
  ! Allocations and Initializations
  allocate( datain(nt*n2) , dataout(ntau*npx*npy) )

 ! write(0,*) 'npx,npy,ntau',npx,npy,ntau

  call tp_trans_irr_init(  nt_in=nt  ,  dt_in=dt  ,  ot_in=ot  ,nxy_in=n2, dataxy=dataxy ,&
                        ntau_in=ntau,dtau_in=dtau,otau_in=otau,npx_in=npx,dpx_in=dpx,opx_in=opx,npy_in=npy,dpy_in=dpy,opy_in=opy)
  
  
  ! Kernel
  do mm=1,n5;  do ll=1,n4; do kk=1,n3;
    write(0,*) "Set",(mm-1)*n5*n4+(ll-1)*n4+kk,"of",n3*n4*n5
    datain =0.
    dataout=0.
    call sreed("in" , datain , 4*size(datain ))
    call tp_trans_irr_adj(dataout,datain)   ! Simple Adjoint
    call srite("out", dataout, 4*size(dataout))

  end do;      end do;     end do;
  
  
  ! Deallocations and closures  
  deallocate(datain,dataout,dataxy)
  call tp_trans_irr_close()
  call stop_timer_num (tm)
  call print_timers()
  call sep_close()

  call exit(0)

end program
