program sdrTape
! Tape a datafile
  ! taper types: 
  ! taper types: 
  ! 1) 'zero'            --  needed: 'nothing' 
  ! 2) 'rect'            --  needed: t1,t2 
  ! 3) 'linear_up'       --  needed: t1,t2
  ! 4) 'linear_down'     --  needed: t1,t2
  ! 5) 'pyramid'         --  needed: t1,t2
  ! 6) 'pyramid_band'    --  needed: t1,t2,t3,t4
  ! 7) 'hann'            --  needed: t1,t2
  ! 8) 'hann_half'       --  needed: t1,t2
  ! 9) 'hann_band'       --  needed: t2,t2,t3,t4
  ! 10) 'cosine'         --  needed: t1,t2
  ! 11) 'gaussian'       --  needed: t1,t2,sigma,spread
  ! 12) 'gaussian_half'  --  needed: t1,t2,sigma,spread
  ! 13) 'gaussian_block' --  needed: t1,t2,t3,t4,sigma,spread
  !
  ! All tapers have the form of a band or lowcut,
  ! Notch tapers are constructed with inv=1 option.
  ! Sum result onto old data using    add=1 option.
  
  ! Multiple calls take input and overwrite if add=1
  
  use sep                      ! Main SEP module
  use tapers_mod
  use tape_mod
  implicit none
  
  real, allocatable               :: taper(:,:,:),data(:,:,:)
  integer                         :: n(3), ii
  integer                         :: type(3),dir(3)
  real                            :: d(3),o(3)
  real                            :: t1(3),t2(3),t3(3),t4(3)
  integer                         :: t1_ind(3),t2_ind(3),t3_ind(3),t4_ind(3)
  logical                         :: add,inv,inv_3(3)
  real                            :: sigma(3),scale(3)
  integer                         :: n4,n5,n6,n7,ll,mm,nn,oo
  
  ! Open I/O headerfiles
  call sep_init()              ! Call to SEP module 
  
  call from_history("n1", n(1) ,1 )
  call from_history("n2", n(2) ,1 )
  call from_history("n3", n(3) ,1 )
  call from_history("d1", d(1) ,1.)
  call from_history("d2", d(2) ,1.)
  call from_history("d3", d(3) ,1.)
  call from_history("o1", o(1) ,0.)
  call from_history("o2", o(2) ,0.)
  call from_history("o3", o(3) ,0.)

  type=0

  ! Read input parameters for taper, there is lots of options :-)
 ! call from_param ("dir"  ,dir, (/ 1,2,3 /) )
  call from_param ("type" ,type,(/ 0,0,0 /) )
  call from_param ("inv"  ,inv,.false. )
  call from_param ("add"  ,add,.true.  )
  call from_param ("sigma",sigma, (/.1 ,.1, .1 /) )
  call from_param ("scale",scale, (/ 1., 1., 1./) )
  call from_param ("dir"  ,dir  , (/ 1 , 2 , 3 /) )
  
  t1=0.; 
  t2=0.; 
  t3=0.; 
  t4=0.; 

  call from_param ("t1",t1, o               )
  call from_param ("t2",t2, o + d*real(n-1) )
  call from_param ("t3",t3, o + d*real(n-1) )
  call from_param ("t4",t4, o + d*real(n-1) )

  write(0,*) 't1',t1
  write(0,*) 't2',t2
  write(0,*) 't3',t3
  write(0,*) 't4',t4
  
  if (exist_file ("taper_out")) then
    ! Write to auxiliary .H file
    call to_history ( "n1", n(1), "taper_out" )   
    call to_history ( "n2", n(2), "taper_out" )
    call to_history ( "n3", n(3), "taper_out" )
    call to_history ( "d1", d(1), "taper_out" )
    call to_history ( "d2", d(2), "taper_out" )
    call to_history ( "d3", d(3), "taper_out" )
    call to_history ( "o1", o(1), "taper_out" )
    call to_history ( "o2", o(2), "taper_out" )
    call to_history ( "o3", o(3), "taper_out" )
  end if
  
  call from_history( "n4" ,n4 ,1 )
  call from_history( "n5" ,n5 ,1 )
  call from_history( "n6" ,n6 ,1 )
  call from_history( "n7" ,n7 ,1 )

  ! Close properly
  call sep_close ()            ! Close all I/O to headerfiles
  
  ! Make sure that no taper specification gives a 1 in that direction
  do ii=1,3;
    if (type(ii)==0) then; inv_3(ii)=.true.; type(ii)=1
    else; inv_3(ii)=inv; end if
  end do
  
  ! Determine tie indices
  do ii=1,3
    call taper_indice(t1(ii),t1_ind(ii),d_in=d(ii),o_in=o(ii))
    call taper_indice(t2(ii),t2_ind(ii),d_in=d(ii),o_in=o(ii))
    call taper_indice(t3(ii),t3_ind(ii),d_in=d(ii),o_in=o(ii))
    call taper_indice(t4(ii),t4_ind(ii),d_in=d(ii),o_in=o(ii))
  end do
  
  ! Initialize tape functions
  call tape_init       (n(1),n(2),n(3))

  ! Initialize taper functions in tapers_mod
  call tapers_init     (n(1),n(2),n(3))
  call tapers_interface(3,dir_in=dir,t1_in=t1_ind,t2_in=t2_ind,t3_in=t3_ind,t4_in=t4_ind,type_in=type,sigma_in=sigma,scale_in=scale,inv_in=inv_3)
  call tapers_insert_tape()
  call tapers_close()

  ! Allocate the arrays datain and dataout
  allocate ( data(n(1),n(2),n(3)) )

  do oo=1,n7; do nn=1,n6; do mm=1,n5; do ll=1,n4

  ! Read in data from input file
  call sep_read  (data)
  
  ! Execute Taper
  call tape_op   (data)
  
  ! Write data to output file
  call sep_write (data)

  end do;  end do;  end do;  end do

  ! Retrieve and put out taper, if required.
  if ( exist_file("taper_out") ) then
     allocate  (taper(n(1),n(2),n(3)))
     call tape_gettaper(taper)
     call srite("taper_out", taper(:,:,:), 4*size(taper))
     deallocate(taper          )
  end if
  
  ! Deallocate memory  
  deallocate     (data)
  call tape_close()
  
  ! Force clean exit
  call exit (0)
  
end program

