program srDemean1d
  ! Program to remove a dc component in double precision
  ! Sjoerd de Ridder, Stanford, 1/25/2012

  use sep
  implicit none

  integer                          :: n1, n2, n3, n4, n5, n6
  integer                          :: jj
  real, allocatable, dimension(:)  :: data
  double precision                 :: mean
  logical                          :: verbose
 
  ! Open I/O headerfiles 
  call sep_init()
  call from_param('verbose',verbose,.false.)
  call from_param('verb'   ,verbose,verbose)
  if (verbose) write(0,*) '-----------------------------------'
  if (verbose) write(0,*) '    DOUBLE PRECISION DC REMOVAL    '
  if (verbose) write(0,*) '-----------------------------------'
  if (verbose) write(0,*) '    i/o looping over traces        '

  ! Read and write History
  call from_history('n1',n1,1 )
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  call from_history('n4',n4,1 )
  call from_history('n5',n5,1 )
  call from_history('n6',n6,1 )
  if (verbose) write(0,"(A,I7.1,I7.1,I7.1,I7.1,I7.1,I7.1)") 'n1,n2,n3,n4,n5,n6 :  ',(/n1,n2,n3,n4,n5,n6/)
  if (verbose) write(0,*) '-----------------------------------'
  call sep_close()

  ! Allocate memory
  allocate ( data(n1) )


  if (verbose) write(0,*) '-------------------------------------------'
  !!!! Actual Kernel
  do jj=1,n2*n3*n4*n5*n6

    data = 0.
    ! Read the data
    call sreed('in',data,4*n1)
  
    ! Demean the array
    mean = sum ( dble(data) )/dble(n1)
    if (verbose) write(0,"(A,I7.1,A,I7.3,A,D)") 'trace: ',jj,' out of',n2*n3*n4*n5*n6,' mean: ', mean
    data = real( dble(data) -  mean  )

    ! Write the data
    call srite('out',data,4*n1)

  end do
  

  ! Deallocate memory
  if (verbose) write(0,*) '-----------------------------------'
  deallocate(data)
  call exit(0)
  
end program
