program srRunningFilterPatch
  ! Program to apply a running filter on passive data
  ! Sjoerd de Ridder, Stanford, 2/20/2012
  !
  ! Applying a softclip to all (new-)values exceeding the top quantile (q)
  ! of a previous window
  ! input: win   ( window to evaluate )
  !        q     ( quantile to clip )
  ! 
  ! Implemented trace by trace
  
  use sep
  use filterpatch_mod
  implicit none
  
  ! Include for FFTW
  include "fftw3.f"
  
  integer                          :: n1, n2, n3, n4, n5, n6, nw, nb,nf
  real                             :: d1, d2, d3, d4, d5, d6
  real                             :: o1, o2, o3, o4, o5, o6
  integer                          :: a,b,j1
  real                             :: df,of
  integer                          :: ii,jj,iw,wi,ij,ic,ib
  real, allocatable, dimension(:)  :: buffer1,buffer2,buffer3,weight,taper,datain,dataot
  real                             :: clip,q,buffer
  double precision                 :: mean
  logical                          :: verbose
  real, parameter                  :: pi=3.1415926535

  integer*8                        :: fft_for,fft_inv

  integer                          :: output

  integer                          :: funit    ! 1: Hz      2: Radians
  
  real,allocatable, dimension(:)     :: temp
  complex,allocatable, dimension(:)  :: bufferf

  ! For Tapers
  integer                          :: f1_ind,f2_ind,f3_ind,f4_ind,hind
  real                             :: f1,    f2,    f3,    f4
  
  ! Open I/O headerfiles 
  call sep_init()
  call from_param('verbose',verbose,.false.)
  call from_param('verb'   ,verbose,verbose)
  if (verbose) write(0,*) '-----------------------------------'
  if (verbose) write(0,*) '          RUNNING FILTER           '
  if (verbose) write(0,*) '-----------------------------------'
  if (verbose) write(0,*) '    i/o looping over traces        '
  
  call from_param('output',output,2)
  
  ! Read and write History
  call from_history('n1',n1,1 )
  call from_history('d1',d1,1.)
  call from_history('o1',o1,0.)
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  call from_history('n4',n4,1 )
  call from_history('n5',n5,1 )
  call from_history('d6',n6,1 )
  call from_history('d2',d2,1.)
  call from_history('d3',d3,1.)
  call from_history('d4',d4,1.)
  call from_history('d5',d5,1.)
  call from_history('d6',d6,1.)
  call from_history('o2',o2,0.)
  call from_history('o3',o3,0.)
  call from_history('o4',o4,0.)
  call from_history('o5',o5,0.)
  call from_history('o6',o6,0.)
  
  call from_param('q',q,.99)
  call from_param('win',nw,nint(1800./d1/2)*2)
  
  ! How many buffers     ! Redo this later (this is just first guess)
  nb = real(n1)/real(nw)*2-1
  
  call from_param ("j1",j1,1)
  if (mod(nw,j1).ne.0) write(0,*) 'Very nasty j1:',j1
  
  if (verbose) write(0,*) 'Buffer Size', nw
  if (verbose) write(0,*) '# Bufer Cycles', nb
  
  
  
  select case(output)
    case(1)
      call to_history('n1',n1/j1 )
      call to_history('d1',d1*j1 )
      call to_history('o1',o1 )
      
      call to_history('n2',n2 )
      call to_history('o2',o2 )
      call to_history('d2',d2 )
      
      call to_history('n3',n3 )
      call to_history('n4',n4 )
      call to_history('n5',n5 )
      call to_history('n6',n6)
      
      call to_history('d3',d3 )
      call to_history('d4',d4 )
      call to_history('d5',d5 )
      call to_history('d6',d6)
      
      call to_history('o3',o3 )
      call to_history('o4',o4 )
      call to_history('o5',o5 )
      call to_history('o6',o6 )
    case(2)
      call to_history('n1',nw/j1 )
      call to_history('d1',d1*j1 )
      call to_history('o1',o1 )
      
      call to_history('n2',nb )
      call to_history('o2',o1 )
      call to_history('d2',d2*nw/2 )
      
      call to_history('n3',n2 )
      call to_history('n4',n3 )
      call to_history('n5',n4 )
      call to_history('n6',n5*n6)
      
      call to_history('d3',d2 )
      call to_history('d4',d3 )
      call to_history('d5',d4 )
      call to_history('d6',d5*d6)
      
      call to_history('o3',o2 )
      call to_history('o4',o3 )
      call to_history('o5',o4 )
      call to_history('o6',o5*o6)
    case(3)
      call to_history('n1',nw/j1 )
      call to_history('d1',d1*j1 )
      call to_history('o1',o1 )
      
      call to_history('n2',nb )
      call to_history('o2',o1 )
      call to_history('d2',d2*nw/2 )

      call to_history('n3',n2 )
      call to_history('n4',n3 )
      call to_history('n5',n4 )
      call to_history('n6',n5*n6)

      call to_history('d3',d2 )
      call to_history('d4',d3 )
      call to_history('d5',d4 )
      call to_history('d6',d5*d6)

      call to_history('o3',o2 )
      call to_history('o4',o3 )
      call to_history('o5',o4 )
      call to_history('o6',o5*o6)
  end select
  
  
  ! Fourier units
  of = 0.
  df = 1./(nw*d1)
  nf = nw/2+1
  if (funit.eq.2) df=2.*pi*df
  
  ! Read Tie points from param
  call from_param ("f1"    ,f1    , of          )
  call from_param ("f1_ind",f1_ind, nint((f1-of)/df)+1)
  
  call from_param ("f2"    ,f2    , of+1*nf/3*df)
  call from_param ("f2_ind",f2_ind, nint((f2-of)/df)+1)
  
  call from_param ("f3"    ,f3    , of+2*nf/3*df)
  call from_param ("f3_ind",f3_ind, nint((f3-of)/df)+1)
  
  call from_param ("f4"    ,f4    , of+(nf-1)*df)
  call from_param ("f4_ind",f4_ind, nint((f4-of)/df)+1)
  
  ! Make sure they fall within range
  if (f1_ind.lt.1.) f1_ind=1
  if (f2_ind.lt.1.) f2_ind=1
  if (f3_ind.lt.1.) f3_ind=1
  if (f4_ind.lt.1.) f4_ind=1
  if (f1_ind.gt.n1) f1_ind=nw
  if (f2_ind.gt.n1) f2_ind=nw
  if (f3_ind.gt.n1) f3_ind=nw
  if (f4_ind.gt.n1) f4_ind=nw

  if (verbose) then
      if (verbose) write(0,"(A,F,X,F,X,F,X,F)") ' taper tie values : ',f1    ,f2    ,f3    ,f4
      if (verbose) write(0,"(A,I,X,I,X,I,X,I)") ' taper tie indices: ',f1_ind,f2_ind,f3_ind,f4_ind
  end if



  ! Allocate memory
  allocate ( buffer1(nw),buffer2(nw/j1),buffer3(nw/2/j1),weight(nw),taper(nw),datain(n1))
  allocate ( bufferf(nf),temp(nw/2))
  ! Prepare fftw plans
  call sfftw_plan_dft_r2c_1d(fft_for, nw, buffer1, bufferf, FFTW_ESTIMATE);
  call sfftw_plan_dft_c2r_1d(fft_inv, nw, bufferf, buffer1, FFTW_ESTIMATE);
  
  ! Construct taper
  do iw=1,nw
    weight(iw) = sin(pi*(iw-1)/real(nw))**2.
  end do
  
  ! Stitching weights as output
  if (exist_file("weight")) then
    if (verbose) write(0,*) 'write taper to file'
    call to_history("n1", nw ,"weight")
    call to_history("d1", d1 ,"weight")
    call to_history("o1", 0. ,"weight")
    call srite("weight",weight,4*nw)
  end if
  
  ! Taper for bandpass weights as output
  do ii=f1_ind,f2_ind
    taper(ii)= .5*(1.-cos(pi*(ii-f1_ind)/real(f2_ind-f1_ind)))
  end do
  taper(f2_ind:f3_ind) = 1.
  do ii=f3_ind,f4_ind
    taper(ii)= .5*(1.+cos(pi*(ii-f3_ind)/real(f4_ind-f3_ind)))
  end do
  
  ! Taper as output
  if (exist_file("taper")) then
    if (verbose) write(0,*) 'write taper to file'
    call to_history("n1", nf ,"taper")
    call to_history("d1", df ,"taper")
    call to_history("o1", of ,"taper")
    call srite("taper",taper,4*nf)
  end if
  ! Use taper to implement fft scaling
  taper = taper / real(nw)
  
  call sep_close()
  
  
  if (verbose) write(0,*) '-----------------------------------'
  !!!! Actual Kernel
  do jj=1,n2*n3*n4*n5*n6
    if (verbose) write(0,"(A,I7.1,A,I7.3)") 'trace: ',jj,' out of',n2*n3*n4*n5*n6
    buffer1=0.
    buffer2=0.
    buffer3=0.
    
    ! Read input stream
    call sreed('in' ,datain,4*n1)
    
    ! If trace is empty, this will be quick:
    if (sum(abs(datain)).eq.0.) then
      select case(output)
        case(1)
          buffer3=0.
          do ib=1,nb
            call srite('out',buffer3,4*nw/2/j1)
          end do
          call srite('out',buffer3,4*nw/2/j1)
        case(2)
          buffer2=0.
          do ib=1,nb
            call srite('out',buffer2,4*nw/j1)
          end do
        case(3)
          buffer3=0.
          call srite('out',buffer3,4*nw/2/j1)
          do ib=1,nb
            call srite('out',buffer3,4*nw/2/j1)
            call srite('out',buffer3,4*nw/2/j1)
          end do
          call srite('out',buffer3,4*nw/2/j1)
      end select
      cycle
    end if  
    
    
    ! Atempt to replace unexpected zeros by the mean value (based on non-zero values)
    where (    datain .ne.     datain ) datain=0.
    where (abs(datain).gt.huge(datain)) datain=0.
    mean = sum ( dble(datain) )/dble(n1)
    clip =  (quantile(nint(q*nw),datain)-quantile(nint((1.-q)*nw),datain))/2.
    if (abs(dble(clip)).lt.abs(mean)) then
      mean = 0; ij=0;
      do ii=2,n1
        if (datain(ii).ne.0) then
          mean = mean + dble(datain(ii))
          ij=ij+1
        end if
      end do
      mean = real( mean/dble(ij) )
      where (datain.eq.0) datain = real(mean)
    end if
    
    ! Do this over a buffer
    do ib=1,nb
      buffer1 = 0.
      buffer2 = 0.
      a =  1+(ib-1)*nw/2
      b = nw+(ib-1)*nw/2
      
      ! Copy from instream
      buffer1 = datain(a:b)
      
      ! Calculate running mean
      mean    = sum ( dble(buffer1) )/dble(nw)
      buffer1 = real( dble(buffer1) - mean )
      buffer1 = weight*buffer1
      
      !!! Bandpass the data
      temp = buffer1(1:nw/2); buffer1(1:nw/2) = buffer1(nw/2+1:nw); buffer1(nw/2+1:nw) = temp
      call sfftw_execute( fft_for )
      bufferf = taper*bufferf
      call sfftw_execute( fft_inv )
      temp = buffer1(nw/2+1:nw); buffer1(nw/2+1:nw) = buffer1(1:nw/2); buffer1(1:nw/2) = temp
      
      !!! Restitch with buffer for old part
      call subsample(buffer1,buffer2,j1)
      select case(output)
        case(1)
          buffer3 = buffer2(        1:nw/2/j1)+buffer3
          call srite('out',buffer3, 4*nw/2/j1)
          buffer3 = buffer2(nw/2/j1+1:nw  /j1)
          if (ib.eq.nb) call srite('out',buffer3,4*nw/2/j1)
        case(2)
          call srite('out',buffer2,4*nw/j1)
        case(3)
          if (ib.eq. 1) then
            buffer3 = buffer2(1:nw/2/j1)+buffer3
            call srite('out',buffer3,4*nw/2/j1)
            buffer3 = buffer2(nw/2/j1+1:nw/j1)
            cycle
          end if
          buffer3 = buffer2(1:nw/2/j1)+buffer3
          call srite('out',buffer3, 4*nw/2/j1)
          call srite('out',buffer3, 4*nw/2/j1)
          buffer3 = buffer2(nw/2/j1+1:nw/j1)
          if (ib.eq.nb) then
            call srite('out',buffer3,4*nw/2/j1)
          end if
      end select
    end do
    
  end do
  
  
  ! Deallocate memory
  if (verbose) write(0,*) '-----------------------------------'
  deallocate(datain,taper,weight,buffer1,buffer2,buffer3)
  call exit(0)
  
end program
