module tp_trans_irr_mod

  use reshapes_mod

  implicit none

  real   , private      :: dt3d,dx3d,dy3d,dtau3d,dpx,dpy,ot3d,ox3d,oy3d,otau3d,opx,opy
  integer, private      :: nt3d,nxy,ntau3d,npx,npy
  real,allocatable      :: t3d(:),x3d(:),y3d(:),tau3d(:),px(:),py(:)
  
  real, dimension(:,:)  , allocatable,private   :: data2
  real, dimension(:,:,:), allocatable,private   :: modl3

  include "openmp.inc"

  private
  public :: tp_trans_irr_init, tp_trans_irr_close, tp_trans_irr_lop, tp_trans_irr_op,tp_trans_irr_adj

  contains
  
  


  subroutine tp_trans_irr_init(nt_in,dt_in,ot_in,nxy_in,dataxy,ntau_in,dtau_in,otau_in,npx_in,dpx_in,opx_in,npy_in,dpy_in,opy_in)
    real                   ::       dt_in,ot_in,                          dtau_in,otau_in,       dpx_in,opx_in,       dpy_in,opy_in
    integer                :: nt_in,                nxy_in,       ntau_in,                npx_in,              npy_in
    integer                :: it,ixy,itau,ipx,ipy
    real,allocatable       ::                              dataxy(:,:)

    call tp_trans_irr_close()
    include "openmp_init.inc"

    ! Get axis parameters
      nt3d = nt_in;        dt3d =   dt_in;      ot3d =   ot_in
    ntau3d = ntau_in;    dtau3d = dtau_in;    otau3d = otau_in   
      nxy  = nxy_in
      npx  = npx_in;       dpx  =  dpx_in;      opx  =  opx_in  
      npy  = npy_in;       dpy  =  dpy_in;      opy  =  opy_in  
    

    allocate(  data2(nt3d,nxy) )
    allocate(  modl3(ntau3d,npx,npy) )

    ! Allocate axis memory variables
    allocate(  t3d(nt3d) )
    allocate(  x3d(nxy) )
    allocate(  y3d(nxy) )
    allocate( tau3d(ntau3d) )
    allocate(   px(npx) )
    allocate(   py(npy) )
    
    ! Calculate Axes
    do   it=1,nt3d;   t3d(it)    =(  it-1)*dt3d  +ot3d;   end do;
    do itau=1,ntau3d; tau3d(itau)=(itau-1)*dtau3d+otau3d; end do;
    do  ipx=1,npx;     px(ipx)   =( ipx-1)*dpx   +opx;   end do;
    do  ipy=1,npy;     py(ipy)   =( ipy-1)*dpy   +opy;   end do;
    
    ! Split x and y axis for input data
    x3d = dataxy(1,:)
    y3d = dataxy(2,:)
    
  !  write(0,*) 'x3d:',x3d
  !  write(0,*) 'y3d:',y3d

  end subroutine
  
  
  subroutine tp_trans_irr_close()
    if (allocated(t3d))   then;  deallocate(t3d);    end if;
    if (allocated(x3d))   then;  deallocate(x3d);    end if;
    if (allocated(y3d))   then;  deallocate(y3d);    end if;
    if (allocated(tau3d)) then;  deallocate(tau3d);  end if;
    if (allocated(px))    then;  deallocate(px);     end if;
    if (allocated(py))    then;  deallocate(py);     end if;
    if (allocated(data2)) then;  deallocate(data2);  end if;
    if (allocated(modl3)) then;  deallocate(modl3);  end if;
  end subroutine



  ! Generic operators for SEP solvers
  function tp_trans_irr_lop(adj,add,m,d) result(stat)
    integer               :: stat
    logical,intent(in)               :: adj,add
    real                 ::          d(:),m(:)
    call adj_null(adj,add,m,d)
    if (.not.adj) call tp_trans_irr_for(m,d)
    if (     adj) call tp_trans_irr_adj(m,d)
    stat=0
  end function
  subroutine tp_trans_irr_op(adj,add,m,d)
    logical,intent(in)               :: adj,add
    real                 ::          d(:),m(:)
    call adj_null(adj,add,m,d)
    if (.not.adj) call tp_trans_irr_for(m,d)
    if (     adj) call tp_trans_irr_adj(m,d)
  end subroutine



  subroutine tp_trans_irr_adj(m,d)
    real                 ::          d(:),m(:)
    integer              ::  it,ixy,itau,ipx,ipy
    
    modl3=0.;data2=0.

    ! Perform Transform by pushing
    call conv1dto2d(d,data2,nt3d,nxy)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(modl3,data2,t3d,x3d,y3d,px,py,dtau3d,npx,npy,nt3d,nxy,ntau3d)
    do ipy = 1,npy
    do ipx = 1,npx
      do it = 1,nt3d
        do ixy = 1,nxy
          itau = ( t3d(it) + px(ipx)*x3d(ixy) + py(ipy)*y3d(ixy) ) / dtau3d + .5
          if (itau.ge.1 .and. itau.le.ntau3d) then
            modl3(itau,ipx,ipy) = modl3(itau,ipx,ipy) + data2(it,ixy)
          end if
    end do;  end do;  end do;  end do
    !$OMP END PARALLEL DO
    call conv3dto1d(modl3,m,ntau3d,npx,npy)

  end subroutine

  subroutine tp_trans_irr_for(m,d)
    real                 ::          d(:),m(:)
    integer              ::  it,ixy,itau,ipx,ipy
    
    modl3=0.;data2=0.
    
    ! Perform Transform by pushing
    call conv1dto3d(m,modl3,ntau3d,npx,npy)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(modl3,data2,t3d,x3d,y3d,px,py,dtau3d,npx,npy,nt3d,nxy,ntau3d)
    do ipy = 1,npy
    do ipx = 1,npx
      do it = 1,nt3d
        do ixy = 1,nxy
          itau = ( t3d(it) + px(ipx)*x3d(ixy) + py(ipy)*y3d(ixy) ) / dtau3d + .5
          if (itau.ge.1 .and. itau.le.ntau3d) then
           ! d(itau,ipx,ipy) = d(itau,ipx,ipy) + m(it,ixy)
            data2(it,ixy) = data2(it,ixy) + modl3(itau,ipx,ipy)
          end if
    end do;  end do;  end do;  end do 
    !$OMP END PARALLEL DO
    call conv2dto1d(data2,d,nt3d,nxy)

  end subroutine
  
  
  subroutine adj_null(adj,add,m,d)
    logical        :: adj,add
    real                 ::          d(:),m(:)
    if (.not.add) then;
      if (     adj)  m = 0.
      if (.not.adj)  d = 0.
    end if
  end subroutine
  
end module
