module average_mod_omp

  implicit none

  interface average_1_op
    module procedure average_1_op_1d_real
    module procedure average_1_op_2d_real
    module procedure average_1_op_3d_real
    module procedure average_1_op_1d_cmplx
    module procedure average_1_op_2d_cmplx
    module procedure average_1_op_3d_cmplx
  end interface

  interface average_2_op
    module procedure average_2_op_2d_real
    module procedure average_2_op_3d_real
    module procedure average_2_op_2d_cmplx
    module procedure average_2_op_3d_cmplx
  end interface

  interface average_3_op
    module procedure average_3_op_3d_real
    module procedure average_3_op_3d_cmplx
  end interface

  interface average_12_op
    module procedure average_12_op_2d_real
    module procedure average_12_op_3d_real
    module procedure average_12_op_2d_cmplx
    module procedure average_12_op_3d_cmplx
  end interface

  interface average_13_op
    module procedure average_13_op_3d_real
    module procedure average_13_op_3d_cmplx
  end interface

  interface average_23_op
    module procedure average_23_op_3d_real
    module procedure average_23_op_3d_cmplx
  end interface


  include "openmp.inc"

  contains

  subroutine average_init()
    call average_close()
    include "openmp_init.inc"
  end subroutine

  subroutine average_close()
  end subroutine

  subroutine average_1_op_1d_real(data_in   ,data_out)
    real,intent(in)       :: data_in(:)
    real,intent(out)      :: data_out
    integer               :: n1
    real(8)               :: sum

    n1 = size(data_in,1)
      data_out        = real( sum(dble(data_in         ))/dble(n1) )

  end subroutine

  subroutine average_1_op_2d_real(data_in     ,data_out)
    real,intent(in)       :: data_in(:,:)
    real,intent(out)      :: data_out(:)
    integer               :: n1,n2,jj
    real(8)               :: sum

    n1 = size(data_in,1)
    n2 = size(data_in,2)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2)
    do jj=1,n2
      data_out(jj   ) = real( sum(dble(data_in(:,jj   )))/dble(n1) )
    end do
    !$OMP END PARALLEL DO

  end subroutine

  subroutine average_1_op_3d_real(data_in       ,data_out)
    real,intent(in)       :: data_in(:,:,:)
    real,intent(out)      :: data_out(:,:)
    integer               :: n1,n2,jj,n3,kk
    real(8)               :: sum

    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2,n3)
    do kk=1,n3
    do jj=1,n2
      data_out(jj,kk) = real( sum(dble(data_in(:,jj,kk)))/dble(n1) )
    end do
    end do
    !$OMP END PARALLEL DO

  end subroutine







  subroutine average_2_op_2d_real(data_in     ,data_out)
    real,intent(in)       :: data_in(:,:)
    real,intent(out)      :: data_out(:)
    integer               :: n1,ii,n2
    real(8)               :: sum

    n1 = size(data_in,1)
    n2 = size(data_in,2)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2)
    do ii=1,n1
      data_out(ii   ) = real( sum(dble(data_in(ii,:   )))/dble(n2) )
    end do
    !$OMP END PARALLEL DO

  end subroutine

  subroutine average_2_op_3d_real(data_in       ,data_out)
    real,intent(in)       :: data_in(:,:,:)
    real,intent(out)      :: data_out(:,:)
    integer               :: n1,ii,n2,n3,kk
    real(8)               :: sum

    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2,n3)
    do kk=1,n3
    do ii=1,n1
      data_out(ii,kk) = real( sum(dble(data_in(ii,:,kk)))/dble(n2) )
    end do
    end do
    !$OMP END PARALLEL DO

  end subroutine
  
  
  


  subroutine average_3_op_3d_real(data_in       ,data_out)
    real,intent(in)       :: data_in(:,:,:)
    real,intent(out)      :: data_out(:,:)
    integer               :: n1,ii,n2,jj,n3
    real(8)               :: sum

    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2,n3)
    do jj=1,n2
    do ii=1,n1
      data_out(ii,jj) = real( sum(dble(data_in(ii,jj,:)))/dble(n3) )
    end do
    end do
    !$OMP END PARALLEL DO

  end subroutine






  subroutine average_12_op_2d_real(data_in    ,data_out)
    real,intent(in)       :: data_in(:,:)
    real,intent(out)      :: data_out
    integer               :: n1,ii,n2,jj,n3,kk
    real(8)               :: sum

    n1 = size(data_in,1)
    n2 = size(data_in,2)
      data_out     = real( sum(sum(dble(data_in        ),1),1) / dble(n1*n2) )

  end subroutine

  
  subroutine average_12_op_3d_real(data_in      ,data_out)
    real,intent(in)       :: data_in(:,:,:)
    real,intent(out)      :: data_out(:)
    integer               :: n1,ii,n2,jj,n3,kk
    real(8)               :: sum

    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2,n3)
    do kk=1,n3
      data_out(kk) = real( sum(sum(dble(data_in(:,:,kk)),1),1) / dble(n1*n2) )
    end do
    !$OMP END PARALLEL DO

  end subroutine
  



  
  

  subroutine average_13_op_3d_real(data_in      ,data_out)
    real,intent(in)       :: data_in(:,:,:)
    real,intent(out)      :: data_out(:)
    integer               :: n1,ii,n2,jj,n3,kk
    real(8)               :: sum
    
    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2,n3)
    do jj=1,n2
      data_out(jj) = real( sum(sum(dble(data_in(:,jj,:)),1),1) / dble(n1*n3) )
    end do
    !$OMP END PARALLEL DO
    
  end subroutine
  



  

  subroutine average_23_op_3d_real(data_in      ,data_out)
    real,intent(in)       :: data_in(:,:,:)
    real,intent(out)      :: data_out(:)
    integer               :: n1,ii,n2,jj,n3,kk
    real(8)               :: sum
    
    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2,n3)
    do jj=1,n1
      data_out(jj) = real( sum(sum(dble(data_in(ii,:,:)),1),1) / dble(n2*n3) )
    end do
    !$OMP END PARALLEL DO
    
  end subroutine

















  subroutine average_1_op_1d_cmplx(data_in   ,data_out)
    complex,intent(in)       :: data_in(:)
    complex,intent(out)      :: data_out
    integer               :: n1
    complex(8)               :: sum

    n1 = size(data_in,1)
      data_out        = cmplx( sum(dble(data_in         ))/dble(n1) )

  end subroutine

  subroutine average_1_op_2d_cmplx(data_in     ,data_out)
    complex,intent(in)       :: data_in(:,:)
    complex,intent(out)      :: data_out(:)
    integer               :: n1,n2,jj
    complex(8)               :: sum

    n1 = size(data_in,1)
    n2 = size(data_in,2)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2)
    do jj=1,n2
      data_out(jj   ) = cmplx( sum(dble(data_in(:,jj   )))/dble(n1) )
    end do
    !$OMP END PARALLEL DO

  end subroutine

  subroutine average_1_op_3d_cmplx(data_in       ,data_out)
    complex,intent(in)       :: data_in(:,:,:)
    complex,intent(out)      :: data_out(:,:)
    integer               :: n1,n2,jj,n3,kk
    complex(8)               :: sum

    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2,n3)
    do kk=1,n3
    do jj=1,n2
      data_out(jj,kk) = cmplx( sum(dble(data_in(:,jj,kk)))/dble(n1) )
    end do
    end do
    !$OMP END PARALLEL DO

  end subroutine







  subroutine average_2_op_2d_cmplx(data_in     ,data_out)
    complex,intent(in)       :: data_in(:,:)
    complex,intent(out)      :: data_out(:)
    integer               :: n1,ii,n2
    complex(8)               :: sum

    n1 = size(data_in,1)
    n2 = size(data_in,2)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2)
    do ii=1,n1
      data_out(ii   ) = cmplx( sum(dble(data_in(ii,:   )))/dble(n2) )
    end do
    !$OMP END PARALLEL DO

  end subroutine

  subroutine average_2_op_3d_cmplx(data_in       ,data_out)
    complex,intent(in)       :: data_in(:,:,:)
    complex,intent(out)      :: data_out(:,:)
    integer               :: n1,ii,n2,n3,kk
    complex(8)               :: sum

    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2,n3)
    do kk=1,n3
    do ii=1,n1
      data_out(ii,kk) = cmplx( sum(dble(data_in(ii,:,kk)))/dble(n2) )
    end do
    end do
    !$OMP END PARALLEL DO

  end subroutine
  
  
  


  subroutine average_3_op_3d_cmplx(data_in       ,data_out)
    complex,intent(in)       :: data_in(:,:,:)
    complex,intent(out)      :: data_out(:,:)
    integer               :: n1,ii,n2,jj,n3
    complex(8)               :: sum

    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2,n3)
    do jj=1,n2
    do ii=1,n1
      data_out(ii,jj) = cmplx( sum(dble(data_in(ii,jj,:)))/dble(n3) )
    end do
    end do
    !$OMP END PARALLEL DO

  end subroutine






  subroutine average_12_op_2d_cmplx(data_in    ,data_out)
    complex,intent(in)       :: data_in(:,:)
    complex,intent(out)      :: data_out
    integer               :: n1,ii,n2,jj,n3,kk
    complex(8)               :: sum

    n1 = size(data_in,1)
    n2 = size(data_in,2)
      data_out     = cmplx( sum(sum(dble(data_in        ),1),1) / dble(n1*n2) )

  end subroutine

  
  subroutine average_12_op_3d_cmplx(data_in      ,data_out)
    complex,intent(in)       :: data_in(:,:,:)
    complex,intent(out)      :: data_out(:)
    integer               :: n1,ii,n2,jj,n3,kk
    complex(8)               :: sum

    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2,n3)
    do kk=1,n3
      data_out(kk) = cmplx( sum(sum(dble(data_in(:,:,kk)),1),1) / dble(n1*n2) )
    end do
    !$OMP END PARALLEL DO

  end subroutine
  



  
  

  subroutine average_13_op_3d_cmplx(data_in      ,data_out)
    complex,intent(in)       :: data_in(:,:,:)
    complex,intent(out)      :: data_out(:)
    integer               :: n1,ii,n2,jj,n3,kk
    complex(8)               :: sum
    
    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2,n3)
    do jj=1,n2
      data_out(jj) = cmplx( sum(sum(dble(data_in(:,jj,:)),1),1) / dble(n1*n3) )
    end do
    !$OMP END PARALLEL DO
    
  end subroutine
  



  

  subroutine average_23_op_3d_cmplx(data_in      ,data_out)
    complex,intent(in)       :: data_in(:,:,:)
    complex,intent(out)      :: data_out(:)
    integer               :: n1,ii,n2,jj,n3,kk
    complex(8)               :: sum
    
    n1 = size(data_in,1)
    n2 = size(data_in,2)
    n3 = size(data_in,3)
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(data_out,data_in,n1,n2,n3)
    do jj=1,n1
      data_out(jj) = cmplx( sum(sum(dble(data_in(ii,:,:)),1),1) / dble(n2*n3) )
    end do
    !$OMP END PARALLEL DO
    
  end subroutine









end module
