module bitcompression_mod

  use random_mod
  use adjnull_mod

  integer,private             :: pc,bs

  ! For the scale functions
  real,private                :: scale_scl,scale_dth
  real,private,allocatable    :: r(:)
 
  contains
 
 
  subroutine compression_init(pc_in, bs_in, seed_in)
    integer, optional      :: pc_in, bs_in, seed_in

    if (present(pc_in)) then; pc = pc_in; else; pc = 32          ; end if;
    if (present(bs_in)) then; bs = bs_in; else; bs = bit_size(pc); end if;

  end subroutine


  subroutine compression_close()
    
  end subroutine


  subroutine compression_dither_init( y,seed_in )
    integer, optional      ::           seed_in
    real                   ::         y(:)
    integer                :: n1
    real                   :: fudge_factor

    n1           = size(y)
    fudge_factor = 1.999999
    scale_dth    = fudge_factor*max( abs(maxval(y)),abs(minval(y)), epsilon(y(1) ) )/(2**pc)

    write(0,*) fudge_factor, max( abs(maxval(y)),abs(minval(y)), epsilon(y(1) ) ), (2**pc), scale_dth

    allocate( r(n1) )
    if (present(seed_in)) then;
      call random_init( seed_in )
    else
      call random_init( )
    end if

  end subroutine
  
  
  subroutine compression_dither_close()

    call random_close()
    deallocate( r )

  end subroutine
  
  
  subroutine compression_scale_init(scale_inout,y)
    ! This initialization actually modifies the scale factor it is initialized with.
    ! To cercomvent some issues with float prescision
    real,optional,intent(in) ::              y(:)
    real,intent(inout)       ::     scale_inout
    real                     ::     fudge_factor
    
    if ( present(y) ) then
      ! compte scale
      scale_inout = (2**(bs-1)-1)/max( abs(maxval(y)),abs(minval(y)), epsilon(y(1) ) )
      ! fudge_factor
      fudge_factor = 1.-epsilon(fudge_factor)
      ! Compute Scaling factor to cover machine integer prescision range
      scale_inout = fudge_factor*scale_inout
      scale_scl   = scale_inout
    else
      scale_scl   = scale_inout
    end if

  end subroutine
  
  
  subroutine compression_scale_close( )
    
  end subroutine
  
  
  subroutine compression(adj,add,x,y)
    logical,intent(in)   :: adj,add
    integer              :: x(:), y(:)

    integer              :: ii,jj,kk,ll,xb,yb

    call adjnull(adj,add,x,y)

    ll = 0
    do ii=1,size(y)
      do jj=bs-1,0,-pc
        ll = ll+1

        yb = jj-pc+1
        xb = bs-pc

        if ( adj ) then
          call mvbits(y(ii), yb, pc, x(ll), xb)
        else
          call mvbits(x(ll), xb, pc, y(ii), yb)
        end if

      end do
    end do
    
  end subroutine
  
  
  subroutine compression_r2i(adj,add,x,y)
    logical,intent(in)   ::  adj,add
    integer              ::            y(:)
    real                 ::          x(:)
    
    call adjnull(adj,add,x,y)
    
    if ( adj ) then
      x = x + real( y + 2**(bs-(pc+1)) )
    else
      y = y + int ( x + 2**(bs-(bs+1)) )
    end if
    
  end subroutine

  
  subroutine compression_scale_lop(adj,add,x,y)
    logical,intent(in)   ::        adj,add
    real                 ::                x(:),y(:)
    
    call adjnull(adj,add,x,y)
    
    if ( adj ) then
      ! Inverse scale to cover approximately the full 32bit integere range
      x = y*scale_scl
    else
      ! Scale to cover approximately the full 32bit integere range
      y = x*scale_scl
    end if
    
  end subroutine
  
  
  subroutine compression_scale_fop(inv,add,x,y)
    logical,intent(in)   ::        inv,add
    real                 ::                x(:),y(:)
    
    call adjnull(inv,add,x,y)
    
    if ( inv ) then
      ! Inverse scale to cover approximately the full 32bit integere range
      x = y/scale_scl
    else
      ! Scale to cover approximately the full 32bit integere range
      y = x*scale_scl
    end if
    
  end subroutine
  
  
  subroutine compression_dither(data)
    real,intent(inout)    ::    data(:)
    
    call random(r)              ! Get random numbers
    data = data +  r*scale_dth 
    
  end subroutine


end module
