module collect_mod

  use fftshift_mod

  implicit none

  integer, private                               :: n1,n2,n3,n4,n5

  real(8), private                               :: collectedr
  real(8), private                               :: collectedi
  real(8), private   , allocatable               :: collected1r(:),collected2r(:,:),collected3r(:,:,:),collected4r(:,:,:,:),collected5r(:,:,:,:,:)
  real(8), private   , allocatable               :: collected1i(:),collected2i(:,:),collected3i(:,:,:),collected4i(:,:,:,:),collected5i(:,:,:,:,:)


  interface collect
    module procedure collect_r5
    module procedure collect_r4
    module procedure collect_r3
    module procedure collect_r2
    module procedure collect_r1
    module procedure collect_r
    module procedure collect_c5
    module procedure collect_c4
    module procedure collect_c3
    module procedure collect_c2
    module procedure collect_c1
    module procedure collect_c
  end interface

  interface collect_fftshift1d
    module procedure collect_fftshift1d_r5
    module procedure collect_fftshift1d_r4
    module procedure collect_fftshift1d_r3
    module procedure collect_fftshift1d_r2
    module procedure collect_fftshift1d_r1
  end interface

  interface get_collected
    module procedure get_collected_d1
    module procedure get_collected_d2
    module procedure get_collected_d3
    module procedure get_collected_s1
    module procedure get_collected_s2
    module procedure get_collected_s3
  end interface


  interface get_collected_average
    module procedure get_collected_average_r1
    module procedure get_collected_average_r2
    module procedure get_collected_average_r3
    module procedure get_collected_average_r4
    module procedure get_collected_average_r5
    module procedure get_collected_average_c1
    module procedure get_collected_average_c2
    module procedure get_collected_average_c3
    module procedure get_collected_average_c4
    module procedure get_collected_average_c5
  end interface



  contains

  subroutine collect_init         (n1_in,n2_in,n3_in,n4_in,n5_in,complex_in)
    integer,optional,intent(in) :: n1_in,n2_in,n3_in,n4_in,n5_in
    logical,optional,intent(in) ::                   complex_in
    logical                     ::                   comx
    
    call collect_close()
    
    if (present(     n5_in)) then;   n5 =      n5_in; else;   n5=0;       end if
    if (present(     n4_in)) then;   n4 =      n4_in; else;   n4=0;       end if
    if (present(     n3_in)) then;   n3 =      n3_in; else;   n3=0;       end if
    if (present(     n2_in)) then;   n2 =      n2_in; else;   n2=0;       end if
    if (present(     n1_in)) then;   n1 =      n1_in; else;   n1=0;       end if
    if (present(complex_in)) then; comx = complex_in; else; comx=.false.; end if
    
    if (comx) then
      if (n5.ge.1) then; allocate(collected5r(n1,n2,n3,n4,n5)); end if
      if (n4.ge.1) then; allocate(collected4r(n1,n2,n3,n4   )); end if
      if (n3.ge.1) then; allocate(collected3i(n1,n2,n3      )); end if
      if (n2.ge.1) then; allocate(collected2i(n1,n2         )); end if
      if (n1.ge.1) then; allocate(collected1i(n1            )); end if
      if (n5.ge.1) then; allocate(collected5r(n1,n2,n3,n4,n5)); end if
      if (n4.ge.1) then; allocate(collected4r(n1,n2,n3,n4   )); end if
      if (n3.ge.1) then; allocate(collected3r(n1,n2,n3      )); end if
      if (n2.ge.1) then; allocate(collected2r(n1,n2         )); end if
      if (n1.ge.1) then; allocate(collected1r(n1            )); end if
    else
      if (n5.ge.1) then; allocate(collected5r(n1,n2,n3,n4,n5)); end if
      if (n4.ge.1) then; allocate(collected4r(n1,n2,n3,n4   )); end if
      if (n3.ge.1) then; allocate(collected3r(n1,n2,n3      )); end if
      if (n2.ge.1) then; allocate(collected2r(n1,n2         )); end if
      if (n1.ge.1) then; allocate(collected1r(n1            )); end if
    end if
    
  end subroutine

  subroutine collect_close()
    if (allocated(collected1r)) then;  deallocate(collected1r);  end if
    if (allocated(collected2r)) then;  deallocate(collected2r);  end if
    if (allocated(collected3r)) then;  deallocate(collected3r);  end if
    if (allocated(collected4r)) then;  deallocate(collected4r);  end if
    if (allocated(collected5r)) then;  deallocate(collected5r);  end if
    if (allocated(collected1i)) then;  deallocate(collected1i);  end if
    if (allocated(collected2i)) then;  deallocate(collected2i);  end if
    if (allocated(collected3i)) then;  deallocate(collected3i);  end if
    if (allocated(collected4i)) then;  deallocate(collected4i);  end if
    if (allocated(collected5i)) then;  deallocate(collected5i);  end if
  end subroutine














  subroutine collect_r1(array)
    real(4),intent(in)  ::   array(:)
    collected1r = collected1r + dble(array)
  end subroutine

  subroutine collect_r2(array)
    real(4),intent(in)::   array(:,:)
    collected2r = collected2r + dble(array)
  end subroutine

  subroutine collect_r3(array)
    real(4) ,intent(in)  ::   array(:,:,:)
    collected3r = collected3r + dble(array)
  end subroutine

  subroutine collect_r4(array)
    real(4) ,intent(in)  ::   array(:,:,:,:)
    collected4r = collected4r + dble(array)
  end subroutine

  subroutine collect_r5(array)
    real(4) ,intent(in)  ::   array(:,:,:,:,:)
    collected5r = collected5r + dble(array)
  end subroutine

  subroutine collect_c1(array)
    complex(4),intent(in)  ::    array(:)
    collected1r = collected1r + dble( real(array))
    collected1i = collected1i + dble(aimag(array))
  end subroutine
  
  subroutine collect_c2(array)
    complex(4),intent(in)  ::   array(:,:)
    collected2r = collected2r + dble( real(array))
    collected2i = collected2i + dble(aimag(array))
  end subroutine
  
  subroutine collect_c3(array)
    complex(4) ,intent(in)  ::     array(:,:,:)
    collected3r = collected3r + dble( real(array))
    collected3i = collected3i + dble(aimag(array))
  end subroutine
  
  subroutine collect_c4(array)
    complex(4),intent(in)  ::   array(:,:,:,:)
    collected4r = collected4r + dble( real(array))
    collected4i = collected4i + dble(aimag(array))
  end subroutine
  
  subroutine collect_c5(array)
    complex(4) ,intent(in)  ::     array(:,:,:,:,:)
    collected5r = collected5r + dble( real(array))
    collected5i = collected5i + dble(aimag(array))
  end subroutine


  subroutine collect_r(value)
    real ,intent(in)  ::  value
    collectedr = collectedr + dble( value )
  end subroutine
  
  subroutine collect_c(value)
    complex,intent(in)  ::  value
    collectedr = collectedr + dble( real(value))
    collectedi = collectedi + dble(aimag(value))
  end subroutine




  subroutine collect_fftshift1d_r1(arrayA,arrayB)
    real(4),intent(in)  ::   arrayA(:),arrayB(:)
    collected1r(     1:n1/2) = collected1r(     1:n1/2) + dble(arrayB)
    collected1r(n1/2+1:n1  ) = collected1r(n1/2+1:n1  ) + dble(arrayA)
  end subroutine

  subroutine collect_fftshift1d_r2(arrayA,arrayB)
    real(4),intent(in)::   arrayA(:,:),arrayB(:,:)
    collected2r(      1:n1/2 ,:) = collected2r(      1:n1/2 ,:) + dble(arrayB)
    collected2r( n1/2+1:n1   ,:) = collected2r( n1/2+1:n1   ,:) + dble(arrayA)
  end subroutine

  subroutine collect_fftshift1d_r3(arrayA,arrayB)
    real(4) ,intent(in)  ::   arrayA(:,:,:),arrayB(:,:,:)
    collected3r(      1:n1/2 ,:,:) = collected3r(      1:n1/2 ,:,:) + dble(arrayB)
    collected3r( n1/2+1:n1   ,:,:) = collected3r( n1/2+1:n1   ,:,:) + dble(arrayA)
  end subroutine

  subroutine collect_fftshift1d_r4(arrayA,arrayB)
    real(4),intent(in)::   arrayA(:,:,:,:),arrayB(:,:,:,:)
    collected4r(      1:n1/2 ,:,:,:) = collected4r(      1:n1/2 ,:,:,:) + dble(arrayB)
    collected4r( n1/2+1:n1   ,:,:,:) = collected4r( n1/2+1:n1   ,:,:,:) + dble(arrayA)
  end subroutine

  subroutine collect_fftshift1d_r5(arrayA,arrayB)
    real(4) ,intent(in)  ::   arrayA(:,:,:,:,:),arrayB(:,:,:,:,:)
    collected5r(      1:n1/2 ,:,:,:,:) = collected5r(      1:n1/2 ,:,:,:,:) + dble(arrayB)
    collected5r( n1/2+1:n1   ,:,:,:,:) = collected5r( n1/2+1:n1   ,:,:,:,:) + dble(arrayA)
  end subroutine


  subroutine get_collected_d3(array)
    real(8),intent(out)  ::  array(:,:,:)    
    array = collected3r
  end subroutine
  subroutine get_collected_d2(array)
    real(8),intent(out)  ::  array(:,:)    
    array = collected2r
  end subroutine
  subroutine get_collected_d1(array)
    real(8),intent(out)  ::  array(:)    
    array = collected1r
  end subroutine

  subroutine get_collected_s3(array)
    real(4) ,intent(out)  ::  array(:,:,:)    
    array = real(collected3r)
  end subroutine
  subroutine get_collected_s2(array)
    real(4) ,intent(out)  ::  array(:,:)    
    array = real(collected2r)
  end subroutine
  subroutine get_collected_s1(array)
    real(4),intent(out)  ::    array(:)    
    array = real(collected1r)
  end subroutine

  subroutine get_collected_c3(array)
    complex(4) ,intent(out)  ::   array(:,:,:)
    array = cmplx(real(collected3r),real(collected3i))
  end subroutine
  subroutine get_collected_c2(array)
    complex(4) ,intent(out)  ::  array(:,:)
    array = cmplx(real(collected2r),real(collected2i))
  end subroutine
  subroutine get_collected_c1(array)
    complex(4)   ,intent(out)  ::   array(:)  
    array = cmplx(real(collected1r),real(collected1i))
  end subroutine
  
  
  
  subroutine get_collected_average_r5(array,n)
    real(4) ,intent(out)      ::      array(:,:,:,:,:)
    integer                   ::            n
    array = real(collected5r/dble(n))
  end subroutine
  subroutine get_collected_average_r4(array,n)
    real(4) ,intent(out)      ::      array(:,:,:,:)
    integer                   ::            n
    array = real(collected4r/dble(n))
  end subroutine
  subroutine get_collected_average_r3(array,n)
    real(4) ,intent(out)      ::      array(:,:,:)
    integer                   ::            n
    array = real(collected3r/dble(n))
  end subroutine
  subroutine get_collected_average_r2(array,n)
    real(4),intent(out)      ::     array(:,:)    
    integer                   ::            n
    array = real(collected2r/dble(n))
  end subroutine
  subroutine get_collected_average_r1(array,n)
    real(4),intent(out)      ::        array(:)    
    integer                   ::            n
    array = real(collected1r/dble(n))
  end subroutine
  
  subroutine get_collected_average_c5(array,n)
    complex(4) ,intent(out)      ::     array(:,:,:,:,:)
    integer                   ::            n
    array = cmplx(real(collected5r/dble(n)),real(collected5i/dble(n)))
  end subroutine
  subroutine get_collected_average_c4(array,n)
    complex(4) ,intent(out)      ::     array(:,:,:,:)
    integer                   ::            n
    array = cmplx(real(collected4r/dble(n)),real(collected4i/dble(n)))
  end subroutine
  subroutine get_collected_average_c3(array,n)
    complex(4) ,intent(out)      ::     array(:,:,:)
    integer                   ::            n
    array = cmplx(real(collected3r/dble(n)),real(collected3i/dble(n)))
  end subroutine
  subroutine get_collected_average_c2(array,n)
    complex(4) ,intent(out)      ::      array(:,:)
    integer                   ::            n
    array = cmplx(real(collected2r/dble(n)),real(collected2i/dble(n)))
  end subroutine
  subroutine get_collected_average_c1(array,n)
    complex(4) ,intent(out)      ::       array(:)  
    integer                   ::            n
    array = cmplx(real(collected1r/dble(n)),real(collected1i/dble(n)))
  end subroutine


end module
