  subroutine fftw2d_12_r2c_lop_init(n1_in,n2_in)
    integer,intent(in),optional::n1_in,n2_in
    integer                    ::n1,n2,nk1,nk2
    
    call fftw2d_12_r2c_lop_close()

    if ( present( n1_in) ) then; n1 = n1_in; end if;
    if ( present( n2_in) ) then; n2 = n2_in; end if;
    nk1 = n1/2+1
    nk2 = n2

    ! Prepare IFFTW
    allocate( in_2d_12(n1,n2), out_2d_12(nk1,nk2) )
    
    call sfftw_plan_dft_r2c_2d(planf_2d_12, n1, n2,  in_2d_12, out_2d_12, FFTW_ESTIMATE);
    call sfftw_plan_dft_c2r_2d(plani_2d_12, n1, n2, out_2d_12,  in_2d_12, FFTW_ESTIMATE);
    
  end subroutine


  subroutine fftw2d_12_r2c_lop_close()
    ! TO BE DONE: DESTROY PLANS
    if (allocated( in_2d_12)) then; deallocate( in_2d_12); end if
    if (allocated(out_2d_12)) then; deallocate(out_2d_12); end if
  end subroutine



  subroutine fftw2d_12_r2c_lop_2d(adj,add,m,d)
    logical, intent(in)      :: adj, add
    real                     ::           m(:,:)
    complex                  ::              d(:,:)
    integer                  :: n1,n2
    
    call adjnull( adj, add, m, d )
    
    
    n1 = size(m,1)
    n2 = size(m,2)



    if ( adj ) then
      out_2d_12 = d
      call sfftw_execute( plani_2d_12 )
      m  = m +  in_2d_12/ sqrt( real(n1*n2) )
    else
      in_2d_12 = m
      call sfftw_execute( planf_2d_12 )
      d  = d + out_2d_12/ sqrt( real(n1*n2) )
    end if
    
  end subroutine



  subroutine fftw2d_12_r2c_lop_3d(adj,add,m,d)
    logical, intent(in)      :: adj, add
    real                     ::           m(:,:,:)
    complex                  ::              d(:,:,:)
    integer                  :: n1,n2,n3,kk

    call adjnull( adj, add, m, d )

    n1 = size(m,1)
    n2 = size(m,2)
    n3 = size(m,3)
    if ( adj ) then
      do kk=1,n3
        out_2d_12 = d(:,:,kk)
        call sfftw_execute( plani_2d_12 )
        m(:,:,kk) = m(:,:,kk) +  in_2d_12/ sqrt( real(n1*n2) )
      end do
    else
      do kk=1,n3
        in_2d_12  = m(:,:,kk)
        call sfftw_execute( planf_2d_12 )
        d(:,:,kk) = d(:,:,kk) + out_2d_12/ sqrt( real(n1*n2) )
      end do
    end if

  end subroutine







 subroutine fftw2d_23_r2c_lop_init(n2_in,n3_in)
    integer,intent(in),optional::n2_in,n3_in
    integer                    ::n2,n3,nk2,nk3

    call fftw2d_23_r2c_lop_close()

    if ( present( n2_in) ) then; n2 = n2_in; end if;
    if ( present( n3_in) ) then; n3 = n3_in; end if;
    nk2 = n2/2+1
    nk3 = n3

    ! Prepare IFFTW
    allocate( in_2d_23(n2,n3), out_2d_23(nk2,nk3) )

    call sfftw_plan_dft_r2c_2d(planf_2d_23, n2, n3, in_2d_23, out_2d_23, FFTW_ESTIMATE);
    call sfftw_plan_dft_c2r_2d(planf_2d_23, n2, n3,out_2d_23,  in_2d_23, FFTW_ESTIMATE);
    
  end subroutine


  subroutine fftw2d_23_r2c_lop_close()
    ! TO BE DONE: DESTROY PLANS
    if (allocated( in_2d_23)) then; deallocate( in_2d_23); end if
    if (allocated(out_2d_23)) then; deallocate(out_2d_23); end if
  end subroutine




  subroutine fftw2d_23_r2c_lop_3d(adj,add,m,d)
    logical, intent(in)      :: adj, add
    real                     ::           m(:,:,:)
    complex                  ::              d(:,:,:)
    integer                  :: n1,n2,n3,ii

    call adjnull( adj, add, m, d )

    n1 = size(m,1)
    n2 = size(m,2)
    n3 = size(m,3)
    if ( adj ) then
      do ii=1,n1
        out_2d_23 = d(ii,:,:)
        call sfftw_execute( plani_2d_23 )
        m(ii,:,:) = m(ii,:,:) +  in_2d_23/ sqrt( real(n2*n3) )
      end do
    else
      do ii=1,n1
        in_2d_23  = m(ii,:,:)
        call sfftw_execute( planf_2d_23 )
        d(ii,:,:) = d(ii,:,:) + out_2d_23/ sqrt( real(n2*n3) )
      end do
    end if

  end subroutine






 subroutine fftw2d_13_r2c_lop_init(n1_in,n3_in)
    integer,intent(in),optional::n1_in,n3_in
    integer                    ::n1,n3,nk1,nk3

    call fftw2d_13_r2c_lop_close()

    if ( present( n1_in) ) then; n1 = n1_in; end if;
    if ( present( n3_in) ) then; n3 = n3_in; end if;
    nk1 = n1/2+1
    nk3 = n3

    ! Prepare IFFTW
    allocate( in_2d_13(n1,n3), out_2d_13(nk1,nk3) )

    call sfftw_plan_dft_r2c_2d(planf_2d_13, n1, n3, in_2d_13, out_2d_13, FFTW_ESTIMATE);
    call sfftw_plan_dft_c2r_2d(planf_2d_13, n1, n3,out_2d_13,  in_2d_13, FFTW_ESTIMATE);
    
  end subroutine
  
             
  subroutine fftw2d_13_r2c_lop_close()
    ! TO BE DONE: DESTROY PLANS
    if (allocated( in_2d_13)) then; deallocate( in_2d_13); end if
    if (allocated(out_2d_13)) then; deallocate(out_2d_13); end if
  end subroutine




  subroutine fftw2d_13_r2c_lop_3d(adj,add,m,d)
    logical, intent(in)      :: adj, add
    real                     ::           m(:,:,:)
    complex                  ::              d(:,:,:)
    integer                  :: n1,n2,n3,jj

    call adjnull( adj, add, m, d )

    n1 = size(m,1)
    n2 = size(m,2)
    n3 = size(m,3)
    if ( adj ) then
      do jj=1,n2
        out_2d_13 = d(:,jj,:)
        call sfftw_execute( plani_2d_13 )
        m(:,jj,:) = m(:,jj,:) +  in_2d_13/ sqrt( real(n1*n3) )
      end do
    else
      do jj=1,n2
        in_2d_13  = m(:,jj,:)
        call sfftw_execute( planf_2d_23 )
        d(:,jj,:) = d(:,jj,:) + out_2d_13/ sqrt( real(n1*n3) )
      end do
    end if

  end subroutine
