module fftw_mod
  
  use adjnull_mod
  
  implicit none

  ! Include for FFTW
  include "fftw3.f"
  
  ! NOTES:
  !* Change sign when x direction in 1d FFTW (implement via init, and defaults)

  real, parameter, private                :: pi=3.14159265
  
  ! For FFTW1d
  integer*8,private                       :: planf_1d_1,plani_1d_1
  integer*8,private                       :: planf_1d_2,plani_1d_2
  integer*8,private                       :: planf_1d_3,plani_1d_3
  real, private, allocatable              ::  in_1d_1(:), in_1d_2(:), in_1d_3(:)
  complex, private, allocatable           :: out_1d_1(:),out_1d_2(:),out_1d_3(:)
  
  ! For FFTW2d
  integer*8,private                       :: planf_2d_12,plani_2d_12
  integer*8,private                       :: planf_2d_23,plani_2d_23
  integer*8,private                       :: planf_2d_13,plani_2d_13
  real, private, allocatable              ::  in_2d_12(:,:), in_2d_23(:,:), in_2d_13(:,:)
  complex, private, allocatable           :: out_2d_12(:,:),out_2d_23(:,:),out_2d_13(:,:)
  
  ! For FFTW3d
  integer*8,private                       :: planf_3d_1,plani_3d_1
  integer*8,private                       :: planf_3d_2,plani_3d_2
  integer*8,private                       :: planf_3d_3,plani_3d_3
  real, private, allocatable              ::  in_3d_3(:,:,:),  in_3d_2(:,:),  in_3d_1(:)
  complex, private, allocatable           :: out_3d_3(:,:,:), out_3d_2(:,:), out_3d_1(:)


  ! In fftw_1d.inc
  interface fftw1d_1_r2c_lop
    module procedure  fftw1d_1_r2c_lop_1d
    module procedure  fftw1d_1_r2c_lop_2d
    module procedure  fftw1d_1_r2c_lop_3d
  end interface

  interface fftw1d_2_r2c_lop
    module procedure  fftw1d_2_r2c_lop_2d
    module procedure  fftw1d_2_r2c_lop_3d
  end interface

  interface fftw1d_3_r2c_lop
    module procedure  fftw1d_3_r2c_lop_3d
  end interface


  ! In fftw_2d.inc
  interface fftw2d_r2c_lop_init
    module procedure fftw2d_12_r2c_lop_init
  end interface

  interface fftw2d_r2c_lop_close
    module procedure fftw2d_12_r2c_lop_close
  end interface


  interface fftw2d_r2c_lop
    module procedure fftw2d_12_r2c_lop_2d
  end interface

  interface fftw2d_12_r2c_lop
    module procedure fftw2d_12_r2c_lop_2d
    module procedure fftw2d_12_r2c_lop_3d
  end interface

  interface fftw2d_13_r2c_lop
    module procedure fftw2d_13_r2c_lop_3d
  end interface

  interface fftw2d_23_r2c_lop
    module procedure fftw2d_23_r2c_lop_3d
  end interface

  ! In fftw_3d
  interface fftw3d_r2c_lop
    module procedure fftw3d_r2c_lop_1d
    module procedure fftw3d_r2c_lop_2d
    module procedure fftw3d_r2c_lop_3d
  end interface
  
  contains
  
  include "fftw_1d.inc"
  include "fftw_2d.inc"
  include "fftw_3d.inc"
  
  
  
  
  
  
  
  
  subroutine fft_r2c_units_t (adj,nt,dt,nw,dw,nf,df)
    logical,intent(in) :: adj
    integer            ::     nt
    integer, optional  ::           nw,   nf
    real               ::        dt
    real   , optional  ::              dw,   df
    
    if ( adj ) then
      nt = 0
      dt = 0.      
      if ( present ( nf ) ) then
        nt = (nf-1)*2
      end if
      if ( present ( nw ) ) then
        nt = (nw-1)*2
      end if
      if ( present ( df ) ) then
        dt = 1./(nt*df)
      end if
      if ( present ( dw ) ) then
        dt = 2.*pi/(nt*dw)
      end if
      if ( .not. (present(nf).or.present(nw).or.present(df).or.present(dw) ) ) then;
        write(0,*) 'fft_units_t - error  :  no input specified'; call exit();
      end if
    else
      if ( present ( nf ) ) then
        nf = nt/2+1
      end if
      if ( present ( nw ) ) then
        nw = nt/2+1
      end if
      if ( present ( df ) ) then
        df = 1./(nt*dt)
      end if
      if ( present ( dw ) ) then
        dw = 2.*pi/(nt*dt)
      end if
      if ( .not. (present(nf).or.present(nw).or.present(df).or.present(dw) ) ) then;
        write(0,*) 'fft_units_t - error  :  no input specified'; call exit();
      end if
    end if
    
  end subroutine


  subroutine fft_r2c_units_x  (adj,nx,dx,nrk,drk,nk,dk)
    logical, intent(in)     :: adj
    integer                 ::     nx
    integer, optional       ::           nrk,    nk
    real                    ::        dx
    real   , optional       ::               drk,   dk
    
    if ( adj ) then
      nx = 0
      dx = 0.
      if ( present ( nk ) ) then
        nx = (nk-1)*2
      end if
      if ( present ( nrk ) ) then
        nx = (nk-1)*2
      end if
      if ( present ( dk ) ) then
        dx = 1./(nx*dk)
      end if
      if ( present ( drk ) ) then
        dx = 2.*pi/(nx*drk)
      end if
      if ( .not. (present(nk).or.present(nrk).or.present(dk).or.present(drk) ) ) then;
        write(0,*) 'fft_units_x - error  :  no input specified'; call exit();
      end if
    else
      if ( present ( nk ) ) then
        nk  = nx/2+1
      end if
      if ( present ( nrk ) ) then
        nrk = nx/2+1
      end if
      if ( present ( dk  ) ) then
        dk  = 1./(nx*dx)
      end if
      if ( present ( drk ) ) then
        drk = 2.*pi/(nx*dx)
      end if
      if ( .not. (present(nk).or.present(nk).or.present(dk).or.present(drk) ) ) then
        write(0,*) 'fft_units_x - error  :  no input specified'; call exit();
      end if
    end if
    
  end subroutine


  subroutine fft_c2c_units_t (adj,nt,dt,nw,dw,nf,df)
    logical,intent(in) :: adj
    integer            ::     nt
    integer, optional  ::           nw,   nf
    real               ::        dt
    real   , optional  ::              dw,   df
    
    if ( adj ) then
      nt = 0
      dt = 0.      
      if ( present ( nf ) ) then
        nt = nf
      end if
      if ( present ( nw ) ) then
        nt = nw
      end if
      if ( present ( df ) ) then
        dt = 1./(nt*df)
      end if
      if ( present ( dw ) ) then
        dt = 2.*pi/(nt*dw)
      end if
      if ( .not. (present(nf).or.present(nw).or.present(df).or.present(dw) ) ) then;
        write(0,*) 'fft_units_t - error  :  no input specified'; call exit();
      end if
    else
      if ( present ( nf ) ) then
        nf = nt
      end if
      if ( present ( nw ) ) then
        nw = nt
      end if
      if ( present ( df ) ) then
        df = 1./(nt*dt)
      end if
      if ( present ( dw ) ) then
        dw = 2.*pi/(nt*dt)
      end if
      if ( .not. (present(nf).or.present(nw).or.present(df).or.present(dw) ) ) then;
        write(0,*) 'fft_units_t - error  :  no input specified'; call exit();
      end if
    end if
    
  end subroutine


  subroutine fft_c2c_units_x  (adj,nx,dx,nrk,drk,nk,dk)
    logical, intent(in)     :: adj
    integer                 ::     nx
    integer, optional       ::           nrk,    nk
    real                    ::        dx
    real   , optional       ::               drk,   dk
    
    if ( adj ) then
      nx = 0
      dx = 0.
      if ( present ( nk ) ) then
        nx = nk
      end if
      if ( present ( nrk ) ) then
        nx = nk
      end if
      if ( present ( dk ) ) then
        dx = 1./(nx*dk)
      end if
      if ( present ( drk ) ) then
        dx = 2.*pi/(nx*drk)
      end if
      if ( .not. (present(nk).or.present(nrk).or.present(dk).or.present(drk) ) ) then;
        write(0,*) 'fft_units_x - error  :  no input specified'; call exit();
      end if
    else
      if ( present ( nk ) ) then
        nk  = nx
      end if
      if ( present ( nrk ) ) then
        nrk = nx
      end if
      if ( present ( dk  ) ) then
        dk  = 1./(nx*dx)
      end if
      if ( present ( drk ) ) then
        drk = 2.*pi/(nx*dx)
      end if
      if ( .not. (present(nk).or.present(nk).or.present(dk).or.present(drk) ) ) then
        write(0,*) 'fft_units_x - error  :  no input specified'; call exit();
      end if
    end if
    
  end subroutine

end module
