module igrad2_mod
  implicit none
  private

  public  :: igrad2_init, igrad2_close, igrad2_lop

  integer,private :: nx,ny

  contains
  
  subroutine igrad2_init(n1,n2)
    integer   :: n1,n2
    call igrad2_close()
    nx=n1; ny=n2
  end subroutine
  subroutine igrad2_close()
  end subroutine
  
  function igrad2_lop(adj, add, x, y)  result(stat)
    integer            :: stat 
    logical,intent(in) :: adj, add 
    real,dimension(:)  :: x,y
    if( .not. add) then
      if(.not.adj)  y = 0.
      if(     adj)  x = 0.
    end if
    if ( adj) then
      call igrad2_adj(x,y)
    else
      call igrad2_for(x,y)
    end if
    stat=0
  end function

  subroutine igrad2_adj(x, y )
    real                     :: x(nx,ny  ) 
    real                     :: y(nx,ny,2)
    integer                  :: ix,iy
    do ix=1,nx-1
      do iy=1,ny-1
        x(ix+1,iy  ) = x(ix+1,iy  ) + y(ix,iy,1)
        x(ix  ,iy  ) = x(ix  ,iy  ) - y(ix,iy,1)
        x(ix+1,iy+1) = x(ix+1,iy+1) + y(ix,iy,2)
        x(ix  ,iy  ) = x(ix  ,iy  ) - y(ix,iy,2)
      end do
    end do
  end subroutine 



  subroutine igrad2_for( x, y )
    real                     :: x(nx,ny  ) 
    real                     :: y(nx,ny,2)
    integer                  :: ix,iy
    do ix=1,nx-1
      do iy=1,ny-1
        y(ix ,iy ,1) = y(ix ,iy ,1) + (x(ix+1 ,iy) - x(ix ,iy))
        y(ix ,iy ,2) = y(ix ,iy ,2) + (x(ix ,iy+1) - x(ix ,iy))
      end do
    end do
  end subroutine 


end module
