module inter_mod
  ! Module to calculate time domain 
  ! For simple interferometry tests from command line wrapper
  ! It is reccomended to write most programs seperating the FFTW and correlation steps
  ! Instead of doing it trace by trace
  
  use spectra_mod
  use mtspectra_mod
  
  implicit none
  
  integer            ,private       :: n1,n2,n3,nf,nt
  real               ,private       :: tbp
  complex,allocatable,private       :: aw(:,:,:),bw(:,:,:),cw(:,:,:)
  
  interface inter_crsp
    module procedure inter_crsp_1d_1d
    module procedure inter_crsp_2d_1d
    module procedure inter_crsp_3d_1d
    module procedure inter_crsp_1d_2d
    module procedure inter_crsp_1d_3d
    module procedure inter_crsp_2d_2d
    module procedure inter_crsp_3d_3d
  end interface
  
  interface inter_corr
    module procedure inter_corr_1d_1d
    module procedure inter_corr_2d_1d
    module procedure inter_corr_3d_1d
    module procedure inter_corr_1d_2d
    module procedure inter_corr_1d_3d
    module procedure inter_corr_2d_2d
    module procedure inter_corr_3d_3d
  end interface

  interface inter_decv
    module procedure inter_decv_1d_1d
    module procedure inter_decv_2d_1d
    module procedure inter_decv_3d_1d
    module procedure inter_decv_1d_2d
    module procedure inter_decv_1d_3d
    module procedure inter_decv_2d_2d
    module procedure inter_decv_3d_3d
  end interface



  
  interface inter_mtcrsp
    module procedure inter_mtcrsp_1d_1d
    module procedure inter_mtcrsp_2d_1d
    module procedure inter_mtcrsp_3d_1d
    module procedure inter_mtcrsp_1d_2d
    module procedure inter_mtcrsp_1d_3d
    module procedure inter_mtcrsp_2d_2d
    module procedure inter_mtcrsp_3d_3d
  end interface
  
  interface inter_mtcorr
    module procedure inter_mtcorr_1d_1d
    module procedure inter_mtcorr_2d_1d
    module procedure inter_mtcorr_3d_1d
    module procedure inter_mtcorr_1d_2d
    module procedure inter_mtcorr_1d_3d
    module procedure inter_mtcorr_2d_2d
    module procedure inter_mtcorr_3d_3d
  end interface

  interface inter_mtdecv
    module procedure inter_mtdecv_1d_1d
    module procedure inter_mtdecv_2d_1d
    module procedure inter_mtdecv_3d_1d
    module procedure inter_mtdecv_1d_2d
    module procedure inter_mtdecv_1d_3d
    module procedure inter_mtdecv_2d_2d
    module procedure inter_mtdecv_3d_3d
  end interface
  




  contains
  
  subroutine inter_init       (n1_in)
    integer,intent(in),optional  ::  n1_in
    if (present( n1_in)) then;  n1= n1_in; else;  n1 = 1;  end if
    nf = n1/2+1
    call    fftw1d_1_r2c_lop_init(n1)
  end subroutine
  
  subroutine inter_mt_init       (n1_in,nt_in,tbp_in)
    integer,intent(in),optional  ::  n1_in,nt_in
    real   ,intent(in),optional  :: tbp_in
    if (present( n1_in)) then;  n1= n1_in; else;  n1 = 1;  end if
    if (present( nt_in)) then;  nt= nt_in; else;  nt = 1;  end if
    if (present(tbp_in)) then; tbp=tbp_in; else; tbp = 10; end if
    nf = n1/2+1
    call mt_fftw_r2c_lop_init    (nt_in=n1,nk_in=nt,tbp_in=tbp)
  end subroutine


  subroutine inter_close ( )
    call fftw1d_1_r2c_lop_close( )
  end subroutine
  

  subroutine inter_mt_close ( )
    call  mt_fftw_r2c_lop_close( )
  end subroutine

  
  subroutine inter_crsp_1d_1d(a,b,c)
    real,intent(in)        :: a(:),b(:)
    real,intent(out)       ::      c(:)
    complex,allocatable    :: aw(:),bw(:),cw(:)
    allocate   ( aw(nf), bw(nf), cw(nf) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_crsp(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine
  
  subroutine inter_crsp_1d_2d(a,b,c)
    real,intent(in)        :: a(:),b(:,:)
    real,intent(out)       ::      c(:,:)
    complex,allocatable    :: aw(:),bw(:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf), bw(nf,n2), cw(nf,n2) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_crsp(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_crsp_1d_3d(a,b,c)
    real,intent(in)        :: a(:),b(:,:,:)
    real,intent(out)       ::      c(:,:,:)
    complex,allocatable    :: aw(:),bw(:,:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf), bw(nf,n2,n3), cw(nf,n2,n3) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_crsp(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine
  
  subroutine inter_crsp_2d_1d(a,b,c)
    real,intent(in)        :: a(:,:),b(:)
    real,intent(out)       ::        c(:,:)
    complex,allocatable    :: aw(:,:),bw(:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,n2), bw(nf), cw(nf,n2) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_crsp(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_crsp_3d_1d(a,b,c)
    real,intent(in)        :: a(:,:,:),b(:)
    real,intent(out)       ::        c(:,:,:)
    complex,allocatable    :: aw(:,:,:),bw(:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,n2,n3), bw(nf), cw(nf,n2,n3) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_crsp(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_crsp_2d_2d(a,b,c)
    real,intent(in)        :: a(:,:),b(:,:)
    real,intent(out)       ::        c(:,:)
    complex,allocatable    :: aw(:,:),bw(:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,n2), bw(nf,n2), cw(nf,n2) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_crsp(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_crsp_3d_3d(a,b,c)
    real,intent(in)        :: a(:,:,:),b(:,:,:)
    real,intent(out)       ::        c(:,:,:)
    complex,allocatable    :: aw(:,:,:),bw(:,:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,n2,n3), bw(nf,n2,n3), cw(nf,n2,n3) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_crsp(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine


  subroutine inter_cohr_1d_1d(a,b,c)
    real,intent(in)        :: a(:),b(:)
    real,intent(out)       ::      c(:)
    complex,allocatable    :: aw(:),bw(:),cw(:)
    allocate   ( aw(nf), bw(nf), cw(nf) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_cohr(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine
  
  subroutine inter_cohr_1d_2d(a,b,c)
    real,intent(in)        :: a(:),b(:,:)
    real,intent(out)       ::      c(:,:)
    complex,allocatable    :: aw(:),bw(:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf), bw(nf,n2), cw(nf,n2) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_cohr(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_cohr_1d_3d(a,b,c)
    real,intent(in)        :: a(:),b(:,:,:)
    real,intent(out)       ::      c(:,:,:)
    complex,allocatable    :: aw(:),bw(:,:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf), bw(nf,n2,n3), cw(nf,n2,n3) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_cohr(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine
  
  subroutine inter_cohr_2d_1d(a,b,c)
    real,intent(in)        :: a(:,:),b(:)
    real,intent(out)       ::        c(:,:)
    complex,allocatable    :: aw(:,:),bw(:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,n2), bw(nf), cw(nf,n2) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_cohr(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_cohr_3d_1d(a,b,c)
    real,intent(in)        :: a(:,:,:),b(:)
    real,intent(out)       ::          c(:,:,:)
    complex,allocatable    :: aw(:,:,:),bw(:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,n2,n3), bw(nf), cw(nf,n2,n3) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_cohr(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_cohr_2d_2d(a,b,c)
    real,intent(in)        :: a(:,:),b(:,:)
    real,intent(out)       ::        c(:,:)
    complex,allocatable    :: aw(:,:),bw(:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,n2), bw(nf,n2), cw(nf,n2) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_cohr(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_cohr_3d_3d(a,b,c)
    real,intent(in)        :: a(:,:,:),b(:,:,:)
    real,intent(out)       ::          c(:,:,:)
    complex,allocatable    :: aw(:,:,:),bw(:,:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,n2,n3), bw(nf,n2,n3), cw(nf,n2,n3) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_cohr(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine




  subroutine inter_decv_1d_1d(a,b,c)
    real,intent(in)        :: a(:),b(:)
    real,intent(out)       ::      c(:)
    complex,allocatable    :: aw(:),bw(:),cw(:)
    allocate   ( aw(nf), bw(nf), cw(nf) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_decv(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine
  
  subroutine inter_decv_1d_2d(a,b,c)
    real,intent(in)        :: a(:),b(:,:)
    real,intent(out)       ::      c(:,:)
    complex,allocatable    :: aw(:),bw(:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf), bw(nf,n2), cw(nf,n2) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_decv(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_decv_1d_3d(a,b,c)
    real,intent(in)        :: a(:),b(:,:,:)
    real,intent(out)       ::      c(:,:,:)
    complex,allocatable    :: aw(:),bw(:,:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf), bw(nf,n2,n3), cw(nf,n2,n3) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_decv(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine
  
  subroutine inter_decv_2d_1d(a,b,c)
    real,intent(in)        :: a(:,:),b(:)
    real,intent(out)       ::        c(:,:)
    complex,allocatable    :: aw(:,:),bw(:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,n2), bw(nf), cw(nf,n2) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_decv(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_decv_3d_1d(a,b,c)
    real,intent(in)        :: a(:,:,:),b(:)
    real,intent(out)       ::          c(:,:,:)
    complex,allocatable    :: aw(:,:,:),bw(:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,n2,n3), bw(nf), cw(nf,n2,n3) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_decv(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_decv_2d_2d(a,b,c)
    real,intent(in)        :: a(:,:),b(:,:)
    real,intent(out)       ::        c(:,:)
    complex,allocatable    :: aw(:,:),bw(:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,n2), bw(nf,n2), cw(nf,n2) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_decv(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_decv_3d_3d(a,b,c)
    real,intent(in)        :: a(:,:,:),b(:,:,:)
    real,intent(out)       ::          c(:,:,:)
    complex,allocatable    :: aw(:,:,:),bw(:,:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,n2,n3), bw(nf,n2,n3), cw(nf,n2,n3) )
    call fftw1d_1_r2c_lop(.false.,.false.,a,aw)
    call fftw1d_1_r2c_lop(.false.,.false.,b,bw)
    call spectra_decv(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine









  subroutine inter_mtcrsp_1d_1d(a,b,c)
    real,intent(in)        :: a(:),b(:)
    real,intent(out)       ::      c(:)
    complex,allocatable    :: aw(:,:),bw(:,:),cw(:)
    allocate   ( aw(nf,nt), bw(nf,nt), cw(nf) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call mtspectra_crsp(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop ( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine
  
  subroutine inter_mtcrsp_1d_2d(a,b,c)
    real,intent(in)        :: a(:),b(:,:)
    real,intent(out)       ::      c(:,:)
    complex,allocatable    :: aw(:,:),bw(:,:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,nt), bw(nf,nt,n2), cw(nf,n2) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtcrsp(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_mtcrsp_1d_3d(a,b,c)
    real,intent(in)        :: a(:),b(:,:,:)
    real,intent(out)       ::      c(:,:,:)
    complex,allocatable    :: aw(:,:),bw(:,:,:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,nt), bw(nf,nt,n2,n3), cw(nf,n2,n3) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtcrsp(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine
  
  subroutine inter_mtcrsp_2d_1d(a,b,c)
    real,intent(in)        :: a(:,:),b(:)
    real,intent(out)       ::        c(:,:)
    complex,allocatable    :: aw(:,:,:),bw(:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,nt,n2), bw(nf,nt), cw(nf,n2) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtcrsp(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_mtcrsp_3d_1d(a,b,c)
    real,intent(in)        :: a(:,:,:),b(:)
    real,intent(out)       ::        c(:,:,:)
    complex,allocatable    :: aw(:,:,:,:),bw(:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,nt,n2,n3), bw(nf,nt), cw(nf,n2,n3) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtcrsp(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_mtcrsp_2d_2d(a,b,c)
    real,intent(in)        :: a(:,:),b(:,:)
    real,intent(out)       ::        c(:,:)
    complex,allocatable    :: aw(:,:,:),bw(:,:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,nt,n2), bw(nf,nt,n2), cw(nf,n2) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtcrsp(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_mtcrsp_3d_3d(a,b,c)
    real,intent(in)        :: a(:,:,:),b(:,:,:)
    real,intent(out)       ::        c(:,:,:)
    complex,allocatable    :: aw(:,:,:,:),bw(:,:,:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,nt,n2,n3), bw(nf,nt,n2,n3), cw(nf,n2,n3) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtcrsp(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine


  subroutine inter_mtcohr_1d_1d(a,b,c)
    real,intent(in)        :: a(:),b(:)
    real,intent(out)       ::      c(:)
    complex,allocatable    :: aw(:,:),bw(:,:),cw(:)
    allocate   ( aw(nf,nt), bw(nf,nt), cw(nf) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtcohr(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine
  
  subroutine inter_mtcohr_1d_2d(a,b,c)
    real,intent(in)        :: a(:),b(:,:)
    real,intent(out)       ::      c(:,:)
    complex,allocatable    :: aw(:,:),bw(:,:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,nt), bw(nf,nt,n2), cw(nf,n2) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtcohr(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_mtcohr_1d_3d(a,b,c)
    real,intent(in)        :: a(:),b(:,:,:)
    real,intent(out)       ::      c(:,:,:)
    complex,allocatable    :: aw(:,:),bw(:,:,:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,nt), bw(nf,nt,n2,n3), cw(nf,n2,n3) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtcohr(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine
  
  subroutine inter_mtcohr_2d_1d(a,b,c)
    real,intent(in)        :: a(:,:),b(:)
    real,intent(out)       ::        c(:,:)
    complex,allocatable    :: aw(:,:,:),bw(:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,nt,n2), bw(nf,nt), cw(nf,n2) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtcohr(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_mtcohr_3d_1d(a,b,c)
    real,intent(in)        :: a(:,:,:),b(:)
    real,intent(out)       ::          c(:,:,:)
    complex,allocatable    :: aw(:,:,:,:),bw(:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,nt,n2,n3), bw(nf,nt), cw(nf,n2,n3) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtcohr(   aw,  bw,  cw   )
    call fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_mtcohr_2d_2d(a,b,c)
    real,intent(in)        :: a(:,:),b(:,:)
    real,intent(out)       ::        c(:,:)
    complex,allocatable    :: aw(:,:,:),bw(:,:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,nt,n2), bw(nf,nt,n2), cw(nf,n2) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtcohr(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_mtcohr_3d_3d(a,b,c)
    real,intent(in)        :: a(:,:,:),b(:,:,:)
    real,intent(out)       ::          c(:,:,:)
    complex,allocatable    :: aw(:,:,:,:),bw(:,:,:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,nt,n2,n3), bw(nf,nt,n2,n3), cw(nf,n2,n3) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtcohr(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine




  subroutine inter_mtdecv_1d_1d(a,b,c)
    real,intent(in)        :: a(:),b(:)
    real,intent(out)       ::      c(:)
    complex,allocatable    :: aw(:,:),bw(:,:),cw(:)
    allocate   ( aw(nf,nt), bw(nf,nt), cw(nf) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtdecv(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine
  
  subroutine inter_mtdecv_1d_2d(a,b,c)
    real,intent(in)        :: a(:),b(:,:)
    real,intent(out)       ::      c(:,:)
    complex,allocatable    :: aw(:,:),bw(:,:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,nt), bw(nf,nt,n2), cw(nf,n2) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtdecv(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_mtdecv_1d_3d(a,b,c)
    real,intent(in)        :: a(:),b(:,:,:)
    real,intent(out)       ::      c(:,:,:)
    complex,allocatable    :: aw(:,:),bw(:,:,:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,nt), bw(nf,nt,n2,n3), cw(nf,n2,n3) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtdecv(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine
  
  subroutine inter_mtdecv_2d_1d(a,b,c)
    real,intent(in)        :: a(:,:),b(:)
    real,intent(out)       ::        c(:,:)
    complex,allocatable    :: aw(:,:,:),bw(:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,nt,n2), bw(nf,nt), cw(nf,n2) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtdecv(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_mtdecv_3d_1d(a,b,c)
    real,intent(in)        :: a(:,:,:),b(:)
    real,intent(out)       ::          c(:,:,:)
    complex,allocatable    :: aw(:,:,:,:),bw(:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,nt,n2,n3), bw(nf,nt), cw(nf,n2,n3) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtdecv(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_mtdecv_2d_2d(a,b,c)
    real,intent(in)        :: a(:,:),b(:,:)
    real,intent(out)       ::        c(:,:)
    complex,allocatable    :: aw(:,:,:),bw(:,:,:),cw(:,:)
    n2 = size(c,2)
    allocate   ( aw(nf,nt,n2), bw(nf,nt,n2), cw(nf,n2) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtdecv(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine

  subroutine inter_mtdecv_3d_3d(a,b,c)
    real,intent(in)        :: a(:,:,:),b(:,:,:)
    real,intent(out)       ::          c(:,:,:)
    complex,allocatable    :: aw(:,:,:,:),bw(:,:,:,:),cw(:,:,:)
    n2 = size(c,2)
    n3 = size(c,3)
    allocate   ( aw(nf,nt,n2,n3), bw(nf,nt,n2,n3), cw(nf,n2,n3) )
    call mt_fftw1d_r2c_lop(.false.,.false.,a,aw)
    call mt_fftw1d_r2c_lop(.false.,.false.,b,bw)
    call spectra_mtdecv(   aw,  bw,  cw   )
    call  fftw1d_1_r2c_lop( .true.,.false.,c,cw)
    deallocate ( aw    , bw    , cw     )
  end subroutine


end module
