module masker_mod
  ! subroutine to determine and apply a mask to a Data and set to predetermined datavalue
  
  implicit none
  
  integer,private             :: n1,n2
  real,private                :: d1,d2
  real,private                :: o1,o2
  
  real,private                :: value
  real,private                :: rmax
  real,private                :: angrmax

  
  real,private,parameter      :: pi=3.14159265

  contains
  
  subroutine mask_init(n1_in,n2_in,d1_in,d2_in,o1_in,o2_in,rmax_in,angrmax_in,value_in)
    integer          :: n1_in,n2_in
    real             :: d1_in,d2_in,o1_in,o2_in
    real             :: rmax_in,angrmax_in,value_in
    
    ! Underlying map of data
    n1 = n1_in
    n2 = n2_in
    d1 = d1_in
    d2 = d2_in
    o1 = o1_in
    o2 = o2_in
    
    ! Mask criteria
    rmax = rmax_in
    angrmax = angrmax_in
    value = value_in
    
  end subroutine



  subroutine do_mask(data,head_xy)
    real      :: data(:,:),head_xy(:,:)
    integer      :: ntr
    logical      :: mask
    real  :: rad,ang
    integer         :: nang
  integer       :: ii,jj,it

  real                   :: angv(4)


    ntr = size(head_xy,1)


  ! Apply mask
  do jj=1,n2
  do ii=1,n1
    
    mask=.true.
    
    ! Apply the first acceptance condition
   ! write(0,*) "Do first test"
    do it=1,ntr
      
      rad = sqrt ( (o1+(ii- 1   )*d1-head_xy(it,1))**2. + (o2+(jj-1)*d2-head_xy(it,2))**2. )
      
      if ( rad.le.rmax) then
        mask = .false.
      end if
      
    end do

    ! Apply the second acceptance condition
    if (mask) then
    
    nang = 0
    angv = 0.
    
    do it=1,ntr
      rad = sqrt ( (o1+(ii- 1   )*d1-head_xy(it,1))**2. +    (o2+(jj-1)*d2-head_xy(it,2))**2. )
      ang = atan2( (o1+(ii- 1   )*d1-head_xy(it,1))     ,    (o2+(jj-1)*d2-head_xy(it,2))     )
      if (rad.le.angrmax) then
        if ( ang .ge.-1. *pi .and. ang .lt.- .5*pi ) then;  angv(1) = 1;  end if
        if ( ang .ge.- .5*pi .and. ang .lt. 0.     ) then;  angv(2) = 1;  end if
        if ( ang .ge. 0.     .and. ang .lt.  .5*pi ) then;  angv(3) = 1;  end if
        if ( ang .ge.  .5*pi .and. ang .lt.     pi ) then;  angv(4) = 1;  end if
      end if
    end do
    
    
    
    if (sum(angv).eq.4) then
      mask = .false.
    end if
    
    end if
    
    ! Actual masking
    if (mask) then
      data(ii,jj) = value
    end if    
    
  end do
  end do

  
  end subroutine


end module
