module mdtape_mod
  ! Single and multitaper, op's, lop's and fop's
  ! An interface to initialize the tapers in this module is located in mdtapers_mod

  use adjnull_mod
  use addnull_mod
  use fftshift_mod

  implicit none

  real, private,allocatable    :: taper(:,:,:)
  integer,private              :: ii,jj,kk
  integer,private              :: n1,n2,n3

  
  interface mdtape_op
    module procedure mdtape_op_real_1d
    module procedure mdtape_op_real_2d
    module procedure mdtape_op_real_3d
    module procedure mdtape_op_complex_1d
    module procedure mdtape_op_complex_2d
    module procedure mdtape_op_complex_3d
  end interface

  interface mdtape_gettaper
    module procedure mdtape_gettaper_1d
    module procedure mdtape_gettaper_2d
    module procedure mdtape_gettaper_3d
  end interface

  
  contains
  
  
  subroutine mdtape_init            (n1_in,n2_in,n3_in,nt_in)
    integer, intent(in),optional  :: n1_in,n2_in,n3_in,nt_in
    if (present( n1_in)) then; n1  = n1_in;  else; n1 = 1;  end if
    if (present( n2_in)) then; n2  = n2_in;  else; n2 = 1;  end if
    if (present( n3_in)) then; n3  = n3_in;  else; n3 = 1;  end if

    allocate(  taper(n1,n2,n3) )

    taper = 1.

  end subroutine
  
  
  subroutine  mdtape_insert(add, taper_in,t1,t2)
    real   , intent(inout)       :: taper_in(:,:,:)
    logical, intent(in)          :: add
    integer                      :: t1(3),t2(3)
    
    if (.not. size(taper_in) == n1*n2*n3) then; write(0,*) "size(taper_in)!=size(taper) in next call to taper_mod"; call exit(); end if      
    call addnull(add,taper(t1(1):t2(1),t1(2):t2(2),t1(3):t2(3)))

    taper(t1(1):t2(1),t1(2):t2(2),t1(3):t2(3)) = taper(t1(1):t2(1),t1(2):t2(2),t1(3):t2(3)) + taper_in(t1(1):t2(1),t1(2):t2(2),t1(3):t2(3))

  end subroutine
  
  
  subroutine mdtape_shift(shift_in)
    logical,intent(in) ::shift_in(3)
    real,allocatable   :: dummy(:,:,:)


    allocate(dummy(n1,n2,n3))

    if (shift_in(1)) then
      call fftshift1d_1_lop(.false.,.false.,taper,dummy)
      taper=dummy
    end if
    
    if (shift_in(2)) then
      call fftshift1d_2_lop(.false.,.false.,taper,dummy)
      taper=dummy
    end if
    
    if (shift_in(3)) then
      call fftshift1d_3_lop(.false.,.false.,taper,dummy)
      taper=dummy
    end if

    deallocate(dummy)

  end subroutine


  subroutine mdtape_close()

    if (allocated( taper )) then; deallocate( taper ); end if

  end subroutine
  
  
  subroutine mdtape_gettaper_1d(taper_out)
    real,intent(inout)             :: taper_out(:)
    taper_out = taper(:,1,1)
  end subroutine

  subroutine mdtape_gettaper_2d(taper_out)
    real,intent(inout)             :: taper_out(:,:)
    !write(0,*) taper
    taper_out = taper(:,:,1)
  end subroutine

  subroutine mdtape_gettaper_3d(taper_out)
    real,intent(inout)             :: taper_out(:,:,:)
    taper_out = taper
  end subroutine


  subroutine mdtape_op_real_1d(x)
    real, intent(inout) :: x(:)
    
    do ii=1,n1
        x(ii) = taper(ii,1,1)*x(ii)
    end do
    
  end subroutine


  subroutine mdtape_op_real_2d(x)
    real, intent(inout) :: x(:,:)
    
    do jj=1,n2
    do ii=1,n1
        x(ii,jj) = taper(ii,jj,1)*x(ii,jj)
    end do
    end do
    
  end subroutine

  subroutine mdtape_op_real_3d(x)
    real, intent(inout) :: x(:,:,:)
    
    do kk=1,n3
    do jj=1,n2
    do ii=1,n1
        x(ii,jj,kk) = taper(ii,jj,kk)*x(ii,jj,kk)
    end do
    end do
    end do
    
  end subroutine


  subroutine mdtape_op_complex_1d(x)
    complex, intent(inout) :: x(:)
    
    do ii=1,n1
        x(ii) = taper(ii,1,1)*x(ii)
    end do
    
  end subroutine


  subroutine mdtape_op_complex_2d(x)
    complex, intent(inout) :: x(:,:)
    
    do jj=1,n2
    do ii=1,n1
        x(ii,jj) = taper(ii,jj,1)*x(ii,jj)
    end do
    end do
    
  end subroutine

  subroutine mdtape_op_complex_3d(x)
    complex, intent(inout) :: x(:,:,:)
    
    do kk=1,n3
    do jj=1,n2
    do ii=1,n1
        x(ii,jj,kk) = taper(ii,jj,kk)*x(ii,jj,kk)
    end do
    end do
    end do
    
  end subroutine


  subroutine mdtape_lop  (adj,add,x,y)
    logical, intent(in) :: adj,add
    real, intent(inout) ::         x(:,:,:),y(:,:,:)

    call adjnull          (adj,add,x,y)

    if (adj) then
      do kk=1,n3
      do jj=1,n2
      do ii=1,n1
        x(ii,jj,kk) = x(ii,jj,kk) + taper(ii,jj,kk)*y(ii,jj,kk)
      end do
      end do
      end do
    else
      do kk=1,n3
      do jj=1,n2
      do ii=1,n1
        y(ii,jj,kk) = y(ii,jj,kk) + taper(ii,jj,kk)*x(ii,jj,kk)
      end do
      end do
      end do
    end if

  end subroutine
  
  
  
  
  subroutine mdtape_fop    (inv,add,x,y)
    logical, intent(in) :: inv,add
    real, intent(inout) ::         x(:,:,:),y(:,:,:)
    
    call adjnull          (inv,add,x,y)
    
    if (inv) then
      do kk=1,n3
      do jj=1,n2
      do ii=1,n1
        x(ii,jj,kk) = x(ii,jj,kk) + taper(ii,jj,kk)*y(ii,jj,kk)
      end do
      end do
      end do
    else
      do kk=1,n3
      do jj=1,n2
      do ii=1,n1
        y(ii,jj,kk) = y(ii,jj,kk) + y(ii,jj,kk)/taper(ii,jj,kk)
      end do
      end do
      end do
    end if
    
  end subroutine


end module

