module mt_fftw_mod
  ! model is temporal-domain; data is fourier-domain.
  ! it does not make any sense to use the adj=true option.
  ! Evaluate the spectrum, coherence or cross coherence or
  ! deconvolution function, sum over tapers, and use 
  ! fftw_mod to get back to the time domain.
  ! Need be initialized with taper functions
  ! number of tapers nk, nt and nf(nt/2+1)
  
  use adjnull_mod
  use addnull_mod
  use mtapers_mod
  use fftshift_mod
  
  implicit none
  
  ! Include for FFTW
  include "fftw3.f"
  
  integer,           private    :: nt,nf,nk
  real, allocatable, private    :: mtaper(:,:),tbp
  integer*8,private             :: planf,plani
  real   , private,allocatable  :: in(:)
  complex, private,allocatable  :: out(:)
 
  real, parameter,private       :: pi=3.14159265

  interface mt_fftw1d_1_r2c_lop
    module procedure mt_fftw1d_1_r2c_lop_1d
    module procedure mt_fftw1d_1_r2c_lop_2d
    module procedure mt_fftw1d_1_r2c_lop_3d
  end interface
  
contains



subroutine mt_fftw1d_1_r2c_lop_init(nt_in,t1_ind,t2_ind,nk_in,tbp_in,shift_in)
    integer,intent(in),optional ::  nt_in,t1_ind,t2_ind,nk_in
    real   ,intent(in),optional ::                          tbp_in
    logical,intent(in),optional ::  shift_in
    integer                     ::  t1,t2
    real                        ::  scale,tbp
    logical                     ::  shift
    real, allocatable           :: mtaper_sh(:,:)
    call mt_fftw1d_1_r2c_lop_close()
    
    if ( present(    nt_in ) ) then;    nt = nt_in; end if;
    if ( present(    t1_ind) ) then;    t1 = t1_ind;    else;    t1=1;        end if;
    if ( present(    t2_ind) ) then;    t2 = t2_ind;    else;    t2=nt;       end if;
    if ( present(    nk_in ) ) then;    nk = nk_in;     else;    nk=3;        end if;
    if ( present(   tbp_in ) ) then;   tbp = nk_in;     else;   tbp=10.;      end if;
    if ( present( shift_in ) ) then; shift = shift_in;  else; shift=.false.;  end if;

    nf = nt/2+1
    
    ! Prepare IFFTW
    allocate(  in(nt) ); call addnull(.false., in);
    allocate( out(nf) ); call addnull(.false.,out);
    call sfftw_plan_dft_r2c_1d(planf, nt, in , out, FFTW_ESTIMATE);
    call sfftw_plan_dft_c2r_1d(plani, nt, out, in , FFTW_ESTIMATE);
    
    scale = 1.
    
    ! Prepare mtaper
    allocate( mtaper   ( nt, nk ) ); call addnull(.false.,mtaper   );
    allocate( mtaper_sh( nt, nk ) ); call addnull(.false.,mtaper_sh);
    
    call mtapers_init     ( n1_in=(t2-t1+1), nt_in=nk );
    call mtapers_interface( .false., t1_in=1, t2_in=(t2-t1+1), scale_in=scale, inv_in=.false., tbp_in=tbp )
    call mtapers_gettaper ( mtaper_out=mtaper_sh(t1:t2,:)  )
    call mtapers_close    (  )

    if (shift) then;
     call fftshift1d_1_lop(.false.,.false.,mtaper_sh,mtaper)
    else
      mtaper = mtaper_sh
    end if

    deallocate(mtaper_sh)

end subroutine



subroutine mt_fftw1d_1_r2c_lop_close()
  if (allocated(   in )) then; deallocate (  in ); end if;
  if (allocated(  out )) then; deallocate(  out ); end if;
  if (allocated(mtaper)) then; deallocate(mtaper); end if;
end subroutine



subroutine mt_fftw1d_1_r2c_lop_1d(adj,add,m,d)
  logical,intent(in)     :: adj,add
  real                   ::         m(:)
  complex                ::           d(:,:)
  integer                :: ii
  
  call adjnull( adj, add, m, d )

  if ( adj ) then
    do ii=1,nk
      out      = d(:,ii)
      call sfftw_execute( plani )
      m        = m + in*mtaper(:,ii)/ sqrt( real(nt) )
    end do
  else
    do ii=1,nk
      in      = m*mtaper(:,ii)
      call sfftw_execute( planf )
      d(:,ii) = d(:,ii) + out/sqrt( real(nt) )
    end do
  end if

end subroutine


subroutine mt_fftw1d_1_r2c_lop_2d(adj,add,m,d)
  logical,intent(in)     :: adj,add
  real                   ::         m(:,:)
  complex                ::           d(:,:,:)
  integer                :: ii,jj,n2
  
  call adjnull( adj, add, m, d )
  
  n2 = size(m,2)

  if ( adj ) then
    do jj=1,n2
      do ii=1,nk
        out        = d(:,ii,jj)
        call sfftw_execute( plani )
        m(:,jj)    = m(:,jj) + in*mtaper(:,ii)/ sqrt( real(nt) )
      end do
    end do
  else
    do jj=1,n2
      do ii=1,nk
        in         = m(:,jj)*mtaper(:,ii)
        call sfftw_execute( planf )
        d(:,ii,jj) = d(:,ii,jj) + out/sqrt( real(nt) )
      end do
    end do
  end if

end subroutine


subroutine mt_fftw1d_1_r2c_lop_3d(adj,add,m,d)
  logical,intent(in)     :: adj,add
  real                   ::         m(:,:,:)
  complex                ::           d(:,:,:,:)
  integer                :: n2,n3,ii,jj,kk

  call adjnull( adj, add, m, d )
  
  n2 = size(m,2)
  n3 = size(m,3)
  
  if ( adj ) then
    do kk=1,n3
    do jj=1,n2
      do ii=1,nk
        out        = d(:,ii,jj,kk)
        call sfftw_execute( plani )
        m(:,jj,kk) = m(:,jj,kk) + in*mtaper(:,ii)/ sqrt( real(nt) )
      end do
    end do
    end do
  else
    do kk=1,n3
    do jj=1,n2
      do ii=1,nk
        in            = m(:,jj,kk)*mtaper(:,ii)
        call sfftw_execute( planf )
        d(:,ii,jj,kk) = d(:,ii,jj,kk) + out/sqrt( real(nt) )
      end do
    end do
    end do
  end if

end subroutine


subroutine mt_fft_r2c_units_t (adj,nt,dt,nw,dw,nf,df)
  logical,intent(in)      ::   adj
  integer                 ::       nt
  integer, optional       ::             nw,   nf
  real                    ::          dt
  real   , optional       ::                dw,   df
    
  if ( adj ) then
    nt = 0
    dt = 0.      
    if ( present ( nf ) ) then
      nt = (nf-1)*2
    end if
    if ( present ( nw ) ) then
      nt = (nw-1)*2
    end if
    if ( present ( df ) ) then
      dt = 1./(nt*df)
    end if
    if ( present ( dw ) ) then
      dt = 2.*pi/(nt*dw)
    end if
    if ( .not. (present(nf).or.present(nw).or.present(df).or.present(dw) ) ) then;
      write(0,*) 'fft_units_t - error  :  no input specified'; call exit();
    end if
  else
    if ( present ( nf ) ) then
      nf = nt/2+1
    end if
    if ( present ( nw ) ) then
      nw = nt/2+1
    end if
    if ( present ( df ) ) then
      df = 1./(nt*dt)
    end if
    if ( present ( dw ) ) then
      dw = 2.*pi/(nt*dt)
    end if
    if ( .not. (present(nf).or.present(nw).or.present(df).or.present(dw) ) ) then;
      write(0,*) 'fft_units_t - error  :  no input specified'; call exit();
    end if
  end if
    
end subroutine


end module
