module mtape_mod
  ! Single and multitaper, op's, lop's and fop's
  ! An interface to initialize the tapers in this module is located in tapers_mod

  use adjnull_mod
  use addnull_mod

  implicit none

  real, private,allocatable    :: mtaper(:,:)
  integer,private              :: ii,jj,kk,tt
  integer,private              :: n1,n2,n3,nt


  interface mtape_lop
    module procedure mtape1d_lop
    module procedure mtape2d_lop
    module procedure mtape3d_lop
  end interface


  contains


  subroutine mtape_init             (n1_in,n2_in,n3_in,nt_in)
    integer, intent(in),optional  :: n1_in,n2_in,n3_in,nt_in
    if (present( n1_in)) then; n1  = n1_in;  else; n1 = 1;  end if
    if (present( n2_in)) then; n2  = n2_in;  else; n2 = 1;  end if
    if (present( n3_in)) then; n3  = n3_in;  else; n3 = 1;  end if
    if (present( nt_in)) then; nt  = nt_in;  else; nt = 1;  end if
    
    if (allocated(mtaper)) then; deallocate(mtaper); end if;
    allocate( mtaper(n1,nt) ); call addnull(.false.,mtaper);

  end subroutine
  
  subroutine mtape_close()
    if (allocated(mtaper)) then; deallocate(mtaper); end if
  end subroutine
  
  
  subroutine  mtape_insert(add,mtaper_in,t1,t2)
    real   , intent(inout)       :: mtaper_in(:,:)
    logical, intent(in)          :: add
    integer                      :: t1,t2

    !write(0,*) t1,t2

    if (.not. size(mtaper_in) == n1*nt) then; write(0,*) "size(mtaper_in)!=size(mtaper) in next call to taper_mod"; call exit(); end if      

    call addnull(add,mtaper(t1:t2,:))
    do ii=t1,t2;
      mtaper(ii,:) = mtaper(ii,:) + mtaper_in(ii,:)
    end do

    
  end subroutine
  

  subroutine mtape_gettaper(mtaper_out)
    real   , intent(out)       :: mtaper_out(:,:)
    mtaper_out = mtaper
  end subroutine


  subroutine mtape1d_lop (adj,add,x,y)
    logical,intent(in)  :: adj,add
    real, intent(inout) ::         x(:),y(:,:)
    
    call adjnull          (adj,add,x,y)
    
    if (adj) then
      do tt=1,nt
      do ii=1,n1
        x(ii)    = x(ii)    + y(ii,tt)*mtaper(ii,tt)
      end do
      end do
    else
      do tt=1,nt
      do ii=1,n1
        y(ii,tt) = y(ii,tt) + x(ii)   *mtaper(ii,tt)
      end do
      end do
    end if
    
  end subroutine
  
  
  subroutine mtape2d_lop (adj,add,x,y)
    logical,intent(in)  :: adj,add
    real, intent(inout) ::         x(:,:),y(:,:,:)
    
    call adjnull          (adj,add,x,y)
    
    if (adj) then
      do jj=1,n2
      do tt=1,nt
      do ii=1,n1
        x(ii,jj)    = x(ii,jj)    + y(ii,tt,jj)*mtaper(ii,tt)
      end do
      end do
      end do
    else
      do jj=1,n2
      do tt=1,nt
      do ii=1,n1
        y(ii,tt,jj) = y(ii,tt,jj) + x(ii,jj)   *mtaper(ii,tt)
      end do
      end do
      end do
    end if
    
  end subroutine
  
  
  subroutine mtape3d_lop (adj,add,x,y)
    logical, intent(in) :: adj,add
    real, intent(inout) ::         x(:,:,:),y(:,:,:,:)
    
    call adjnull          (adj,add,x,y)
    
    if (adj) then
      do kk=1,n3
      do jj=1,n2
      do tt=1,nt
      do ii=1,n1
        x(ii,jj,kk)    = x(ii,jj,kk)    + y(ii,tt,jj,kk)*mtaper(ii,tt)
      end do
      end do
      end do
      end do
    else
      do kk=1,n3
      do jj=1,n2
      do tt=1,nt
      do ii=1,n1
        y(ii,tt,jj,kk) = y(ii,tt,jj,kk) + x(ii,jj,kk)   *mtaper(ii,tt)
      end do
      end do
      end do
      end do
    end if
    
  end subroutine




end module

