module mtapers_mod

  use slepian_mod
  use addnull_mod
  use mtape_mod
  implicit none

  real(8), allocatable, private  :: lambda(:),theta(:),mtaper8(:,:),mtaper8_com(:,:)
  integer, private               :: n1,nt


  interface mtapers_invert
    module procedure setinvm
    module procedure setinvm8
  end interface

  contains

  ! (Re)Initialize this module
  subroutine mtapers_init(n1_in,nt_in)
    integer, intent(in),optional  :: n1_in,nt_in

    if (present(n1_in )) then;  n1 = n1_in;  else;  n1 = 1024;  end if;
    if (present(nt_in )) then;  nt = nt_in;  else;  nt = 4   ;  end if;
    
    if (.not. allocated(mtaper8)) then; allocate( mtaper8(n1,nt) ); call addnull(.false.,mtaper8); end if;

  end subroutine
  
  
  ! Close this module
  subroutine mtapers_close()
    if (allocated(mtaper8)) then; deallocate( mtaper8 ); end if;
    if (allocated(lambda )) then; deallocate( lambda  ); end if; 
    if (allocated(theta  )) then; deallocate(  theta  ); end if;
  end subroutine
  
  
  subroutine mtapers_init_new(n_in)
    integer,intent(in),optional :: n_in
    integer                     :: n

    if (present(n_in)) then; n = n_in; else; n = n1; end if

    if (allocated(mtaper8_com)) then; deallocate(mtaper8_com); end if
    if (allocated(  lambda    )) then; deallocate(    lambda  ); end if
    if (allocated(   theta    )) then; deallocate(     theta  ); end if

    allocate(mtaper8_com(n,nt));  call addnull(.false.,mtaper8_com);
    allocate(      lambda(n)   ); call addnull(.false.,   lambda   );
    allocate(       theta(n)   ); call addnull(.false.,    theta   );

  end subroutine

  subroutine mtapers_insert(add,t1_in,t2_in,inv_in)
    logical, intent(in)            :: add
    integer, intent(in),optional   :: t1_in,t2_in
    logical, intent(in),optional   :: inv_in
    integer                        :: t1, t2
    logical                        :: inv


    if (present(t1_in )) then; t1  = t1_in;  else; t1 = 1;       end if
    if (present(t2_in )) then; t2  = t2_in;  else; t2 = n1;       end if
    if (present(inv_in)) then; inv = inv_in; else; inv=.false.; end if

    call mtapers_invert(inv,mtaper8_com)
    call addnull(add,mtaper8(t1:t2,:) )
    mtaper8(t1:t2,:) = mtaper8(t1:t2,:) + mtaper8_com
    call mtapers_invert(inv,mtaper8_com)

  end subroutine


  subroutine mtapers_interface       (add_in,t1_in,t2_in,rerr_in,scale_in,inv_in,tbp_in)
    integer, intent(in), optional  :: t1_in,t2_in
    real(8), intent(in), optional  ::             rerr_in
    real   , intent(in), optional  ::                     scale_in,tbp_in
    logical, intent(in), optional  :: add_in,inv_in
    integer                        :: n,t1,t2
    real                           :: scale,tbp,rerr
    logical                        :: add,inv
    
    if ( present(rerr_in ) ) then; rerr  = rerr_in; else; rerr = 9.5d-7;  end if;
    if ( present(  t1_in ) ) then;   t1  = t1_in;   else;   t1 = 1 ;      end if;
    if ( present(  t2_in ) ) then;   t2  = t2_in;   else;   t2 = n1;      end if;
    if ( present( add_in ) ) then;  add  = add_in;  else;  add = .false.; end if;
    if ( present( tbp_in ) ) then;  tbp  = tbp_in;  else;  tbp = 10.;     end if;
    if ( present( scale_in ) ) then;  scale  = scale_in;  else;  scale = 10.;     end if;
    
    n = t2 - t1 + 1;
    
    call mtapers_init_new(n_in=n)
    call         slepian (n_in=n,tbp_in=tbp,scale_in=scale)
    call mtapers_insert  (add,t1_in=t1,t2_in=t2,inv_in=inv)
    
  end subroutine
  
  
  ! To just getting a 1d taper, for external use
  subroutine mtapers_gettaper(mtaper_out,inv_in,mtaper8_out,lambda_out,theta_out)
    real   ,intent(out),optional   :: mtaper_out(:,:)
    real(8),intent(out),optional   :: mtaper8_out(:,:)
    real(8),intent(out),optional   :: lambda_out(:),theta_out(:)
    logical,intent(in) ,optional   :: inv_in
    logical                        :: inv

    if (present(inv_in)) then; inv  = inv_in;  else; inv=.false.; end if
    call mtapers_invert(inv,mtaper8)
    if (present( mtaper_out)) then;  mtaper_out = real(mtaper8); end if
    if (present(mtaper8_out)) then; mtaper8_out = mtaper8; end if
    if (present( lambda_out)) then; lambda_out  = lambda; end if
    if (present(  theta_out)) then;  theta_out  = theta; end if
    call mtapers_invert(inv,mtaper8)
    
  end subroutine


  subroutine mtapers_insert_mtape(add_in,inv_in,t1_in,t2_in)
    logical,optional,intent(in)    :: inv_in,add_in
    integer,optional,intent(in)    :: t1_in,t2_in
    integer                        :: t1,t2,dir
    logical                        :: inv,add
    real,allocatable               :: mtaper_com(:,:)
    
    integer                        :: ii,jj,kk
    
    if (present(t1_in )) then; t1  = t1_in;  else; t1 = 1;       end if
    if (present(t2_in )) then; t2  = t2_in;  else; t2 = n1;    end if
    if (present(inv_in)) then; inv = inv_in; else; inv=.false.; end if
    if (present(add_in)) then; add = add_in; else; add=.false.; end if
    
    allocate( mtaper_com(size(mtaper8_com,1),size(mtaper8_com,2)) )
    mtaper_com = real(mtaper8_com)
    call mtapers_invert (inv,mtaper8_com)
    call mtape_insert   (add, mtaper_com,t1,t2)
    call mtapers_invert (inv,mtaper8_com)
    deallocate(mtaper_com)    

  end subroutine


  ! Get an indice given an absolute time, d and o
  subroutine mtapers_indice(t,t_ind,d_in,o_in)
    real,optional,intent(in) :: d_in,o_in
    real                     :: d   ,o
    integer                  :: t_ind
    real                     :: t
    if (present(d_in)) then; d = d_in; else; d = 1.; end if
    if (present(o_in)) then; o = o_in; else; o = 1.; end if
    
    t_ind = int((t-o)/d)+1
    
  end subroutine



 subroutine setinvm     (inv,x)
    real,intent(inout) :: x(:,:)
    logical,intent(in) :: inv    
    
    if (inv) then
      x = 1.-x
    end if

  end subroutine

 subroutine setinvm8     (inv,x)
    real(8),intent(inout) :: x(:,:)
    logical,intent(in) :: inv    
    
    if (inv) then
      x = 1.-x
    end if

  end subroutine

  subroutine slepian        (n_in,tbp_in,scale_in)
    integer, intent(in), optional :: n_in
    real   , intent(in), optional ::      tbp_in,scale_in
    integer                       :: n
    real                          :: tbp,scale
    logical                       :: inv,add
    
    if (present(    n_in)) then;     n = n_in;     else;    n  = n1;   end if;
    if (present(  tbp_in)) then;   tbp = tbp_in;   else;   tbp = 4.;  end if;
    if (present(scale_in)) then; scale = scale_in; else; scale = 1. ; end if;

    call addnull(.false.,mtaper8_com)
    ! Calculating tapers
    if (n<20000) then
      ! write(0,*) "This one",n,tbp,nt
       call dpss(             n,dble(tbp),nt,mtaper8_com,lambda,theta)
    else
       !  write(0,*) 'Computing DPSS with interpolation'
       call dpss_spline(10000,n,dble(tbp),nt,mtaper8_com,lambda,theta)
    end if
    !write(0,*) mtaper8_com

    mtaper8_com = mtaper8_com*scale

   ! write(0,*)  
   ! write(0,*) "Compare the Taoers"
   ! write(0,*) "nt,tbp,n1:" ,nt,tbp,n
   ! write(0,*) "vn4:", real(mtaper8_com)

  end subroutine

end module
