module padd_mod

  use adjnull_mod
  
  implicit none

  integer, private :: b1,b2,b3,e1,e2,e3
  integer, private :: p1,p2,p3
  logical, private :: center=.false.

  interface padd_lop
    module procedure padd1d
    module procedure padd2d
    module procedure padd3d
  end interface

  contains

  subroutine padd_mod_init(center_in,p1_in,p2_in,p3_in)
    logical,intent(in),optional :: center_in
    integer,intent(in),optional :: p1_in,p2_in,p3_in
    if (present(center_in)) then; center = center_in; else; center=.false.; end if
    if (present(    p1_in)) then;     p1 =     p1_in; else;     p1=1      ; end if
    if (present(    p2_in)) then;     p2 =     p2_in; else;     p2=1      ; end if
    if (present(    p3_in)) then;     p3 =     p3_in; else;     p3=1      ; end if
  end subroutine
  
  subroutine padd_mod_close()

  end subroutine
  
  
  subroutine padd1d(adj,add,d,m)
    logical,intent(in) :: adj,add
    real               ::   d(:),m(:)
    integer            :: n1
    
    call adjnull(adj,add,d,m)
    
    n1 = size(d,1)
    if (center) then
      b1 = 1 + (p1-n1)/2;   e1 = n1 + (p1-n1)/2
    else
      b1 = 1;               e1 = n1
    end if;
    
    if (adj) then;
      d( 1:n1) = d( 1:n1) + m(b1:e1)
    else
      m(b1:e1) = m(b1:e1) + d( 1:n1)
    end if  
    
  end subroutine
  
  
  subroutine padd2d(adj,add,d,m)
    logical,intent(in) :: adj,add
    real               ::   d(:,:),m(:,:)
    integer            :: n1,n2
    
    call adjnull(adj,add,d,m)
    
    n1 = size(d,1)
    n2 = size(d,2)
    if (center) then
      b1 = 1 + (p1-n1)/2;   e1 = n1 + (p1-n1)/2
      b2 = 1 + (p2-n2)/2;   e2 = n2 + (p2-n2)/2
    else
      b1 = 1;               e1 = n1
      b2 = 1;               e2 = n2
    end if;
    
    if (adj) then;
      d( 1:n1, 1:n2) = d( 1:n1, 1:n2) + m(b1:e1,b2:e2)
    else
      m(b1:e1,b2:e2) = m(b1:e1,b2:e2) + d( 1:n1, 1:n2)
    end if  
    
  end subroutine
  
  
  subroutine padd3d(adj,add,d,m)
    logical,intent(in) :: adj,add
    real               ::   d(:,:,:),m(:,:,:)
    integer            :: n1,n2,n3
    
    call adjnull(adj,add,d,m)
    
    n1 = size(d,1)
    n2 = size(d,2)
    n3 = size(d,3)
    if (center) then
      b1 = 1 + (p1-n1)/2;   e1 = n1 + (p1-n1)/2
      b2 = 1 + (p2-n2)/2;   e2 = n2 + (p2-n2)/2
      b3 = 1 + (p3-n3)/2;   e3 = n3 + (p3-n3)/2
    else
      b1 = 1;               e1 = n1
      b2 = 1;               e2 = n2
      b3 = 1;               e3 = n3
    end if;
    
    if (adj) then;
      d( 1:n1, 1:n2, 1:n3) = d( 1:n1, 1:n2, 1:n3) + m(b1:e1,b2:e2,b3:e3)
    else
      m(b1:e1,b2:e2,b3:e3) = m(b1:e1,b2:e2,b3:e3) + d( 1:n1, 1:n2, 1:n3)
    end if  
    
  end subroutine
  
end module
