module phase_unwrap_mod

  use solver_mod
  use cgstep_mod
  use igrad2_mod
  implicit none
  
  private

  public  :: unwrapper_init, unwrapper_close, unwrapper
  public  :: wop_init, wop_close, wop_lop

  real,allocatable, private  ::  wop_unwrap(:,:)
  integer         , private  ::  nx,ny

  contains

  subroutine unwrapper_init(n1,n2,wop)
    integer        :: n1,n2
    real,optional  :: wop(:)
    call unwrapper_close()
    nx=n1; ny=n2
    call igrad2_init(nx,ny)
    if (present(wop)) call wop_init(wop)
  end subroutine
  subroutine unwrapper_close()
  end subroutine
  
  subroutine unwrapper(zz,hh,niter)
    integer  :: niter
    complex  :: zz(nx*ny)
    real     :: hh(nx*ny)
    real,allocatable :: rt(:)
    
    allocate(rt(nx*ny*2))
    call grad2init(zz,rt)
    call solver_init(nx*ny,nx*ny)
    call solver_smp(Fop=igrad2_lop, stepper=cgstep, m = hh, d=rt, niter=niter)
    call cgstep_close()
    deallocate(rt)
    
  end subroutine

  subroutine grad2init(z,rt)
   integer              ::    ix,iy
   complex              ::    z (nx,ny  )
   real                 ::    rt(nx,ny,2)
   complex              ::    a,b,c
   rt = 0.
   do ix=1,nx-1
   do iy=2,ny-1
     a = z(ix  ,iy  )
     c = z(ix+1,iy  );  rt(ix,iy,1) = aimag( clog( c*conjg(a) ) )
     b = z(ix  ,iy+1);  rt(ix,iy,2) = aimag( clog( b*conjg(a) ) )
   end do
   end do
  end subroutine
  
  
  
  
  
  
  
  
  
  ! Masking operator (interpolated phase outside array can't be trusted)
  subroutine wop_init(   w_in)
    real             ::  w_in(nx,ny)
    call wop_close()
    allocate(wop_unwrap(nx,ny))
    wop_unwrap=w_in
  end subroutine
  subroutine wop_close()
    deallocate(wop_unwrap)
  end subroutine

  function wop_lop(adj, add, x, y)  result(stat)
    integer            :: stat 
    logical,intent(in) :: adj, add 
    real,dimension(:)  :: x,y
    if( .not. add) then
      if(.not.adj)  y = 0.
      if(     adj)  x = 0.
    end if
    if ( adj) then
      call wop_adj(x,y)
    else
      call wop_for(x,y)
    end if
    stat=0
  end function

  subroutine wop_for( x       , y        )
    real           :: x(nx,ny), y(nx,ny)
    integer        ::ix       ,iy
    do ix=1,nx
      do iy=1,ny
        y(ix,iy) = y(ix,iy) + wop_unwrap(iy,ix) * x(ix,iy)
      end do
    end do
  end subroutine

  subroutine wop_adj( x       , y        )
    real           :: x(nx,ny), y(nx,ny)
    integer        ::ix       ,iy
    do ix=1,nx
      do iy=1,ny
        x(ix,iy) = x(ix,iy) + wop_unwrap(iy,ix) * y(ix,iy)
      end do
    end do
  end subroutine


end module
