module regurize2d_splines_mod
  
  use regurize_splines_LU_mod
  implicit none
  
  private
  
  public  :: regurize2d_isplinest_init
  public  :: regurize2d_isplinest
  
  public  :: regurize2d_isplines_init
  public  :: regurize2d_isplines
  
  public  :: regurize2d_rsplinest_init
  public  :: regurize2d_rsplinest
  
  public  :: regurize2d_rsplines_init
  public  :: regurize2d_rsplines
  
  public  :: regurize2d_splines_close
  
  
  real, allocatable       :: regspl2d_headerin(:,:),regspl2d_headeout(:,:),regspl2d_splngrid(:,:)
  integer, private        :: n1in,n1out,n1spl
  real, private           :: p
  
  ! For LU decomposition
  real,   allocatable     :: regspl2d_F (:,:)
  real*8, allocatable     :: regspl2d_FI(:,:),regspl2d_MM(:)
  real*8, private         :: DETER
  integer, private        :: CODE,DD
  integer, allocatable    :: regspl2d_INDX(:)
  
  
  interface regurize2d_rsplinest
    module procedure rsplinest2d_2d
    module procedure rsplinest2d_3d
  end interface
  
  interface regurize2d_rsplines
    module procedure rsplines2d_2d
    module procedure rsplines2d_3d
  end interface
  
  interface regurize2d_isplinest
    module procedure isplinest2d_2d
    module procedure isplinest2d_3d
  end interface
  
  interface regurize2d_isplines
    module procedure isplines2d_2d
    module procedure isplines2d_3d
  end interface
  
  
  
  contains
  
  subroutine regurize2d_rsplinest_init(headerin_in,headeout_in,splngrid_in,p_in)
    real     ::   headerin_in(:,:), headeout_in(:,:), splngrid_in(:,:)
    real     ::   p_in, r
    integer  ::   ii, jj
  real             :: besy0
    call regurize2d_splines_close()
    n1in  = size(headerin_in,1)
    n1out = size(headeout_in,1)
    n1spl = size(splngrid_in,1)
    p     = p_in

    ! Allocate headerIO vectors
    allocate( regspl2d_headerin(n1in ,2)); regspl2d_headerin = headerin_in
    allocate( regspl2d_headeout(n1out,2)); regspl2d_headeout = headeout_in
    allocate( regspl2d_splngrid(n1spl,2)); regspl2d_splngrid = splngrid_in
    
    ! Allocate array for forward and inverse operators
    allocate( regspl2d_F(n1in,n1spl) , regspl2d_FI(n1spl,n1spl) );
    ! Allocate dummy-array for solver data->model
    allocate( regspl2d_MM(n1spl) );
    ! Allocate index array for permutations
    allocate( regspl2d_INDX(n1spl) )
    
    ! Explicitly construct forward operator as matrix
    do jj=1,n1spl
      do ii=1,n1in
        ! See:   P. Wessel, Computers and Geosciences 35 (2009), 1247-1254.
        r        = sqrt( (regspl2d_headerin(ii,1)-regspl2d_splngrid(jj,1))**2. + (regspl2d_headerin(ii,2)-regspl2d_splngrid(jj,2))**2. )
        if (r.eq.0) r = epsilon(r)
        regspl2d_F(ii,jj) = besy0(p*r) + log(p*r)
      end do
    end do
    
   ! write(0,*) 'Initialized regular tensed splines'

  end subroutine
  
  subroutine regurize2d_rsplines_init(headerin_in,headeout_in,splngrid_in)
    real     ::   headerin_in(:,:), headeout_in(:,:), splngrid_in(:,:)
    real     ::   r
    integer  ::   ii, jj
    call regurize2d_splines_close()
    n1in  = size(headerin_in,1)
    n1out = size(headeout_in,1)
    n1spl = size(splngrid_in,1)
    write(0,*) 'n1in,n1out,n1spl',n1in,n1out,n1spl

    ! Allocate headerIO vectors
    allocate( regspl2d_headerin(n1in ,2)); regspl2d_headerin = headerin_in
    allocate( regspl2d_headeout(n1out,2)); regspl2d_headeout = headeout_in
    allocate( regspl2d_splngrid(n1spl,2)); regspl2d_splngrid = splngrid_in

    ! Allocate array for forward and inverse operators
    allocate( regspl2d_F(n1in,n1spl) , regspl2d_FI(n1spl,n1spl) );
    ! Allocate dummy-array for solver data->model
    allocate( regspl2d_MM(n1spl) );
    ! Allocate index array for permutations
    allocate( regspl2d_INDX(n1spl) );
    
    ! Explicitly construct forward operator as matrix
    do jj=1,n1spl
      do ii=1,n1in
        ! See:   P. Wessel, Computers and Geosciences 35 (2009), 1247-1254.
        r        = sqrt( (regspl2d_headerin(ii,1)-regspl2d_splngrid(jj,1))**2. + (regspl2d_headerin(ii,2)-regspl2d_splngrid(jj,2))**2. )
        if (r.eq.0) r = epsilon(r)
        regspl2d_F(ii,jj) = r**2.*(log(r)-1.)
      end do
    end do

  end subroutine
  
  subroutine regurize2d_isplinest_init(headerin_in,headeout_in,p_in)
    real     ::   headerin_in(:,:), headeout_in(:,:)
    real     ::   p_in, r
    integer  ::   ii, jj
    real     ::   besy0
    call regurize2d_splines_close()
    n1in  = size(headerin_in,1)
    n1out = size(headeout_in,1)
    p     = p_in
    
    ! Allocate headerIO vectors
    allocate( regspl2d_headerin(n1in ,2)); regspl2d_headerin = headerin_in
    allocate( regspl2d_headeout(n1out,2)); regspl2d_headeout = headeout_in
    
    ! Allocate array for forward and inverse operators
    allocate( regspl2d_F(n1in,n1in) , regspl2d_FI(n1in,n1in) );
    ! Allocate dummy-array for solver data->model
    allocate( regspl2d_MM(n1in) );
    ! Allocate index array for permutations
    allocate( regspl2d_INDX(n1in) );
    
    ! Explicitly construct forward operator as matrix
    do jj=1,n1in
      do ii=1,n1in
        ! See:   P. Wessel, Computers and Geosciences 35 (2009), 1247-1254.
        r        = sqrt( (regspl2d_headerin(ii,1)-regspl2d_headerin(jj,1))**2. + (regspl2d_headerin(ii,2)-regspl2d_headerin(jj,2))**2. )
        if (r.eq.0) r = epsilon(r)
        regspl2d_F(ii,jj) = besy0(p*r) + log(p*r)
      end do
    end do
    
  !  write(0,*) 'Initialized irregular tensed splines'
    
  end subroutine
  
  subroutine regurize2d_isplines_init(headerin_in,headeout_in)
    real     ::   headerin_in(:,:), headeout_in(:,:)
    real     ::   r
    integer  ::   ii, jj
    call regurize2d_splines_close()
    n1in  = size(headerin_in,1)
    n1out = size(headeout_in,1)
    
    ! Allocate headerIO vectors
    allocate( regspl2d_headerin(n1in ,2)); regspl2d_headerin = headerin_in
    allocate( regspl2d_headeout(n1out,2)); regspl2d_headeout = headeout_in
    
    ! Allocate array for forward and inverse operators
    allocate( regspl2d_F(n1in,n1in) , regspl2d_FI(n1in,n1in) );
    ! Allocate dummy-array for solver data->model
    allocate( regspl2d_MM(n1in) );
    ! Allocate index array for permutations
    allocate( regspl2d_INDX(n1in) );
    
    ! Explicitly construct forward operator as matrix
    do jj=1,n1in
      do ii=1,n1in
        ! See:   P. Wessel, Computers and Geosciences 35 (2009), 1247-1254.
        r        = sqrt( (regspl2d_headerin(ii,1)-regspl2d_headerin(jj,1))**2. + (regspl2d_headerin(ii,2)-regspl2d_headerin(jj,2))**2. )
        if (r.eq.0) r = epsilon(r)
        regspl2d_F(ii,jj) = r**2.*(log(r)-1.)
      end do
    end do

  !  write(0,*) 'Initialized irregular splines'

  end subroutine
  
  

  subroutine rsplinest2d_2d(datain,datout)
    real     :: datain(:),datout(:)
    integer  :: ii, jj, ij
    real     :: r
  real             :: besy0
    !write(0,*) 'Start regular tensed spline regularization'
    regspl2d_FI = 0.
    regspl2d_MM = 0.
    regspl2d_INDX = 0.

    !Call Solver!
      ! Prepare dubbel-precsisions
      do jj=1,n1spl
        do ii=1,n1spl
          do ij=1,n1in
            regspl2d_FI(ii,jj) = regspl2d_FI(ii,jj) + dble( regspl2d_F(ij,ii)*regspl2d_F(ij,jj) )
          end do
        end do
      end do
      do ii=1,n1spl
          do ij=1,n1in
            regspl2d_MM(ii   ) = regspl2d_MM(ii   ) + dble( regspl2d_F(ij,ii)*datain(ij) )
          end do
      end do

      ! Call LU decomposition and back-substitution
      call LUDCMP(regspl2d_FI,n1spl,regspl2d_INDX,DD,CODE)
      call LUBKSB(regspl2d_FI,n1spl,regspl2d_INDX,regspl2d_MM     )
    !!

    ! Compute values on regular grid
    do jj=1,n1spl
      do ii=1,n1out
          r = sqrt( (regspl2d_headeout(ii,1)-regspl2d_splngrid(jj,1))**2. + (regspl2d_headeout(ii,2)-regspl2d_splngrid(jj,2))**2. )
          if (r.eq.0) r = epsilon(r)
          datout(ii) = datout(ii) + real(regspl2d_MM(jj))*( besy0(p*r) + log(p*r) )
      end do
    end do

    !write(0,*) datout

  end subroutine



  subroutine rsplines2d_2d(datain,datout)
    real     :: datain(:),datout(:)
    integer  :: ii, jj, ij
    real     :: r

   ! write(0,*) 'Start regular spline regularization'

    regspl2d_FI = 0.
    regspl2d_MM = 0.
    regspl2d_INDX = 0.

    !Call Solver!
      ! Prepare dubbel-precsisions
      do jj=1,n1spl
        do ii=1,n1spl
          do ij=1,n1in
            regspl2d_FI(ii,jj) = regspl2d_FI(ii,jj) + dble( regspl2d_F(ij,ii)*regspl2d_F(ij,jj) )
          end do
        end do
      end do

      do ii=1,n1spl
          do ij=1,n1in
            regspl2d_MM(ii   ) = regspl2d_MM(ii   ) + dble( regspl2d_F(ij,ii)*datain(ij) )
          end do
      end do

      ! Call LU decomposition and back-substitution
      call LUDCMP(regspl2d_FI,n1spl,regspl2d_INDX,DD,CODE)
      call LUBKSB(regspl2d_FI,n1spl,regspl2d_INDX,regspl2d_MM     )
    !!

    !write(0,*) regspl2d_FI

    ! Compute values on regular grid
    do jj=1,n1spl
      do ii=1,n1out
          r = sqrt( (regspl2d_headeout(ii,1)-regspl2d_splngrid(jj,1))**2. + (regspl2d_headeout(ii,2)-regspl2d_splngrid(jj,2))**2. )
          if (r.eq.0) r = epsilon(r)
          datout(ii) = datout(ii) + real(regspl2d_MM(jj))*r**2.*(log(r)-1.)
      end do
    end do

    !write(0,*) datout

  end subroutine
  
  
  
  
  
  subroutine isplinest2d_2d(datain,datout)
    real     :: datain(:),datout(:)
    integer  :: ii, jj
    real     :: r
  real             :: besy0
    
  !  write(0,*) 'Start irregular tensed spline regularization'

    regspl2d_FI = 0.
    regspl2d_MM = 0.
    regspl2d_INDX = 0.
    
    !Call Solver!
      ! Prepare dubbel-precsisions
      regspl2d_FI = dble(regspl2d_F)
      regspl2d_MM = dble(datain)

      ! Call LU decomposition and back-substitution
      call LUDCMP(regspl2d_FI,n1in,regspl2d_INDX,DD,CODE)
      call LUBKSB(regspl2d_FI,n1in,regspl2d_INDX,regspl2d_MM     )
    !!

    ! Compute values on regular grid
    do jj=1,n1in
      do ii=1,n1out
          r = sqrt( (regspl2d_headeout(ii,1)-regspl2d_headerin(jj,1))**2. + (regspl2d_headeout(ii,2)-regspl2d_headerin(jj,2))**2. )
          if (r.eq.0) r = epsilon(r)
          datout(ii) = datout(ii) + real(regspl2d_MM(jj))*( besy0(p*r) + log(p*r) )
      end do
    end do

  end subroutine
  
  
  
  subroutine isplines2d_2d(datain,datout)
    real     :: datain(:),datout(:)
    integer  :: ii, jj
    real     :: r
    
   ! write(0,*) 'Start irregular spline regularization'

    regspl2d_FI = 0.
    regspl2d_MM = 0.
    regspl2d_INDX = 0.
    
    !Call Solver!
      ! Prepare dubbel-precsisions
      regspl2d_FI = dble(regspl2d_F)
      regspl2d_MM = dble(datain)

      ! Call LU decomposition and back-substitution
      call LUDCMP(regspl2d_FI,n1in,regspl2d_INDX,DD,CODE)
      call LUBKSB(regspl2d_FI,n1in,regspl2d_INDX,regspl2d_MM     )
    !!
    
    ! Compute values on regular grid
    do jj=1,n1in
      do ii=1,n1out
          r = sqrt( (regspl2d_headeout(ii,1)-regspl2d_headerin(jj,1))**2. + (regspl2d_headeout(ii,2)-regspl2d_headerin(jj,2))**2. )
          if (r.eq.0) r = epsilon(r)
          datout(ii) = datout(ii) + real(regspl2d_MM(jj))*r**2.*(log(r)-1.)
      end do
    end do

  end subroutine
  


  
  subroutine rsplinest2d_3d(datain,datout)
    real     :: datain(:,:),datout(:,:)
    integer  :: jj,n2
    n2 = size(datout,2)
    do jj=1,n2
      call rsplinest2d_2d(datain(:,jj),datout(:,jj))
    end do
  end subroutine

  
  subroutine rsplines2d_3d(datain,datout)
    real     :: datain(:,:),datout(:,:)
    integer  :: jj,n2
    n2 = size(datout,2)
    do jj=1,n2
      call rsplines2d_2d(datain(:,jj),datout(:,jj))
    end do
  end subroutine

  
  subroutine isplinest2d_3d(datain,datout)
    real     :: datain(:,:),datout(:,:)
    integer  :: jj,n2
    n2 = size(datout,2)
    do jj=1,n2
      call isplinest2d_2d(datain(:,jj),datout(:,jj))
    end do
  end subroutine

  
  subroutine isplines2d_3d(datain,datout)
    real     :: datain(:,:),datout(:,:)
    integer  :: jj,n2
    n2 = size(datout,2)
    do jj=1,n2
      call isplines2d_2d(datain(:,jj),datout(:,jj))
    end do
  end subroutine








  subroutine regurize2d_splines_close()
  !  write(0,*) 'Closing spline regularization'
    if (allocated(regspl2d_headerin)) deallocate( regspl2d_headerin )
    if (allocated(regspl2d_headeout)) deallocate( regspl2d_headeout )
    if (allocated(regspl2d_F)) deallocate( regspl2d_F )
    if (allocated(regspl2d_FI)) deallocate(regspl2d_FI )
    if (allocated(regspl2d_MM)) deallocate( regspl2d_MM )
    if (allocated(regspl2d_INDX)) deallocate( regspl2d_INDX )
    if (allocated(regspl2d_splngrid)) deallocate(regspl2d_splngrid)
  end subroutine


end module
