module regurize2d_splines_mod_omp
  
  use regurize_splines_LU_mod
  implicit none
  
  private
  
  public  :: regurize2d_splines_omp
  public  :: regurize2d_splines_omp_init
  public  :: regurize2d_splines_omp_close
  
  ! For LU decomposition
  real*8, allocatable     :: regspl2d_F_omp(:,:,:),regspl2d_MM_omp(:,:),regspl2d_datout_omp(:,:)
  integer, allocatable    :: regspl2d_INDX_omp(:,:)
  include "openmp.inc"
  
  contains
  
  subroutine regurize2d_splines_omp_init(n1inmax,n1outmax)
    integer      :: n1inmax,n1outmax
    include "openmp_init.inc"
    call regurize2d_splines_omp_close()
    ! Allocate array for dataout
    allocate( regspl2d_datout_omp(n1outmax,nth) ); regspl2d_datout_omp=0.
    ! Allocate array for forward and inverse operators
    allocate( regspl2d_F_omp (n1inmax,n1inmax,nth) ); regspl2d_F_omp =0.
    ! Allocate dummy-array for solver data->model
    allocate( regspl2d_MM_omp(n1inmax,nth) ); regspl2d_MM_omp=0.
    ! Allocate index array for permutations
    allocate( regspl2d_INDX_omp(n1inmax,nth) ); regspl2d_INDX_omp=0.
  end subroutine
  
  
  subroutine regurize2d_splines_omp(headerin,nhdrin,headeout,nhdrout,datain,    datout     )
    real,intent(in)         ::      headerin(:,:,:),headeout(:,:,:), datain(:,:)
    real,intent(in)         ::               nhdrin(:),      nhdrout(:)
    real,intent(out)        ::                                                   datout(:,:)
    integer                 ::  is,ns,nin,nout
    ns = size(headerin,3)
    
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(SHARED) PRIVATE(is,nin,nout,ith)
    do is=1,ns
      ith  = omp_get_thread_num()+1
      nin  = nhdrin (is)
      nout = nhdrout(is)
      write(0,*) 'Regularize travel time surface', is, 'of', ns, 'on thread', ith
      call regurize2d_splines(headerin(1:nin,1:2,is),nin,headeout(1:nout,1:2,is),nout,datain(1:nin,is),datout(1:nout,is),ith)
    end do
    !$OMP END PARALLEL DO
    
  end subroutine
  

  subroutine regurize2d_splines (headerin,n1in,    headeout,n1out,    datain,    datout,   ithrd)
    integer                 ::            n1in,             n1out,                         ithrd
    real,intent(in)         ::   headerin(n1in,2), headeout(n1out,2), datain(n1in)    
    real,intent(out)        ::                                                   datout(n1out)
    integer                 ::   ii, jj
    real*8                  ::   DETER
    integer                 ::   CODE,DD
    real*8                  ::   r
    
    regspl2d_F_omp     (:,:,ithrd) = 0.
    regspl2d_MM_omp    (:,  ithrd) = 0.
    regspl2d_INDX_omp  (:,  ithrd) = 0.
    regspl2d_datout_omp(:,  ithrd) = 0.
    
    ! Explicitly construct forward operator as matrix
    do jj=1,n1in
      do ii=1,n1in
   !      write(0,*) headerin(ii,1),headerin(jj,1)
        ! See:   P. Wessel, Computers and Geosciences 35 (2009), 1247-1254.
        r        = sqrt( (dble(headerin(ii,1))-dble(headerin(jj,1)))**2. + (dble(headerin(ii,2))-dble(headerin(jj,2)))**2. )
    !    write(0,*) r
        regspl2d_F_omp(ii,jj,ithrd) = r**2.*(log(r)-1.)
        if (r.eq.0.) regspl2d_F_omp(ii,jj,ithrd) = 0.
      end do
    end do   
    
    !Call Solver!
      ! Prepare dubbel-precsisions
      regspl2d_MM_omp(1:n1in,ithrd) = dble( datain(1:n1in) )
      
      ! Call LU decomposition and back-substitution
!      write(0,*) 'regspl2d_F_omp(1:n1in,1:n1in,ithrd)',n1in,ithrd
!      write(0,*)  regspl2d_F_omp(1:n1in,1:n1in,ithrd)

      call LUDCMP(regspl2d_F_omp(1:n1in,1:n1in,ithrd),n1in,regspl2d_INDX_omp(1:n1in,ithrd),DD,CODE)
!      write(0,*) 'regspl2d_INDX_omp(1:n1in,ithrd)',n1in,ithrd
!      write(0,*)  regspl2d_INDX_omp(1:n1in,ithrd)
      call LUBKSB(regspl2d_F_omp(1:n1in,1:n1in,ithrd),n1in,regspl2d_INDX_omp(1:n1in,ithrd),regspl2d_MM_omp(1:n1in,ithrd) )
    !!
    
    ! Compute values on regular grid
    do jj=1,n1in
      do ii=1,n1out
          r = sqrt( (dble(headeout(ii,1))-dble(headerin(jj,1)))**2. + (dble(headeout(ii,2))-dble(headerin(jj,2)))**2. )
          if (r.ne.0) regspl2d_datout_omp(ii,ithrd) = regspl2d_datout_omp(ii,ithrd) + regspl2d_MM_omp(jj,ithrd)*r**2.*(log(r)-1.)
      end do
    end do
    datout(1:n1out) = real(regspl2d_datout_omp(1:n1out,ithrd))
    
  end subroutine
  
  
  
  
  subroutine regurize2d_splines_omp_close()
    if (allocated(regspl2d_F_omp))      deallocate( regspl2d_F_omp )
    if (allocated(regspl2d_MM_omp))     deallocate( regspl2d_MM_omp )
    if (allocated(regspl2d_INDX_omp))   deallocate( regspl2d_INDX_omp )
    if (allocated(regspl2d_datout_omp)) deallocate( regspl2d_datout_omp )
  end subroutine
  
end module
