module solver_mod
  implicit none
  integer, private                   :: nd,nm,nr
  logical, parameter, private        :: T = .true., F = .false.
  real, allocatable, private, target :: rr_solver(:), gg_solver(:),g_solver(:)   ! residual, conj grad
  
  contains
  
  subroutine solver_init(nm_in,nd_in,nr_in)
    integer          ::  nm_in,nd_in
    integer,optional ::              nr_in
    call solver_close()
    nm=nm_in; nd = nd_in
    nr=0; if (present(nr_in)) nr = nr_in
    allocate(rr_solver(nd+nr), gg_solver(nd+nr), g_solver(nm))
  end subroutine
  subroutine solver_close()
    deallocate( rr_solver , gg_solver , g_solver )
    rr_solver=0.; gg_solver=0.; g_solver=0.
  end subroutine
  
  
  
  subroutine solver_smp( Fop, stepper, m, d, niter, r )
    optional                                    ::  r 
    interface
      integer function Fop( adj, add, m, d )
        logical, intent (in) :: adj, add
        real                 :: m(:), d(:)
      end function 
      integer function stepper( forget, m, g, rr, gg)
        logical              :: forget
        real                 :: m(:), g(:), rr(:), gg(:)
      end function
    end interface
    real,    intent (in)     :: d(nd)            ! data, initial
    real,    intent (out)    :: r(nd), m (nm)    ! solution, residual
    integer, intent (in)     :: niter            ! iterations

    integer                  :: i, stat
    logical                  :: forget
    
    ! Initialization Part
    rr_solver = - d                                     ! rr = -d
    stat = Fop( F, T, m, rr_solver) ! rr = rr + F m0
    
    ! Iterations
    forget = F
    do i = 1, niter   
      stat = Fop( T, F, g_solver, rr_solver)                          ! g  <- F' rr       
      stat = Fop( F, F, g_solver, gg_solver)                          ! gg <- F  g
      stat = stepper( forget, m, g_solver, rr_solver, gg_solver)      ! step in x and rr
!      if (mod(i,10).eq.0) then
!        write(0,*) 'iteration ',i,' rdpow=',sum(rr*rr)
!      end if
    end do 
    
    ! Conclusions
    if (present(r))  r = rr_solver

  end subroutine 




  
  
  
  
  subroutine solver_reg( Fop, stepper, Reg, m, d, niter, eps, mnull, r )
    optional                                               :: mnull, r 
    interface
      integer function Fop( adj, add, m, d )
        logical, intent (in) :: adj, add
        real                 :: m(:), d(:)
      end function 
      integer function Reg( adj, add, m, d )
        logical, intent( in) :: adj, add
        real                 :: m(:), d(:)
      end function
      integer function stepper( forget, m, g, rr, gg)
        logical              :: forget
        real                 :: m(:), g(:), rr(:), gg(:)
      end function
    end interface
    real,    intent (in)     :: d(nd), mnull(nm)            ! data, null space model
    real,    intent (out)    :: r(nd), m (nm)               ! solution, residual
    integer, intent (in)     :: niter                       ! iterations
    real,    intent (in)     :: eps                         ! scaling (of objective functions)

    integer                  :: i, stat1,stat2,stat
    logical                  :: forget

    real, dimension(:), pointer :: rd,rm,gd,gm       ! residual and gradient Pointers

    ! Set up residual and gradient Pointers
    rm => rr_solver( 1   : nr)
    rd => rr_solver( 1+nr:   )                       ! model and data resids
    gm => gg_solver( 1   : nr)
    gd => gg_solver( 1+nr:   )                       ! model and data grads

    ! Initialization Part
    rd    = - d                              ! r_d = - d
    stat1 = Fop( F, T, m, rd)                ! r_d = r_d + L m             
    stat2 = Reg( F, F, m, rm);rm = rm*eps    ! r_m = eps A m
    if ( present(mnull) ) rm = rm + mnull    ! Add null space-model to model-residual

    ! Iterations
    forget = F
    do i = 1, niter   
      stat1 = Fop( T, F, g_solver, rd)                             ! g   = L' r_d
      gm = rm*eps; stat2 =  Reg( T, T, g_solver, gm)               ! g   = L' r_d + eps A' r_m
      stat1 =  Fop( F, F, g_solver, gd)                            ! g_d = L g
      stat2 =  Reg( F, F, g_solver, gm); gm = gm*eps               ! g_m = eps A g
      stat  = stepper( forget, m, g_solver, rr_solver, gg_solver)  ! step in x and rr              
!      if (mod(i,10).eq.0) then
!        write(0,*) 'iteration ',i,' rdpow=',sum(rd*rd)
!        write(0,*) 'iteration ',i,' rmpow=',sum(rm*rm)
!      end if
    end do

    ! Conclusions
    if (present(r))  r = rr_solver

  end subroutine 




end module 
