module  stats_mod

  implicit none
  public  :: calcmean, calcmedian,sort
!  private :: sort

  contains

  real function calcmean(q) result(mean)
    real, intent(in)  :: q(:)
    integer           :: n
    n = size(q)
    if (n.gt.0) mean = real(sum(dble(q))/dble(n))
    if (n.eq.0) mean = 0.
  end function


  real function calcstddev(q) result(stddev)
    real, intent(in)  :: q(:)
    real              :: mean
    integer           :: n,ii
    n = size(q); stddev=0.
    mean = calcmean(q)
    do ii=1,n
      stddev = stddev + (dble(mean)-dble(q(ii)))**2.
    end do
    if (n.gt.0) stddev = real( sqrt( dble(stddev)/dble(n*(n-1)) ) )
    if (n.eq.0) stddev = 0.
  end function

  
  real function calcmedian(q) result(median)
    real, intent(inout) :: q(:)
    integer             :: n
    n = size(q); median=0.
    call sort(q)
    if (int(n/2+1).gt.0 .and. int(n/2).gt.0) then
      median = (q(n/2+1)+q(n/2))/2.
    else
      median =  q(n)
    end if
  end function
  
  
  real function calcquantile(q,k) result(quant)
    real, intent(inout) ::   q(:)
    real                ::     k
    integer             :: n
    n = size(q); quant=0.; k = k
    call sort(q)
    if (int(n*k/100.) .gt. 0 .and. int(n*k/100.+1) .gt. 0) then
      quant = (q(n*k/100.+1)+q(n*k/100.))/2.
    else
      quant =  q(n)
    end if
  end function
  
  
  subroutine sort(q)
    real                :: q(:)
    integer             :: i,n
    real                :: p(size(q))
    p = 0.
    n = size(q)
    do i=1,n
      p(                  i   )=       minval(q)
      q(int(minval(minloc(q))))=2.*abs(maxval(q))
    end do
    q = p
  end subroutine


  real function calcstdv(q) result(stdv)
    real, intent(in)  :: q(:)
    double precision  :: stdval, mean
    integer           :: n,i
    n = size(q);
    mean = 0.
    stdval = 0.
    mean=real(sum(dble(q))/dble(n))
    do i=1,n
      stdval = stdval + (dble(q(i))-mean)**2.
    end do
    stdv = real(sqrt( stdval/dble(n*(n-1))))
  end function


end module
