module tape_mod
  ! Single and multitaper, op's, lop's and fop's
  ! An interface to initialize the tapers in this module is located in tapers_mod

  use  adjnull_mod
  use  addnull_mod
  use fftshift_mod

  implicit none

  real, private,allocatable    :: taper1(:),taper2(:),taper3(:)
  integer,private              :: ii,jj,kk
  integer,private              :: n1,n2,n3

  interface tape_op
    module procedure tape1d_real_op
    module procedure tape2d_real_op
    module procedure tape3d_real_op
    module procedure tape1d_complex_op
    module procedure tape2d_complex_op
    module procedure tape3d_complex_op
  end interface


  interface tape_lop
    module procedure tape1d_lop
    module procedure tape2d_lop
    module procedure tape3d_lop
  end interface


  interface tape_fop
    module procedure tape1d_fop
    module procedure tape2d_fop
    module procedure tape3d_fop
  end interface

  interface tape_gettaper
    module procedure tape_gettaper_1d
    module procedure tape_gettaper_2d
    module procedure tape_gettaper_3d
  end interface


  contains


  subroutine tape_init             (n1_in,n2_in,n3_in)
    integer, intent(in),optional  :: n1_in,n2_in,n3_in
    if (present( n1_in)) then; n1  = n1_in;  else; n1 = 1;  end if
    if (present( n2_in)) then; n2  = n2_in;  else; n2 = 1;  end if
    if (present( n3_in)) then; n3  = n3_in;  else; n3 = 1;  end if

    allocate(  taper1(n1)   , taper2(n2)   , taper3(n3)    )
    
    taper1  = 1.
    taper2  = 1.
    taper3  = 1.

    
  end subroutine
  
  
  subroutine  tape_insert(add, taper_in,t1,t2,dir_in)
    real   , intent(inout)       :: taper_in(:)
    logical, intent(in)          :: add
    integer, intent(in),optional :: dir_in
    integer                      :: dir,t1,t2
    
    if ( present (dir_in) ) then; dir = dir_in; else; dir = 1; end if
   
    if (dir==1) then
      if (.not. size(taper_in) == n1) then; write(0,*) "size(taper_in)!=size(taper) in next call to taper_mod"; call exit(); end if      
      call addnull(add,taper1(t1:t2))
      do ii=t1,t2;
        taper1(ii) = taper1(ii) + taper_in(ii)
      end do
    end if
    if (dir==2) then
      if (.not. size(taper_in) == n2) then; write(0,*) "size(taper_in)!=size(taper) in next call to taper_mod"; call exit(); end if      
      call addnull(add,taper2(t1:t2))
      do ii=t1,t2;
        taper2(ii) = taper2(ii) + taper_in(ii)
      end do
    end if
    if (dir==3) then
      if (.not. size(taper_in) == n3) then; write(0,*) "size(taper_in)!=size(taper) in next call to taper_mod"; call exit(); end if      
      call addnull(add,taper3(t1:t2))
      do ii=t1,t2;
        taper3(ii) = taper3(ii) + taper_in(ii)
      end do
    end if
    
  end subroutine
  
  subroutine tape_shift(shift_in)
    logical,intent(in) ::shift_in(3)
    real,allocatable   :: dummy(:)

    if (shift_in(1)) then
      allocate(dummy(n1))
      call fftshift1d_1_lop(.false.,.false.,taper1,dummy)
      taper1=dummy
      deallocate(dummy)
    end if
    
    if (shift_in(2)) then
      allocate(dummy(n2))
      call fftshift1d_1_lop(.false.,.false.,taper2,dummy)
      taper2=dummy
      deallocate(dummy)
    end if
    
    if (shift_in(3)) then
      allocate(dummy(n3))
      call fftshift1d_1_lop(.false.,.false.,taper3,dummy)
      taper3=dummy
      deallocate(dummy)
    end if

  end subroutine

  
  subroutine tape_close()

    if (allocated( taper1)) then; deallocate( taper1); end if
    if (allocated( taper2)) then; deallocate( taper2); end if
    if (allocated( taper3)) then; deallocate( taper3); end if

  end subroutine
  

  subroutine tape_gettaper_1d(taper_out)
    real,intent(inout)             :: taper_out(:)
    
    do ii=1,n1
      taper_out(ii) = taper1(ii)
    end do

  end subroutine

  subroutine tape_gettaper_2d(taper_out)
    real,intent(inout)             :: taper_out(:,:)

    do jj=1,n2
    do ii=1,n1
      taper_out(ii,jj) = taper1(ii)*taper2(jj)
    end do
    end do

  end subroutine
  
  subroutine tape_gettaper_3d(taper_out)
    real,intent(inout)             :: taper_out(:,:,:)

    do kk=1,n3
    do jj=1,n2
    do ii=1,n1
      taper_out(ii,jj,kk) = taper1(ii)*taper2(jj)*taper3(kk)
    end do
    end do
    end do

  end subroutine


  subroutine tape1d_real_op   (x)
    real, intent(inout) :: x(:)

    do ii=1,n1
      x(ii) = taper1(ii)*x(ii)
    end do

  end subroutine
  
  
  subroutine tape2d_real_op   (x)
    real, intent(inout) :: x(:,:)

    do jj=1,n2
    do ii=1,n1
      x(ii,jj) = taper1(ii)*taper2(jj)*x(ii,jj)
    end do
    end do

  end subroutine


  subroutine tape3d_real_op   (x)
    real, intent(inout) :: x(:,:,:)
    
    do kk=1,n3
    do jj=1,n2
    do ii=1,n1
        x(ii,jj,kk) = taper1(ii)*taper2(jj)*taper3(kk)*x(ii,jj,kk)
    end do
    end do
    end do
    
  end subroutine


  subroutine tape1d_complex_op   (x)
    complex, intent(inout) :: x(:)

    do ii=1,n1
      x(ii) = taper1(ii)*x(ii)
    end do

  end subroutine
  
  
  subroutine tape2d_complex_op   (x)
    complex, intent(inout) :: x(:,:)

    do jj=1,n2
    do ii=1,n1
      x(ii,jj) = taper1(ii)*taper2(jj)*x(ii,jj)
    end do
    end do

  end subroutine


  subroutine tape3d_complex_op   (x)
    complex, intent(inout) :: x(:,:,:)
    
    do kk=1,n3
    do jj=1,n2
    do ii=1,n1
        x(ii,jj,kk) = taper1(ii)*taper2(jj)*taper3(kk)*x(ii,jj,kk)
    end do
    end do
    end do

    !write(0,*) "taper1",taper1
    !write(0,*) "taper2",taper2
    !write(0,*) "taper3",taper3
    
  end subroutine

  subroutine tape1d_lop  (adj,add,x,y)
    logical,intent(in)  :: adj,add
    real, intent(inout) ::         x(:),y(:)

    call adjnull          (adj,add,x,y)
 
    if (adj) then
      do ii=1,n1
        x(ii) = x(ii) + taper1(ii)*y(ii)
      end do
    else
      do ii=1,n1
        y(ii) = y(ii) + taper1(ii)*x(ii)
      end do
    end if

  end subroutine

  subroutine tape2d_lop  (adj,add,x,y)
    logical,intent(in)  :: adj,add
    real, intent(inout) ::         x(:,:),y(:,:)

    call adjnull          (adj,add,x,y)

    if (adj) then
      do jj=1,n2
      do ii=1,n1
        x(ii,jj) = x(ii,jj) + taper1(ii)*taper2(jj)*y(ii,jj)
      end do
      end do
    else
      do jj=1,n2
      do ii=1,n1
        y(ii,jj) = y(ii,jj) + taper1(ii)*taper2(jj)*x(ii,jj)
      end do
      end do
    end if

  end subroutine


  subroutine tape3d_lop  (adj,add,x,y)
    logical, intent(in) :: adj,add
    real, intent(inout) ::         x(:,:,:),y(:,:,:)

    call adjnull          (adj,add,x,y)

    if (adj) then
      do kk=1,n3
      do jj=1,n2
      do ii=1,n1
        x(ii,jj,kk) = x(ii,jj,kk) + taper1(ii)*taper2(jj)*taper3(kk)*y(ii,jj,kk)
      end do
      end do
      end do
    else
      do kk=1,n3
      do jj=1,n2
      do ii=1,n1
        y(ii,jj,kk) = y(ii,jj,kk) + taper1(ii)*taper2(jj)*taper3(kk)*x(ii,jj,kk)
      end do
      end do
      end do
    end if

  end subroutine
  
  
  subroutine tape1d_fop    (inv,add,x,y)
    logical,intent(in)  :: inv,add
    real, intent(inout) ::         x(:),y(:)
    
    call adjnull          (inv,add,x,y)
    
    if (inv) then
      do ii=1,n1
        x(ii) = x(ii) + y(ii)*taper1(ii)
      end do
    else
      do ii=1,n1
        y(ii) = y(ii) + x(ii)/taper1(ii)
      end do
    end if

  end subroutine
  
  
  subroutine tape2d_fop    (inv,add,x,y)
    logical,intent(in)  ::   inv,add
    real, intent(inout) ::           x(:,:),y(:,:)
    
    call adjnull          (inv,add,x,y)

    if (inv) then
      do jj=1,n2
      do ii=1,n1
        x(ii,jj) = x(ii,jj) + y(ii,jj)* taper1(ii)*taper2(jj)
      end do
      end do
    else
      do jj=1,n2
      do ii=1,n1
        y(ii,jj) = y(ii,jj) + x(ii,jj)/(taper1(ii)*taper2(jj))
      end do
      end do
    end if
    
  end subroutine
  
  
  subroutine tape3d_fop    (inv,add,x,y)
    logical, intent(in) :: inv,add
    real, intent(inout) ::         x(:,:,:),y(:,:,:)
    
    call adjnull          (inv,add,x,y)
    
    if (inv) then
      do kk=1,n3
      do jj=1,n2
      do ii=1,n1
        x(ii,jj,kk) = x(ii,jj,kk) + y(ii,jj,kk)* taper1(ii)*taper2(jj)*taper3(kk)
      end do
      end do
      end do
    else
      do kk=1,n3
      do jj=1,n2
      do ii=1,n1
        y(ii,jj,kk) = y(ii,jj,kk) + x(ii,jj,kk)/(taper1(ii)*taper2(jj)*taper3(kk))
      end do
      end do
      end do
    end if
    
  end subroutine


end module

