module spectra_mod
  ! Correlation, Coherence and Deconvolution

  implicit none

  interface spectra_ampl
    module procedure spectra_ampl_1d
    module procedure spectra_ampl_2d
    module procedure spectra_ampl_3d
  end interface

  interface spectra_powr
    module procedure spectra_powr_1d
    module procedure spectra_powr_2d
    module procedure spectra_powr_3d
  end interface

  interface spectra_crsp
    module procedure spectra_crsp_1d_1d
    module procedure spectra_crsp_1d_2d
    module procedure spectra_crsp_1d_3d
    module procedure spectra_crsp_2d_1d
    module procedure spectra_crsp_3d_1d
    module procedure spectra_crsp_2d_2d
    module procedure spectra_crsp_3d_3d
  end interface


  interface spectra_cohr
    module procedure spectra_cohr_1d_1d
    module procedure spectra_cohr_1d_2d
    module procedure spectra_cohr_1d_3d
    module procedure spectra_cohr_2d_1d
    module procedure spectra_cohr_3d_1d
    module procedure spectra_cohr_2d_2d
    module procedure spectra_cohr_3d_3d
  end interface


  interface spectra_decv
    module procedure spectra_decv_1d_1d
    module procedure spectra_decv_1d_2d
    module procedure spectra_decv_1d_3d
    module procedure spectra_decv_2d_1d
    module procedure spectra_decv_3d_1d
    module procedure spectra_decv_2d_2d
    module procedure spectra_decv_3d_3d
  end interface

  contains

  subroutine spectra_init()
    call spectra_close()
  end subroutine

  subroutine spectra_close()
  end subroutine

  subroutine spectra_ampl_1d(a,c)
    complex            :: a(:)
    real               :: c(:)
    c = abs(a)
  end subroutine

  subroutine spectra_ampl_2d(a,c)
    complex            :: a(:,:)
    real               :: c(:,:)
    c = abs(a)
  end subroutine

  subroutine spectra_ampl_3d(a,c)
    complex            :: a(:,:,:)
    real               :: c(:,:,:)
    c = abs(a)
  end subroutine


  subroutine spectra_powr_1d(a,c)
    complex            :: a(:)
    real               :: c(:)
    c = a*conjg(a)
  end subroutine

  subroutine spectra_powr_2d(a,c)
    complex            :: a(:,:)
    real               :: c(:,:)
    c = a*conjg(a)
  end subroutine

  subroutine spectra_powr_3d(a,c)
    complex            :: a(:,:,:)
    real               :: c(:,:,:)
    c = a*conjg(a)
  end subroutine

  subroutine spectra_crsp_1d_1d(a,b,c)
    complex            :: a(:),b(:),c(:)
    integer            :: n1,ii
    
    n1 = size(c,1)
    do ii=1,n1
      c(ii      ) = a(ii      )*conjg(b(ii      ))
    end do

  end subroutine

  subroutine spectra_crsp_2d_1d(a,b,c)
    complex            :: a(:,:),b(:),c(:,:)
    integer            :: n1,ii,n2,jj
    
    n1 = size(c,1); n2 = size(c,2)
    do jj=1,n2;    do ii=1,n1
        c(ii,jj   ) = a(ii,jj   )*conjg(b(ii   ))
    end do;        end do
    
  end subroutine

  subroutine spectra_crsp_3d_1d(a,b,c)
    complex            :: a(:,:,:),b(:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = a(ii,jj,kk)*conjg(b(ii   ))
    end do;         end do;         end do

  end subroutine


  subroutine spectra_crsp_1d_2d(a,b,c)
    complex            :: a(:),b(:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = a(ii      )*conjg(b(ii,jj   ))
    end do;         end do

  end subroutine


  subroutine spectra_crsp_1d_3d(a,b,c)
    complex            :: a(:),b(:,:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = a(ii      )*conjg(b(ii,jj,kk))
    end do;       end do;           end do

  end subroutine


  subroutine spectra_crsp_2d_2d(a,b,c)
    complex            :: a(:,:),b(:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = a(ii,jj   )*conjg(b(ii,jj   ))
    end do;         end do

  end subroutine
  

  subroutine spectra_crsp_3d_3d(a,b,c)
    complex            :: a(:,:,:),b(:,:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = a(ii,jj,kk)*conjg(b(ii,jj,kk))
    end do;         end do;         end do

  end subroutine



















  subroutine spectra_cohr_1d_1d(a,b,c)
    complex            :: a(:),b(:),c(:)
    integer            :: n1,ii
    
    n1 = size(c,1)
    do ii=1,n1
      c(ii      ) = a(ii      )*conjg(b(ii      )) / ( abs(a(ii      )) * abs(b(ii      )) )
    end do

  end subroutine

  subroutine spectra_cohr_2d_1d(a,b,c)
    complex            :: a(:,:),b(:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = a(ii,jj   )*conjg(b(ii       )) / ( abs(a(ii,jj   )) * abs(b(ii      )) )
    end do;         end do

  end subroutine

  subroutine spectra_cohr_3d_1d(a,b,c)
    complex            :: a(:,:,:),b(:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = a(ii,jj,kk)*conjg(b(ii      )) / ( abs(a(ii,jj,kk)) * abs(b(ii      )) )
    end do;         end do;         end do

  end subroutine


  subroutine spectra_cohr_1d_2d(a,b,c)
    complex            :: a(:),b(:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = a(ii      )*conjg(b(ii,jj   )) / ( abs(a(ii      )) * abs(b(ii,jj   )) )
    end do;         end do

  end subroutine


  subroutine spectra_cohr_1d_3d(a,b,c)
    complex            :: a(:),b(:,:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = a(ii      )*conjg(b(ii,jj,kk)) / ( abs(a(ii      )) * abs(b(ii,jj,kk)) )
    end do;       end do;           end do

  end subroutine


  subroutine spectra_cohr_2d_2d(a,b,c)
    complex            :: a(:,:),b(:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = a(ii,jj   )*conjg(b(ii,jj   )) / ( abs(a(ii,jj   )) * abs(b(ii,jj   )) )
    end do;         end do

  end subroutine


  subroutine spectra_cohr_3d_3d(a,b,c)
    complex            :: a(:,:,:),b(:,:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = a(ii,jj,kk)*conjg(b(ii,jj,kk)) / ( abs(a(ii,jj,kk)) * abs(b(ii,jj,kk)) )
    end do;         end do;         end do

  end subroutine



















  subroutine spectra_decv_1d_1d(a,b,c)
    complex            :: a(:),b(:),c(:)
    integer            :: n1,ii
    
    n1 = size(c,1)
    do ii=1,n1
      c(ii      ) = a(ii      )*conjg(b(ii      )) / ( abs(b(ii      )) * abs(b(ii      )) )
    end do

  end subroutine

  subroutine spectra_decv_2d_1d(a,b,c)
    complex            :: a(:,:),b(:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = a(ii,jj   )*conjg(b(ii      )) / ( abs(b(ii      )) * abs(b(ii      )) )
    end do;         end do

  end subroutine

  subroutine spectra_decv_3d_1d(a,b,c)
    complex            :: a(:,:,:),b(:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = a(ii,jj,kk)*conjg(b(ii      )) / ( abs(b(ii      )) * abs(b(ii      )) )
    end do;         end do;         end do

  end subroutine


  subroutine spectra_decv_1d_2d(a,b,c)
    complex            :: a(:),b(:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = a(ii      )*conjg(b(ii,jj   )) / ( abs(b(ii,jj   )) * abs(b(ii,jj   )) )
    end do;         end do

  end subroutine


  subroutine spectra_decv_1d_3d(a,b,c)
    complex            :: a(:),b(:,:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = a(ii      )*conjg(b(ii,jj,kk)) / ( abs(b(ii,jj,kk)) * abs(b(ii,jj,kk)) )
    end do;       end do;           end do

  end subroutine


  subroutine spectra_decv_2d_2d(a,b,c)
    complex            :: a(:,:),b(:,:),c(:,:)
    integer            :: n1,ii,n2,jj

    n1 = size(c,1); n2 = size(c,2)
    do jj=1,n2;     do ii=1,n1
      c(ii,jj   ) = a(ii,jj   )*conjg(b(ii,jj   )) / ( abs(b(ii,jj   )) * abs(b(ii,jj   )) )
    end do;         end do

  end subroutine


  subroutine spectra_decv_3d_3d(a,b,c)
    complex            :: a(:,:,:),b(:,:,:),c(:,:,:)
    integer            :: n1,ii,n2,jj,n3,kk

    n1 = size(c,1); n2 = size(c,2); n3 = size(c,3)
    do kk=1,n3;     do jj=1,n2;     do ii=1,n1
      c(ii,jj,kk) = a(ii,jj,kk)*conjg(b(ii,jj,kk)) / ( abs(b(ii,jj,kk)) * abs(b(ii,jj,kk)) )
    end do;         end do;         end do

  end subroutine





end module
