module tp_trans_mod_omp
  
  use adjnull_mod
  
  implicit none
  
  real   , private      :: dt2d,dx2d,dtau2d,dp,ot2d,ox2d,otau2d,op
  integer, private      :: nt2d,nx2d,ntau2d,np
  real,allocatable      :: t2d(:),x2d(:),tau2d(:),p(:)
  integer,allocatable   :: fold2d(:,:)
  
  real   , private      :: dt3d,dx3d,dy3d,dtau3d,dpx,dpy,ot3d,ox3d,oy3d,otau3d,opx,opy
  integer, private      :: nt3d,nx3d,ny3d,ntau3d,npx,npy
  real,allocatable      :: t3d(:),x3d(:),y3d(:),tau3d(:),px(:),py(:)
  integer,allocatable   :: fold3d(:,:,:)

  include "openmp.inc"
  
  contains
  
  
  subroutine tp_trans_2d_init(nt_in,dt_in,ot_in,nx_in,dx_in,ox_in,ntau_in,dtau_in,otau_in,np_in,dp_in,op_in)
    real                    ::      dt_in,ot_in,      dx_in,ox_in,        dtau_in,otau_in,      dp_in,op_in
    integer                 :: nt_in,            nx_in,            ntau_in,                np_in
    integer                 :: it,ix,itau,ip
    
    call tp_trans_2d_close()
    include "openmp_init.inc"

    ! Get axis parameters
      nt2d = nt_in;        dt2d =   dt_in;      ot2d =   ot_in
      nx2d = nx_in;        dx2d =   dx_in;      ox2d =   ox_in
    ntau2d = ntau_in;    dtau2d = dtau_in;    otau2d = otau_in      
      np   = np_in;        dp   =   dp_in;      op   =   op_in  
    
    ! Allocate axis memort variables
    allocate(   t2d(nt2d)   )
    allocate(   x2d(nx2d)     )
    allocate( tau2d(ntau2d) )
    allocate(     p(np)     )
    
    ! Calculate Axes
    do   it=1,nt2d;     t2d(it)  =(  it-1)*dt2d  +ot2d;   end do;
    do   ix=1,nx2d;     x2d(ix)  =(  ix-1)*dx2d  +ox2d;     end do;
    do itau=1,ntau2d; tau2d(itau)=(itau-1)*dtau2d+otau2d; end do;
    do   ip=1,np;         p(ip)  =(  ip-1)*dp    +op;     end do;
    
    ! Prepare fold
    allocate(fold2d(ntau2d,np))
    fold2d = 0
    ! Prepare Fold
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(fold2d,t2d,p,x2d,dtau2d,np,nt2d,nx2d,ntau2d)
    do ip = 1,np
      do it = 1,nt2d
        do ix = 1,nx2d
          itau = ( t2d(it) - p(ip)*x2d(ix) ) / dtau2d + .5
          if (itau.ge.1 .and. itau.le.ntau2d) then
            fold2d(itau,ip) = fold2d(itau,ip) + 1
          end if
    end do;  end do;  end do
    !$OMP END PARALLEL DO
    
  end subroutine
  
  
  subroutine tp_trans_2d_close()
    if (allocated(t2d))   then;  deallocate(t2d);    end if;
    if (allocated(x2d))   then;  deallocate(x2d);    end if;
    if (allocated(tau2d)) then;  deallocate(tau2d);  end if;
    if (allocated(p))     then;  deallocate(p);      end if;
    if (allocated(fold2d))then;  deallocate(fold2d); end if;
  end subroutine
  
  
  
  subroutine tp_trans_2d_fop(inv,add,m,d)
    ! model is xt domain, data is tp domain
    logical,intent(in)   ::  inv,add
    real                 ::          m(:,:),d(:,:)
    integer              ::  it,ix,itau,ip

    call adjnull(inv,add,m,d)

    ! Perform Transform by pushing
    if (inv) then
      !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(m,d,fold2d,t2d,x2d,p,dtau2d,np,nt2d,nx2d,ntau2d)
      do ip = 1,np
        do it = 1,nt2d
          do ix = 1,nx2d
            itau = ( t2d(it) - p(ip)*x2d(ix) ) / dtau2d + .5
            if (itau.ge.1 .and. itau.le.ntau2d) then
              m(it  ,ix) = m(it  ,ix) + d(itau,ip)*fold2d(itau,ip)
            end if
      end do;  end do;  end do
      !$OMP END PARALLEL DO
    else;
      !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(m,d,fold2d,t2d,x2d,p,dtau2d,np,nt2d,nx2d,ntau2d)
      do ip = 1,np
        do it = 1,nt2d
          do ix = 1,nx2d
            itau = ( t2d(it) - p(ip)*x2d(ix) ) / dtau2d + .5
            if (itau.ge.1 .and. itau.le.ntau2d) then
              d(itau,ip) = d(itau,ip) + m(it  ,ix)/fold2d(itau,ip)
            end if
      end do;  end do;  end do 
      !$OMP END PARALLEL DO
    end if

  end subroutine



  subroutine tp_trans_3d_init(nt_in,dt_in,ot_in,nx_in,dx_in,ox_in,ny_in,dy_in,oy_in,ntau_in,dtau_in,otau_in,npx_in,dpx_in,opx_in,npy_in,dpy_in,opy_in)
    real                   ::       dt_in,ot_in,      dx_in,ox_in,      dy_in,oy_in,        dtau_in,otau_in,       dpx_in,opx_in,       dpy_in,opy_in
    integer                :: nt_in,            nx_in,            ny_in,            ntau_in,                npx_in,              npy_in
    integer                :: it,ix,iy,itau,ipx,ipy
    
    call tp_trans_3d_close()
    include "openmp_init.inc"

    ! Get axis parameters
      nt3d = nt_in;        dt3d =   dt_in;      ot3d =   ot_in
      nx3d = nx_in;        dx3d =   dx_in;      ox3d =   ox_in
      ny3d = ny_in;        dy3d =   dy_in;      oy3d =   oy_in
    ntau3d = ntau_in;    dtau3d = dtau_in;    otau3d = otau_in      
      npx  = npx_in;       dpx  =  dpx_in;      opx  =  opx_in  
      npy  = npy_in;       dpy  =  dpy_in;      opy  =  opy_in  
    
    ! Allocate axis memort variables
    allocate(  t3d(nt3d) )
    allocate(  x3d(nx3d) )
    allocate(  y3d(ny3d) )
    allocate( tau3d(ntau3d) )
    allocate(   px(npx) )
    allocate(   py(npy) )
    
    ! Calculate Axes
    do   it=1,nt3d;   t3d(it)    =(  it-1)*dt3d  +ot3d;   end do;
    do   ix=1,nx3d;   x3d(ix)    =(  ix-1)*dx3d  +ox3d;   end do;
    do   iy=1,ny3d;   y3d(iy)    =(  iy-1)*dy3d  +oy3d;   end do;
    do itau=1,ntau3d; tau3d(itau)=(itau-1)*dtau3d+otau3d; end do;
    do  ipx=1,npx;     px(ipx)   =( ipx-1)*dpx   +opx;    end do;
    do  ipy=1,npy;     py(ipy)   =( ipy-1)*dpy   +opy;    end do;

    ! Prepare fold
    allocate(fold3d(ntau3d,npx,npy))
    fold3d = 0
    ! Prepare Fold
    !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(fold3d,t3d,px,py,x3d,y3d,dtau3d,npx,npy,nt3d,nx3d,ny3d,ntau3d)
    do ipy = 1,npy
    do ipx = 1,npx
      do it = 1,nt3d
        do iy = 1,ny3d
        do ix = 1,nx3d
          itau = ( t3d(it) - px(ipx)*x3d(ix) - py(ipy)*y3d(iy) ) / dtau3d + .5
          if (itau.ge.1 .and. itau.le.ntau3d) then
            fold3d(itau,ipx,ipy) = fold3d(itau,ipx,ipy) + 1
          end if
    end do;  end do;  end do;  end do;  end do
    !$OMP END PARALLEL DO
    
  end subroutine


  subroutine tp_trans_3d_close()
    if (allocated(t3d))   then;  deallocate(t3d);    end if;
    if (allocated(x3d))   then;  deallocate(x3d);    end if;
    if (allocated(y3d))   then;  deallocate(y3d);    end if;
    if (allocated(tau3d)) then;  deallocate(tau3d);  end if;
    if (allocated(px))    then;  deallocate(px);     end if;
    if (allocated(py))    then;  deallocate(py);     end if;
    if (allocated(fold3d))then;  deallocate(fold3d); end if;
  end subroutine



  subroutine tp_trans_3d_fop(inv,add,m,d)
    ! model is xt domain, data is tp domain
    logical,intent(in)   ::  inv,add
    real                 ::          m(:,:,:),d(:,:,:)
    integer              ::  it,ix,iy,itau,ipx,ipy

    call adjnull(inv,add,m,d)

    ! Perform Transform by pushing
    if (inv) then
      !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(m,d,fold3d,t3d,x3d,y3d,px,py,dtau3d,npx,npy,nt3d,nx3d,ny3d,ntau3d)
      do ipy = 1,npy
      do ipx = 1,npx
        do it = 1,nt3d
          do iy = 1,ny3d
          do ix = 1,nx3d
            itau = ( t3d(it) - px(ipx)*x3d(ix) - py(ipy)*y3d(iy) ) / dtau3d + .5
            if (itau.ge.1 .and. itau.le.ntau3d) then
              m(it,ix,iy) = m(it,ix,iy) + d(itau,ipx,ipy)*fold3d(itau,ipx,ipy)
            end if
      end do;  end do;  end do;  end do;  end do 
      !$OMP END PARALLEL DO
    else;
      !$OMP PARALLEL DO SCHEDULE(DYNAMIC) DEFAULT(PRIVATE) SHARED(m,d,fold3d,t3d,x3d,y3d,px,py,dtau3d,npx,npy,nt3d,nx3d,ny3d,ntau3d)
      do ipy = 1,npy
      do ipx = 1,npx
        do it = 1,nt3d
          do iy = 1,ny3d
          do ix = 1,nx3d
            itau = ( t3d(it) - px(ipx)*x3d(ix) - py(ipy)*y3d(iy) ) / dtau3d + .5
            if (itau.ge.1 .and. itau.le.ntau3d) then
              d(itau,ipx,ipy) = d(itau,ipx,ipy) + m(it,ix,iy)*fold3d(itau,ipx,ipy)
            end if
      end do;  end do;  end do;  end do;  end do 
      !$OMP END PARALLEL DO
    end if

  end subroutine

end module
